.dojoxRangeSlider .dijitSliderLeftBumper, .dojoxRangeSlider .dijitSliderBottomBumper  {
    background:#FFFFFF !important;
}

.dojoxRangeSliderBarContainer {
    position:relative;
}

.dojoxRangeSlider .dijitSliderProgressBarV {
    position:relative !important;
}

.dojoxRangeSlider .dijitSliderProgressBar {
    overflow:hidden;
    cursor:pointer;
    /* background:yellow !important; */
}

.dojoxRangeSlider .dijitSliderProgressBarV {
    position:absolute !important;
    border-width: 0px;
}

.dojoxRangeSlider .dijitSliderImageHandleH {
	left: -50%;
}
.dijitSlider .dojoxRangeSliderBarContainer .dijitSliderProgressBarH,
.dojoxRangeSlider .dijitSliderMoveableH {
        right:auto !important;
}
