<div class="dojoxDialog" tabindex="-1" role="dialog" aria-labelledby="${id}_title">
	<div dojoAttachPoint="titleBar" class="dojoxDialogTitleBar">
		<span dojoAttachPoint="titleNode" class="dojoxDialogTitle" id="${id}_title">${title}</span>
	</div>
	<div dojoAttachPoint="dojoxDialogWrapper">
		<div dojoAttachPoint="containerNode" class="dojoxDialogPaneContent"></div>
	</div>
	<div dojoAttachPoint="closeButtonNode" class="dojoxDialogCloseIcon" dojoAttachEvent="onclick: onCancel">
			<span dojoAttachPoint="closeText" class="closeText">x</span>
	</div>
</div>
