({"buttonSet":"設定","insertTableTitle":"テーブルの挿入","insertTableRowAfterLabel":"後ろに行を追加","buttonCancel":"キャンセル","center":"中央","deleteTableColumnLabel":"列の削除","right":"右","insertTableColumnBeforeLabel":"前に列を追加","tableWidth":"テーブル幅:","buttonInsert":"挿入","default":"デフォルト","align":"位置合わせ:","insertTableRowBeforeLabel":"前に行を追加","cellSpacing":"セル間隔:","pixels":"ピクセル","selectTableLabel":"テーブルの選択","rows":"行:","modifyTableTitle":"テーブルの変更","cellPadding":"セル余白:","deleteTableRowLabel":"行の削除","backgroundColor":"背景色:","insertTableColumnAfterLabel":"後ろに列を追加","left":"左","borderThickness":"ボーダーの太さ","columns":"列:","percent":"パーセント","borderColor":"ボーダーの色:"})