.dojoxCalcLayout span.dijitButtonNode,
.dojoxCalcLayout span.dijitButtonContents{
	display:block;
}
.dojoxCalc .dojoxCalcLayout .dojoxCalcTextAreaContainer {
	padding-left: 0.2em;
	padding-right: 0.2em;
	#border-right: .3em;
}
.dojoxCalcLayout .dojoxCalcInputContainer .dijitTextBox {
	/* IE bug workaround where the input caret is not visible (extends into the table border) */
	position: relative;
	border-left: 0 none;
	border-right: 0 none;
	width:100%;
}
.dojoxCalcLayout .dojoxCalcInputContainer,
.dojoxCalcLayout .dojoxCalcInputContainer .dijitInputField {
	padding-left: 0;
	padding-right: 0;
}
.dojoxCalcLayout .dojoxCalcInputContainer .dijitInputContainer {
	padding-left: 0;
	#padding-left: 0.5em;
	padding-right: 0.3em;
	#padding-right: 0;
}
.dojoxCalcLayout .dojoxCalcInputContainer .dijitInputInner {
	text-align: left;
	padding-left: 0.2em !important;
	padding-right: 0 !important;
	#padding-right: 0.3em !important; /* IE workaround to make sure the input caret is visible */
	overflow: hidden;
}
.dojoxCalcMinusButtonContainer {
	width: 4em;
	min-width: 4em;
	padding: 0;
}
.dojoxCalcMinusButtonContainer .dijitButton {
	margin: 0.1em;
	width: 3.8em;
}
.dojoxCalcMinusButtonContainer .dijitArrowButtonInner {
	width:1.3em;
}
.dojoxCalcMinusButtonContainer .dijitButtonNode .dijitButtonContents {
	width: 2.1em;
	min-width: 2.1em;
}
.dojoxCalcLayout .dojoxCalcMinusButtonContainer .dijitButtonText,
.dojoxCalcLayout .dojoxCalcMinusButtonContainer .dijitButtonNode {
	padding-left: 0px;
	padding-right: 0px;
}

.dojoxCalcButtonContainer {
	width: 4em;
	min-width: 4em;
	padding: 0;
}
.dojoxCalcButtonContainer .dijitButton {
	margin: 0.1em;
	width: 3.8em;
}

.dojoxCalcLayout .dojoxCalcButtonContainer .dijitButtonText,
.dojoxCalcLayout .dojoxCalcButtonContainer .dijitButtonNode {
	padding-left: 0px;
	padding-right: 0px;
}
.dojoxCalcLayout {
	table-layout:fixed;
	border-width: 0;
	border-style: none;
	width:16.0em;
	font:monospace;
}
.dojoxCalc {
	border: 0.4em ridge #909090;
}

.dojoxCalcGrapherLayout span.dijitButtonNode,
.dojoxCalcGrapherLayout span.dijitButtonContents{
	display:block;
}
.dojoxCalcGrapherButtonContainer {
	width: 10em;
	min-width: 10em;
	padding: 0;
}
.dojoxCalcGrapherButton {
	display:block;
}
.dojoxCalcGrapherLayout {
	table-layout:fixed;
	border-width: 0;
	border-style: none;
	width:20.0em;
	font:monospace;
}
.dojoxCalcExpressionBox {
	width:15em;
}
.dojoxCalcChartHolder {
	position:absolute;
	left:36em;
	top:5em;
}
.dojoxCalcGraphOptionTable {
	width:25em;
}
.dojoxCalcGraphWidth {
	width:5em;
}
.dojoxCalcGraphHeight {
	width:5em;
}
.dojoxCalcGraphMinX {
	width:5em;
}
.dojoxCalcGraphMaxX {
	width:5em;
}
.dojoxCalcGraphMinY {
	width:5em;
}
.dojoxCalcGraphMaxY {
	width:5em;
}
.dojoxCalcGrapherFuncOuterDiv {
	width:35em;
	height:15em;
	overflow-y:scroll;
}
.dojoxCalcGrapherModeTable {
	width:10em;
}
.dojoxCalcFunctionModeSelector {
	width:5em;
}
.dojoxCalcStatusBox {
	border:1px solid;
	padding:1px;
	display:inline;
}
