/* tundra doesn't appear to need any special css rules */

.claro .dojoxTitleGroup .dijitTitlePaneContentOuter {
    border-bottom:none;
}

.claro .dojoxTitleGroup .dijitTitlePaneContentOuter .dijitTitlePaneContentOuter,
.claro .dojoxTitleGroup {
    border-bottom:1px solid #B5BCC7;
}

.soria .dojoxTitleGroup .dijitTitlePaneContentOuter {
    border-bottom:none;
}

.soria .dojoxTitleGroup .dijitTitlePaneContentOuter .dijitTitlePaneContentOuter,
.soria .dojoxTitleGroup {
    border-bottom:1px solid #BFBFBF;
}

.nihilo .dojoxTitleGroup .dijitTitlePaneContentOuter {
    border-bottom:none;
}

.nihilo .dojoxTitleGroup .dijitTitlePaneContentOuter .dijitTitlePaneContentOuter,
.nihilo .dojoxTitleGroup {
    border-bottom:1px solid #BFBFBF;
}
