({"redLabel":"č","valueLabel":"j","hueLabel":"o","saturationLabel":"n","saturationPickerTitle":"Selektor sytosti","huePickerTitle":"Selektor odstínu","greenLabel":"z","blueLabel":"m","hexLabel":"hex","degLabel":"°"})