({"saveLabel":"Сохранить","saveSettingdialogTitle":"Автосохранение","saveSettingdialogButtonOk":"Задать интервал","saveMessageSuccess":"Сохранено ${0}","saveSettingdialogButtonCancel":"Отменить","saveSettingdialogDescription":"Укажите интервал автосохранения","saveSettingLabelOn":"Задать интервал автосохранения...","saveSettingdialogParamLabel":"мин","saveSettingdialogParamName":"Интервал автосохранения","saveMessageFail":"Не удалось сохранить ${0}","saveSettingLabelOff":"Выключить автосохранение"})