({"replaceDialogText":"${0} vervangingen.","eofDialogTextFind":"gevonden","eofDialogText":"Laatste positie ${0}","backwards":"Terug","replaceButton":"Vervangen","replaceLabel":"Vervangen door:","matchCase":"Hoofdlettergevoelig","findTooltip":"Geef de zoektekst op","replaceTooltip":"Geef de vervangende tekst op","replaceAllButton":"Alle vervangen","eofDialogTextReplace":"vervangen","findReplace":"Zoeken en vervangen","backwardsTooltip":"Terugwaarts naar tekst zoeken","replaceAllButtonTooltip":"Gehele tekst vervangen","replaceButtonTooltip":"Tekst vervangen","findLabel":"Zoeken:","findButton":"Zoeken:","matchCaseTooltip":"Hoofdlettergevoelig","findButtonTooltip":"Tekst zoeken","replaceAll":"Alle posities"})