({"replaceDialogText":"Počet nahrazených výskytů: ${0}.","eofDialogTextFind":"byl nalezen","eofDialogText":"Poslední výskyt ${0}","backwards":"V opačném směru","replaceButton":"Nahradit","replaceLabel":"Nahrazující řetězec:","matchCase":"S rozlišením velkých a malých písmen","findTooltip":"Zadejte hledaný text.","replaceTooltip":"Zadejte text pro nahrazení.","replaceAllButton":"Nahradit vše","eofDialogTextReplace":"byl nahrazen","findReplace":"Najít a nahradit","backwardsTooltip":"Hledat text v opačném směru","replaceAllButtonTooltip":"Nahradit všechen text","replaceButtonTooltip":"Nahradit text","findLabel":"Najít:","findButton":"Najít","matchCaseTooltip":"S rozlišením velkých a malých písmen","findButtonTooltip":"Najít text","replaceAll":"Všechny výskyty"})