({"saveLabel":"Spara","saveSettingdialogTitle":"Spara automatiskt","saveSettingdialogButtonOk":"Ange intervall","saveMessageSuccess":"Sparades ${0}","saveSettingdialogButtonCancel":"Avbryt","saveSettingdialogDescription":"Ange intervall för automatiskt sparande","saveSettingLabelOn":"Ange intervall för automatiskt sparande...","saveSettingdialogParamLabel":"min.","saveSettingdialogParamName":"Intervall för automatiskt sparande","saveMessageFail":"Kunde inte sparas ${0}","saveSettingLabelOff":"Avaktivera automatiskt sparande"})