({"close":"[închidere]","title":"Titlu","authors":"Autori","summary":"Sumar","content":"Conţinut","contributors":"Contribuitori","updated":"Actualizat","displayOptions":"[opţiuni afişare]","id":"ID"})