<div class="dijitDialog" tabindex="-1" role="dialog" aria-labelledby="${id}_title">
	<div dojoAttachPoint="titleBar" class="dijitDialogTitleBar">
	<span dojoAttachPoint="titleNode" class="dijitDialogTitle" id="${id}_title">${insertTableTitle}</span>
	<span dojoAttachPoint="closeButtonNode" class="dijitDialogCloseIcon" dojoAttachEvent="onclick: onCancel" title="${buttonCancel}">
		<span dojoAttachPoint="closeText" class="closeText" title="${buttonCancel}">x</span>
	</span>
	</div>
    <div dojoAttachPoint="containerNode" class="dijitDialogPaneContent">
        <table class="etdTable"><tr>
            <td>
                <label>${rows}</label>
			</td><td>
                <span dojoAttachPoint="selectRow" dojoType="dijit.form.TextBox" value="2"></span>
            </td><td><table><tr><td class="inner">
                <label>${columns}</label>
			</td><td class="inner">
                <span dojoAttachPoint="selectCol" dojoType="dijit.form.TextBox" value="2"></span>
            </td></tr></table></td></tr>		
			<tr><td>
                <label>${tableWidth}</label>
            </td><td>
                <span dojoAttachPoint="selectWidth" dojoType="dijit.form.TextBox" value="100"></span>
			</td><td>
                <select dojoAttachPoint="selectWidthType" hasDownArrow="true" dojoType="dijit.form.FilteringSelect">
                  <option value="percent">${percent}</option>
                  <option value="pixels">${pixels}</option>
                </select></td></tr>	
            <tr><td>
                <label>${borderThickness}</label></td>
            </td><td>
                <span dojoAttachPoint="selectBorder" dojoType="dijit.form.TextBox" value="1"></span>
            </td><td>
                ${pixels}
            </td></tr><tr><td>
                <label>${cellPadding}</label></td>
            </td><td>
                <span dojoAttachPoint="selectPad" dojoType="dijit.form.TextBox" value="0"></span>
            </td><td class="cellpad"></td></tr><tr><td>
                <label>${cellSpacing}</label>
            </td><td>
                <span dojoAttachPoint="selectSpace" dojoType="dijit.form.TextBox" value="0"></span>
            </td><td class="cellspace"></td></tr></table>
        <div class="dialogButtonContainer">
            <div dojoType="dijit.form.Button" dojoAttachEvent="onClick: onInsert">${buttonInsert}</div>
            <div dojoType="dijit.form.Button" dojoAttachEvent="onClick: onCancel">${buttonCancel}</div>
        </div>
	</div>
</div>
