({"replaceDialogText":"${0} ricorrenze sostituite.","eofDialogTextFind":"trovato","eofDialogText":"Ultima ricorrenza ${0}","backwards":"Indietro","replaceButton":"Sostituisci","replaceLabel":"Sostituisci con:","matchCase":"Maiuscole/minuscole","findTooltip":"Immettere il testo da trovare","replaceTooltip":"Immettere il testo sostitutivo","replaceAllButton":"Sostituisci tutto","eofDialogTextReplace":"sostituito","findReplace":"Trova e sostituisci","backwardsTooltip":"Cerca testo indietro","replaceAllButtonTooltip":"Sostituisci tutto il testo","replaceButtonTooltip":"Sostituisci il testo","findLabel":"Trova:","findButton":"Trova","matchCaseTooltip":"Maiuscole/minuscole","findButtonTooltip":"Trova il testo","replaceAll":"Tutte le ricorrenze"})