({"conditionBefore":"pred ","filterBarDefButton":"Definiraj filter ","filterBarMsgNoFilterTemplate":"Ni uveljavljenih filtrov ","conditionContains":"vsebuje ","clearFilterMsg":"S tem boste odstranili filter in prikazali vse razpoložljive zapise. ","all":"vse","waiValueBoxTemplate":"Vnesite vrednost za filtriranje za pravilo ${0}","conditionRange":"obseg","waiCancelButton":"Prekliči to pogovorno okno ","relationAll":"vsa pravila ","conditionLarger":"je večje kot ","valueBoxLabel":"Vrednost ","statusTipMsg":"Kliknite to vrstico filtra, da filtrirate vrednosti v ${0}.","waiRelAll":"Ujemanje z vsemi naslednjimi pravilo:","closeFilterBarBtn":"Zapri vrstico filtra ","a11yFilterBarDefButton":"Filter...","trueLabel":"True","conditionIsNot":"ni ","conditionIs":"je","ruleTitleTemplate":"Pravilo ${0}","waiRemoveRuleButtonTemplate":"Odstrani pravilo ${0}","and":"in ","conditionLess":"je manjše kot ","waiColumnSelectTemplate":"Stolpec za pravilo ${0}","anycolumn":"katerikoli stolpec ","filterDefDialogTitle":"Filter","waiConditionSelectTemplate":"Pogoj za pravilo ${0}","waiFilterButton":"Predloži filter ","statusTipTitle":"Vrstica filtra ","statusTipRelPre":"Ujemanje ","anyColumnOption":"Katerikoli stolpec ","conditionSelectLabel":"Pogoj ","cancelButton":"Prekliči","conditionNotStartWith":"se ne začne z ","conditionEndWith":"konča se z ","statusTipTitleNoFilter":"Vrstica filtra ","columnSelectLabel":"Stolpec ","conditionNotContain":"ne vsebuje","clearButton":"Počisti ","conditionLargerEqual":"večje ali enako ","any":"karkoli","relationMsgFront":"Ujemanje ","statusTipHeaderColumn":"Stolpec ","defaultItemsName":"postavke ","conditionEqual":"je enako kot ","relationAny":"katerakoli pravila ","trueLabelEditable":"Označeno ","or":"ali ","filterBarMsgHasFilterTemplate":"${0} od ${1} prikazanih ${2} ","rangeTo":"do","conditionStartsWith":"začne se z ","waiRelAny":"Ujemanje s katerimkoli od naslednjih pravil: ","statusTipRelPost":"pravila. ","relationMsgTail":"","filterBarClearButton":"Počisti filter","incompleteRuleTip":"To pravilo ni dokončano.","conditionLessEqual":"manjše ali enako kot ","waiFilterBarClearButton":"Počisti filter ","conditionAfter":"za ","statusTipHeaderCondition":"Pravila ","falseLabel":"False","filterButton":"Filter","clearFilterDialogTitle":"Počisti filter ","statusTipTitleHasFilter":"Filter","addRuleButton":"Dodaj pravilo ","waiAddRuleButton":"Dodaj novo pravilo ","waiClearButton":"Počisti filter ","conditionNotEqual":"ni enako ","conditionNotEndWith":"se ne konča z ","rangeTemplate":"od ${0} do ${1}","removeRuleButton":"Odstrani pravilo ","waiFilterBarDefButton":"Filtriraj tabelo ","conditionIsEmpty":"is empty"})