({"replaceDialogText":"${0} 個のオカレンスを置換しました。","eofDialogTextFind":"見つかりました","eofDialogText":"最後のオカレンス ${0}","backwards":"後方","replaceButton":"置換","replaceLabel":"次で置換:","matchCase":"大/小文字の区別","findTooltip":"検索するテキストを入力","replaceTooltip":"置換するテキストを入力","replaceAllButton":"すべてを置換","eofDialogTextReplace":"置換されました","findReplace":"検索/置換の切り替え","backwardsTooltip":"テキストを後方検索","replaceAllButtonTooltip":"テキストすべてを置換","replaceButtonTooltip":"テキストを置換","findLabel":"検索内容:","findButton":"検索","matchCaseTooltip":"大/小文字を区別","findButtonTooltip":"テキストを検索","replaceAll":"すべてのオカレンス"})