<div class="dojoxLightbox" dojoAttachPoint="containerNode">
	<div style="position:relative">
		<div dojoAttachPoint="imageContainer" class="dojoxLightboxContainer" dojoAttachEvent="onclick: _onImageClick">
			<img dojoAttachPoint="imgNode" src="${imgUrl}" class="dojoxLightboxImage" alt="${title}">
			<div class="dojoxLightboxFooter" dojoAttachPoint="titleNode">
				<div class="dijitInline LightboxClose" dojoAttachPoint="closeButtonNode"></div>
				<div class="dijitInline LightboxNext" dojoAttachPoint="nextButtonNode"></div>	
				<div class="dijitInline LightboxPrev" dojoAttachPoint="prevButtonNode"></div>
				<div class="dojoxLightboxText" dojoAttachPoint="titleTextNode"><span dojoAttachPoint="textNode">${title}</span><span dojoAttachPoint="groupCount" class="dojoxLightboxGroupText"></span></div>
			</div>
		</div>
	</div>
</div>