({"buttonSet":"Nastavit","insertTableTitle":"Vložit tabulku","insertTableRowAfterLabel":"Přidat řádek za","buttonCancel":"Storno","center":"střed","deleteTableColumnLabel":"Odstranit sloupec","right":"vpravo","insertTableColumnBeforeLabel":"Přidat sloupec před","tableWidth":"Šířka tabulky:","buttonInsert":"Vložit","default":"výchozí","align":"Zarovnat:","insertTableRowBeforeLabel":"Přidat řádek před","cellSpacing":"Odstup buněk:","pixels":"pixelů","selectTableLabel":"Vybrat tabulku","rows":"Řádky:","modifyTableTitle":"Upravit tabulku","cellPadding":"Okraj buňky:","deleteTableRowLabel":"Odstranit řádek","backgroundColor":"Barva pozadí:","insertTableColumnAfterLabel":"Přidat sloupec za","left":"vlevo","borderThickness":"TloušťkaOhraničení","columns":"Sloupce:","percent":"procent","borderColor":"Barva ohraničení:"})