({"replaceDialogText":"Zastąpione wystąpienia: ${0}.","eofDialogTextFind":"znaleziono","eofDialogText":"Ostatnie wystąpienie: ${0}","backwards":"Do tyłu","replaceButton":"Zastąp","replaceLabel":"Zastąp przez:","matchCase":"Uwzględnij wielkość liter","findTooltip":"Wprowadź szukany tekst","replaceTooltip":"Wprowadź tekst zastępujący","replaceAllButton":"Zastąp wszystkie","eofDialogTextReplace":"zastąpiono","findReplace":"Znajdź i zastąp","backwardsTooltip":"Wyszukaj tekst wstecz","replaceAllButtonTooltip":"Zastąp cały tekst","replaceButtonTooltip":"Zastąp tekst","findLabel":"Znajdź:","findButton":"Znajdź","matchCaseTooltip":"Uwzględnij wielkość liter","findButtonTooltip":"Znajdź tekst","replaceAll":"Wszystkie wystąpienia"})