({"valueLabel":"l","hueLabel":"n","saturationLabel":"m","saturationPickerTitle":"Välj mättnad","huePickerTitle":"Välj färgton","blueLabel":"b","greenLabel":"g","redLabel":"r","hexLabel":"hex","degLabel":"°"})