({"set":"Ekle","text":"Açıklama","insertImageTitle":"Resim Ekle","invalidMessage":"Geçersiz resim dosyası tipi","prePopuTextBrowse":" ya da yerel bir dosyaya göz atın.","browse":"Göz at...","prePopuTextUrl":"Bir resim URL'si girin","url":"Resim"})