({"saturationPickerTitle":"Izbirnik nasičenosti","huePickerTitle":"Izbirnik odtenka ","valueLabel":"v","blueLabel":"b","saturationLabel":"s","greenLabel":"g","redLabel":"r","hueLabel":"h","hexLabel":"hex","degLabel":"°"})