.dijitBorderContainer .dojoxToggleSplitterTop,
.dijitBorderContainer .dojoxToggleSplitterBottom {
	height:16px;
}
.dijitBorderContainer .dojoxToggleSplitterLeft,
.dijitBorderContainer .dojoxToggleSplitterRight {
	width:16px;
}

.dijitBorderContainer .dijitSplitterH .dojoxToggleSplitterIcon {
	width:16px;
	height:16px;
	top: 0;
	cursor: pointer;
}
.dijitBorderContainer .dijitSplitterV .dojoxToggleSplitterIcon {
	width:16px;
	height:16px;
	left: 0;
	cursor: pointer;
}

.tundra .dojoxToggleSplitterTop .dojoxToggleSplitterIconOpen, 
.tundra .dojoxToggleSplitterBottom .dojoxToggleSplitterIconClosed 
 {
	background: transparent url('../../../dijit/themes/tundra/images/spriteRoundedIconsSmall.png') no-repeat -45px 50%;
}
.tundra .dojoxToggleSplitterBottom .dojoxToggleSplitterIconOpen, 
.tundra .dojoxToggleSplitterTop .dojoxToggleSplitterIconClosed 
 {
	background: transparent url('../../../dijit/themes/tundra/images/spriteRoundedIconsSmall.png') no-repeat -15px 50%;
}

.tundra .dojoxToggleSplitterLeft .dojoxToggleSplitterIconOpen, 
.tundra .dojoxToggleSplitterRight .dojoxToggleSplitterIconClosed 
 {
	background: transparent url('../../../dijit/themes/tundra/images/spriteRoundedIconsSmall.png') no-repeat 0px 50%;
}
.tundra .dojoxToggleSplitterRight .dojoxToggleSplitterIconOpen, 
.tundra .dojoxToggleSplitterLeft .dojoxToggleSplitterIconClosed 
 {
	background: transparent url('../../../dijit/themes/tundra/images/spriteRoundedIconsSmall.png') no-repeat -30px 50%;
}

.soria .dojoxToggleSplitterTop .dojoxToggleSplitterIconOpen, 
.soria .dojoxToggleSplitterBottom .dojoxToggleSplitterIconClosed 
 {
	background: transparent url('../../../dijit/themes/soria/images/spriteRoundedIconsSmall.png') no-repeat -45px 50%;
}
.soria .dojoxToggleSplitterBottom .dojoxToggleSplitterIconOpen, 
.soria .dojoxToggleSplitterTop .dojoxToggleSplitterIconClosed 
 {
	background: transparent url('../../../dijit/themes/soria/images/spriteRoundedIconsSmall.png') no-repeat -15px 50%;
}

.soria .dojoxToggleSplitterLeft .dojoxToggleSplitterIconOpen, 
.soria .dojoxToggleSplitterRight .dojoxToggleSplitterIconClosed 
 {
	background: transparent url('../../../dijit/themes/soria/images/spriteRoundedIconsSmall.png') no-repeat 0px 50%;
}
.soria .dojoxToggleSplitterRight .dojoxToggleSplitterIconOpen, 
.soria .dojoxToggleSplitterLeft .dojoxToggleSplitterIconClosed 
 {
	background: transparent url('../../../dijit/themes/soria/images/spriteRoundedIconsSmall.png') no-repeat -30px 50%;
}

.nihilo .dojoxToggleSplitterTop .dojoxToggleSplitterIconOpen, 
.nihilo .dojoxToggleSplitterBottom .dojoxToggleSplitterIconClosed 
 {
	background: transparent url('../../../dijit/themes/nihilo/images/spriteRoundedIconsSmall.png') no-repeat -45px 50%;
}
.nihilo .dojoxToggleSplitterBottom .dojoxToggleSplitterIconOpen, 
.nihilo .dojoxToggleSplitterTop .dojoxToggleSplitterIconClosed 
 {
	background: transparent url('../../../dijit/themes/nihilo/images/spriteRoundedIconsSmall.png') no-repeat -15px 50%;
}

.nihilo .dojoxToggleSplitterLeft .dojoxToggleSplitterIconOpen, 
.nihilo .dojoxToggleSplitterRight .dojoxToggleSplitterIconClosed 
 {
	background: transparent url('../../../dijit/themes/nihilo/images/spriteRoundedIconsSmall.png') no-repeat 0px 50%;
}
.nihilo .dojoxToggleSplitterRight .dojoxToggleSplitterIconOpen, 
.nihilo .dojoxToggleSplitterLeft .dojoxToggleSplitterIconClosed 
 {
	background: transparent url('../../../dijit/themes/nihilo/images/spriteRoundedIconsSmall.png') no-repeat -30px 50%;
}
