({"cancel":"キャンセル","toDic":"辞書に追加","skip":"スキップ","iMsg":"スペルの修正候補はありません","replaceAll":"すべてを置換","widgetLabel":"一括スペル・チェック","skipAll":"すべてをスキップ","suggestions":"修正候補","iSkipAll":"これと類似のものをスキップ","replaceWith":"置き換え","msg":"ミススペルはありません","replace":"置換","unfound":"見つかりません","iSkip":"これをスキップ"})