({"saveLabel":"保存","saveSettingdialogTitle":"自動保存","saveSettingdialogButtonOk":"間隔の設定","saveMessageSuccess":"${0} に保存されました","saveSettingdialogButtonCancel":"キャンセル","saveSettingdialogDescription":"自動保存間隔の指定","saveSettingLabelOn":"自動保存間隔の設定...","saveSettingdialogParamLabel":"分","saveSettingdialogParamName":"自動保存間隔","saveMessageFail":"${0} に保存に失敗しました","saveSettingLabelOff":"自動保存をオフにする"})