({"replaceDialogText":"Αντικαταστάθηκαν ${0} εμφανίσεις.","eofDialogTextFind":"εντοπίστηκε","eofDialogText":"Τελευταία εμφάνιση: ${0}","backwards":"Προς τα πίσω","replaceButton":"Αντικατάσταση","replaceLabel":"Αντικατάσταση με:","matchCase":"Διάκριση πεζών/κεφαλαίων","findTooltip":"Καταχωρήστε το κείμενο που θέλετε να εντοπίσετε","replaceTooltip":"Καταχωρήστε το κείμενο με το οποίο θέλετε να αντικαταστήσετε το αναζητούμενο κείμενο","replaceAllButton":"Αντικατάσταση όλων","eofDialogTextReplace":"αντικαταστάθηκε","findReplace":"Εύρεση και αντικατάσταση","backwardsTooltip":"Αναζήτηση κειμένου προς τα πίσω","replaceAllButtonTooltip":"Αντικατάσταση όλου του κειμένου","replaceButtonTooltip":"Αντικατάσταση του κειμένου","findLabel":"Εύρεση:","findButton":"Εύρεση","matchCaseTooltip":"Διάκριση πεζών/κεφαλαίων","findButtonTooltip":"Εύρεση του κειμένου","replaceAll":"Όλες οι εμφανίσεις"})