<div class="playerContainer">
	<div class="PlayerScreen" dojoAttachPoint="playerScreen"></div>
	<table class="Controls">
		<tr>
			<td colspan="2" dojoAttachPoint="progressContainer"></td>
		</tr>
		<tr>
			<td class="PlayContainer" dojoAttachPoint="playContainer"></td>
			<td class="ControlsRight">
			<table class="StatusContainer">
				<tr dojoAttachPoint="statusContainer">
				</tr>
				<tr>
					<td colspan="3" class="ControlsBottom" dojoAttachPoint="controlsBottom"></td>
				</tr>
			</table>
		</td>
		</tr>
	</table>
</div>
