({"replaceDialogText":"${0}개를 대체했습니다.","eofDialogTextFind":"찾음","eofDialogText":"마지막 발생 ${0}","backwards":"뒤로","replaceButton":"바꾸기","replaceLabel":"바꿀 대상:","matchCase":"대소문자 구분","findTooltip":"찾을 텍스트 입력","replaceTooltip":"대체할 텍스트 입력","replaceAllButton":"모두 바꾸기","eofDialogTextReplace":"대체됨","findReplace":"찾기 및 바꾸기","backwardsTooltip":"텍스트 역방향 검색","replaceAllButtonTooltip":"텍스트 모두 바꾸기","replaceButtonTooltip":"텍스트 바꾸기","findLabel":"찾기:","findButton":"찾기","matchCaseTooltip":"대소문자 구분","findButtonTooltip":"텍스트 찾기","replaceAll":"모든 발생"})