({"replaceDialogText":"แทนที่ ${0} ที่เกิดขึ้น","eofDialogTextFind":"หาพบ","eofDialogText":"การเกิดขึ้นล่าสุด ${0}","backwards":"ย้อนกลับ","replaceButton":"แทนที่","replaceLabel":"แทนที่ด้วย:","matchCase":"ตรงตามตัวพิมพ์ใหญ่เล็ก","findTooltip":"ป้อนข้อความเพื่อหา","replaceTooltip":"ป้อนข้อความเพื่อแทนที่ด้วย","replaceAllButton":"แทนที่ทั้งหมด","eofDialogTextReplace":"ถูกแทนที่","findReplace":"ค้นหาและแทนที่","backwardsTooltip":"ค้นหาย้อนกับเพื่อหาข้อความ","replaceAllButtonTooltip":"แทนที่ข้อความทั้งหมด","replaceButtonTooltip":"แทนที่ข้อความ","findLabel":"ค้นหา:","findButton":"ค้นหา","matchCaseTooltip":"ตรงตามตัวพิมพ์ใหญ่เล็ก","findButtonTooltip":"หาข้อความ","replaceAll":"ที่เกิดขึ้นทั้งหมด"})