({"emoticonLaughing":"צחוק ","emoticonCool":"מגניב","emoticonTongue":"לשון ","emoticonCrying":"בוכה","emoticonOops":"אופס ","emoticonFrown":"קימוט מצח","emoticonAngry":"כועס ","emoticonShy":"ביישן","emoticonNo":"לא","emoticonAngel":"מלאך ","smiley":"הוספת רגשון","emoticonIdea":"רעיון","emoticonEyebrow":"גבה","emoticonSmile":"חיוך","emoticonWink":"קריצה","emoticonYes":"כן","emoticonGrin":"גיחוך","emoticonGoofy":"דבילי","emoticonHalf":"חצי "})