({"gotoButtonTitle":"Белгілі бір бетке өту","prevTip":"Алдыңғы бет","pageCountIndication":" (${0} бет)","pageStepLabelTemplate":"Бет ${0}","descTemplate":"${1} ${0} элементтің ${2} - ${3} элементі","lastTip":"Соңғы бет","nextTip":"Келесі бет","pageSizeLabelTemplate":"Бетіне ${0} элемент","dialogCancel":"Болдырмау","itemTitle":"элементтер","dialogConfirm":"Өту","firstTip":"Бірінші бет","allItemsLabelTemplate":"Барлық элементтер","dialogIndication":"Бет нөмірін көрсету","dialogTitle":"Бетке өту"})