({"gotoButtonTitle":"Ugrás adott oldalra","prevTip":"Előző oldal","pageCountIndication":" (${0} oldal)","pageStepLabelTemplate":"${0} oldal","descTemplate":"${2} - ${3}, összesen ${1} ${0}","lastTip":"Utolsó oldal","nextTip":"Következő oldal","pageSizeLabelTemplate":"${0} elem oldalanként","dialogCancel":"Mégse","itemTitle":"elem","dialogConfirm":"Ugrás","firstTip":"Első oldal","allItemsLabelTemplate":"Minden elem","dialogIndication":"Adja meg az oldalszámot","dialogTitle":"Oldalszámhoz ugrás"})