<div class="dojoxGridFDPane">
	<div class="dojoxGridFDPaneRelation">${_relMsgFront}
	<span class="dojoxGridFDPaneModes" dojoAttachPoint="criteriaModeNode">
		<select dojoAttachPoint="_relSelect" dojoType="dijit.form.Select" dojoAttachEvent="onChange: _onRelSelectChange">
			<option value="0">${_relAll}</option>
			<option value="1">${_relAny}</option>
		</select>
	</span>
	${_relMsgTail}
	</div>
	<div dojoAttachPoint="criteriaPane" class="dojoxGridFDPaneRulePane"></div>
	<div dojoAttachPoint="_addCBoxBtn" dojoType="dijit.form.Button" 
		class="dojoxGridFDPaneAddCBoxBtn" iconClass="dojoxGridFDPaneAddCBoxBtnIcon"
		dojoAttachEvent="onClick:_onAddCBox" label="${_addRuleBtnLabel}" showLabel="false">
	</div>
	<div class="dojoxGridFDPaneBtns" dojoAttachPoint="buttonsPane">
		<span dojoAttachPoint="_cancelBtn" dojoType="dijit.form.Button" 
			dojoAttachEvent="onClick:_onCancel" label="${_cancelBtnLabel}">
		</span>
		<span dojoAttachPoint="_clearFilterBtn" dojoType="dijit.form.Button" 
			dojoAttachEvent="onClick:_onClearFilter" label="${_clearBtnLabel}" disabled="true">
		</span>
		<span dojoAttachPoint="_filterBtn" dojoType="dijit.form.Button" 
			dojoAttachEvent="onClick:_onFilter" label="${_filterBtnLabel}" disabled="true">
		</span>
	</div>
</div>
