({"set":"Inserir","text":"Descrição","insertImageTitle":"Inserir imagem","invalidMessage":"Tipo de arquivo de imagem inválido","prePopuTextBrowse":" ou navegue até um arquivo local.","browse":"Procurar...","prePopuTextUrl":"Insira uma URL de imagem","url":"Imagem"})