({"replaceDialogText":"Erstattet ${0} forekomster.","eofDialogTextFind":"funnet","eofDialogText":"Siste forekomst ${0}","backwards":"Bakover","replaceButton":"Erstatt","replaceLabel":"Erstatt med:","matchCase":"Skill mellom store og små bokstaver","findTooltip":"Skriv inn teksten du vil søke etter","replaceTooltip":"Skriv inn teksten du vil erstatte med","replaceAllButton":"Erstatt alle","eofDialogTextReplace":"erstattet","findReplace":"Søk og erstatt","backwardsTooltip":"Søk bakover etter tekst","replaceAllButtonTooltip":"Erstatt all tekst","replaceButtonTooltip":"Erstatt teksten","findLabel":"Søk:","findButton":"Søk","matchCaseTooltip":"Skill mellom store og små bokstaver","findButtonTooltip":"Søk etter teksten","replaceAll":"Alle forekomster"})