({"conditionBefore":"לפני ","filterBarDefButton":"הגדרת מסנן ","filterBarMsgNoFilterTemplate":"לא הוחל מסנן. ","conditionContains":"מכיל ","clearFilterMsg":"פעולה זו תסלק את המסנן ותציג את כל הרשומות הזמינות. ","all":"הכל","waiValueBoxTemplate":"ציינו ערך לסינון עבור הכלל ${0}","conditionRange":"טווח ","waiCancelButton":"ביטול דו-שיח זה ","relationAll":"כל הכללים ","conditionLarger":"גדול מ-","valueBoxLabel":"ערך ","statusTipMsg":"לחצנו על סרגל הסינון כאן כדי לסנן לפי ערכים בתוך ${0}.","waiRelAll":"התאמה לכל הכללים שלהלן: ","closeFilterBarBtn":"סגירת סרגל הסינון","a11yFilterBarDefButton":"סינון...‏","trueLabel":"אמיתי","conditionIsNot":"אינו ","conditionIs":"הוא","ruleTitleTemplate":"כלל ${0}","waiRemoveRuleButtonTemplate":"סילוק הכלל ${0}","and":"וגם","conditionLess":"קטן מ-","waiColumnSelectTemplate":"עמודה עבור הכלל ${0}","anycolumn":"עמודה כלשהי","filterDefDialogTitle":"מסנן","waiConditionSelectTemplate":"תנאי עבור הכלל ${0}","waiFilterButton":"הגשת המסנן","statusTipTitle":"סרגל סינון ","statusTipRelPre":"התאמה","anyColumnOption":"עמודה כלשהי","conditionSelectLabel":"תנאי ","cancelButton":"ביטול","conditionNotStartWith":"אינו מתחיל עם ","conditionEndWith":"מסתיים עם ","statusTipTitleNoFilter":"סרגל סינון","columnSelectLabel":"עמודה ","conditionNotContain":"אינו מכיל ","clearButton":"ניקוי ","conditionLargerEqual":"גדול או שווה ","any":"כלשהו","relationMsgFront":"התאמה ","statusTipHeaderColumn":"עמודה","defaultItemsName":"פריטים ","conditionEqual":"שווה ","relationAny":"כללים כלשהם","trueLabelEditable":"נבדק","or":"או","filterBarMsgHasFilterTemplate":"מוצגים ${0} מתוך ${1} ${2}.‏ ","rangeTo":"עד","conditionStartsWith":"מתחיל עם ","waiRelAny":"התאמה לכלל כלשהו הכללים שלהלן:","statusTipRelPost":"כללים.","relationMsgTail":"","filterBarClearButton":"ניקוי מסנן","incompleteRuleTip":"הכלל אינו שלם. ","conditionLessEqual":"קטן או שווה ","waiFilterBarClearButton":"ניקוי המסנן","conditionAfter":"אחרי ","statusTipHeaderCondition":"כללים ","falseLabel":"שקרי ","filterButton":"מסנן","clearFilterDialogTitle":"ניקוי מסנן ","statusTipTitleHasFilter":"מסנן","addRuleButton":"הוספת כלל","waiAddRuleButton":"הוספת כלל חדש ","waiClearButton":"ניקוי המסנן ","conditionNotEqual":"אינו שווה ","conditionNotEndWith":"אינו מסתיים עם","rangeTemplate":"מ-${0} עד ${1}","removeRuleButton":"סילוק כלל  ","waiFilterBarDefButton":"סינון הטבלה ","conditionIsEmpty":"is empty"})