({"conditionBefore":"antes","filterBarDefButton":"Definir filtro","filterBarMsgNoFilterTemplate":"No se aplica ningún filtro","conditionContains":"contiene","clearFilterMsg":"Eliminará el filtro y mostrará todos los registros disponibles.","all":"todas","waiValueBoxTemplate":"Especifique un valor para el filtro para la regla ${0}","conditionRange":"rango","waiCancelButton":"Cancelar este diálogo","relationAll":"todas las reglas","conditionLarger":"es superior que","valueBoxLabel":"Valor","statusTipMsg":"Pulse la barra de filtros para filtrar los valores de ${0}.","waiRelAll":"Coincide con todas las reglas siguientes:","closeFilterBarBtn":"Cerrar la barra de filtros","a11yFilterBarDefButton":"Filtrar...","trueLabel":"Verdadero","conditionIsNot":"no es","conditionIs":"es","ruleTitleTemplate":"Regla ${0}","waiRemoveRuleButtonTemplate":"Eliminar regla ${0}","and":"y","conditionLess":"es menor que","waiColumnSelectTemplate":"Columna para la regla ${0}","anycolumn":"cualquier columna","filterDefDialogTitle":"Filtro","waiConditionSelectTemplate":"Condición para la regla ${0}","waiFilterButton":"Enviar el filtro","statusTipTitle":"Barra de filtros","statusTipRelPre":"Coincide","anyColumnOption":"Cualquier columna","conditionSelectLabel":"Condición","cancelButton":"Cancelar","conditionNotStartWith":"no empieza por","conditionEndWith":"termina por","statusTipTitleNoFilter":"Barra de filtros","columnSelectLabel":"Columna","conditionNotContain":"no contiene","clearButton":"Borrar","conditionLargerEqual":"superior o igual que","any":"cualquiera","relationMsgFront":"Coincide","statusTipHeaderColumn":"Columna","defaultItemsName":"elementos","conditionEqual":"igual","relationAny":"cualquier regla","trueLabelEditable":"Marcado","or":"o","filterBarMsgHasFilterTemplate":"Se muestran ${0} de ${1} ${2}.","rangeTo":"a","conditionStartsWith":"empieza por","waiRelAny":"Coincide con cualquiera de las reglas siguientes:","statusTipRelPost":"reglas.","relationMsgTail":"","filterBarClearButton":"Borrar filtro","incompleteRuleTip":"Esta regla no está completa.","conditionLessEqual":"es menor o igual que","waiFilterBarClearButton":"Borrar el filtro","conditionAfter":"después","statusTipHeaderCondition":"Reglas","falseLabel":"Falso","filterButton":"Filtro","clearFilterDialogTitle":"Limpiar filtro","statusTipTitleHasFilter":"Filtro","addRuleButton":"Añadir regla","waiAddRuleButton":"Añadir una regla nueva","waiClearButton":"Borrar el filtro","conditionNotEqual":"no es igual","conditionNotEndWith":"no termina por","rangeTemplate":"de ${0} a ${1}","removeRuleButton":"Eliminar regla","waiFilterBarDefButton":"Filtrar la tabla","conditionIsEmpty":"is empty"})