({"gotoButtonTitle":"اذهب الى صفحة خاصة","prevTip":"الصفحة السابقة","pageCountIndication":" (${0} صفحات)","pageStepLabelTemplate":"صفحة ${0}","descTemplate":"${2} - ${3} من ${1} ${0}","lastTip":"الصفحة الأخيرة","nextTip":"الصفحة التالية","pageSizeLabelTemplate":"${0} بنود لكل صفحة","dialogCancel":"الغاء","itemTitle":"البنود","dialogConfirm":"بدء","firstTip":"الصفحة الأولى","allItemsLabelTemplate":"كل البنود","dialogIndication":"تحديد رقم الصفحة","dialogTitle":"اذهب الى صفحة"})