({"conditionBefore":"før","filterBarDefButton":"Definér filter","filterBarMsgNoFilterTemplate":"Intet filter anvendt","conditionContains":"indeholder","clearFilterMsg":"Denne funktion fjerner filtret og viser alle tilgængelige records.","all":"alle","waiValueBoxTemplate":"Angiv værdi for filtrering for regel ${0}","conditionRange":"område","waiCancelButton":"Annullér denne dialogboks","relationAll":"alle regler","conditionLarger":"er større end","valueBoxLabel":"Værdi","statusTipMsg":"Klik på filterlinjen for at filtrere efter værdier i ${0}.","waiRelAll":"Skal matche følgende regler:","closeFilterBarBtn":"Luk filterlinjen","a11yFilterBarDefButton":"Filtrér...","trueLabel":"Sand","conditionIsNot":"er ikke","conditionIs":"er","ruleTitleTemplate":"Regel ${0}","waiRemoveRuleButtonTemplate":"Fjern reglel ${0}","and":"og","conditionLess":"er mindre end","waiColumnSelectTemplate":"Kolonne for regel ${0}","anycolumn":"vilkårlig kolonne","filterDefDialogTitle":"Filtrér","waiConditionSelectTemplate":"Betingelse for regel ${0}","waiFilterButton":"Afsend filtret","statusTipTitle":"Filterlinje","statusTipRelPre":"Match","anyColumnOption":"Vilkårlig kolonne","conditionSelectLabel":"Betingelse","cancelButton":"Annullér","conditionNotStartWith":"begynder ikke med","conditionEndWith":"slutter med","statusTipTitleNoFilter":"Filterlinje","columnSelectLabel":"Kolonne","conditionNotContain":"indeholder ikke","clearButton":"Ryd","conditionLargerEqual":"større end eller lig med","any":"nogle af","relationMsgFront":"Match","statusTipHeaderColumn":"Kolonne","defaultItemsName":"elementer","conditionEqual":"er lig med","relationAny":"nogle regler","trueLabelEditable":"Markeret","or":"eller","filterBarMsgHasFilterTemplate":"${0} af ${1} ${2} vist.","rangeTo":"til","conditionStartsWith":"begynder med","waiRelAny":"Skal matche nogle af følgende regler:","statusTipRelPost":"regler.","relationMsgTail":"","filterBarClearButton":"Ryd filter","incompleteRuleTip":"Reglen er ikke komplet.","conditionLessEqual":"mindre end eller lig med","waiFilterBarClearButton":"Ryd filtret","conditionAfter":"efter","statusTipHeaderCondition":"Regler","falseLabel":"Falsk","filterButton":"Filtrér","clearFilterDialogTitle":"Ryd filter","statusTipTitleHasFilter":"Filtrér","addRuleButton":"Tilføj regel","waiAddRuleButton":"Tilføj en ny regel","waiClearButton":"Ryd filtret","conditionNotEqual":"er ikke lig med","conditionNotEndWith":"slutter ikke med","rangeTemplate":"fra ${0} til ${1}","removeRuleButton":"Fjern regel","waiFilterBarDefButton":"Filtrér tabellen","conditionIsEmpty":"is empty"})