({"cancel":"取消","toDic":"新增至字典","skip":"跳過","iMsg":"沒有拼字建議","replaceAll":"全部取代","widgetLabel":"批次拼字檢查","skipAll":"全部跳過","suggestions":"建議","iSkipAll":"跳過所有如此項的項目","replaceWith":"取代為","msg":"找不到拼錯","replace":"取代","unfound":"找不到","iSkip":"跳過此項"})