.dojoxLegendNode {
	background-color:#F7F7F7;
	border:1px solid #CCCCCC;
	margin:0;
	padding:0;	
}
.dojoxLegendNode td{
	padding: 6px 4px 6px 6px;
}
.dojoxLegendIcon {
	padding: 0px;
	margin: 0 2px 0 4px;
}
.dojoxLegendIcon div{
	float: left;
}
.dj_ie .dojoxLegendNode td{
	padding: 5px;	
}
.dj_ie .dojoxLegendIcon {
	padding: 0px;
	margin: 0 2px 0 5px;
	vertical-align: middle;
}
