<div class="dojoxCalendarMonthLabels" style="left: 0px;"  
	dojoAttachPoint="monthContainer" dojoAttachEvent="onclick: onClick">
    <table cellspacing="0" cellpadding="0" border="0" style="margin: auto;">
        <tbody>
            <tr class="dojoxCalendarMonthGroupTemplate">
                <td class="dojoxCalendarMonthTemplate">
                    <div class="dojoxCalendarMonthLabel"></div>
                </td>
             </tr>
        </tbody>
    </table>
</div>
