({"buttonSet":"הגדרה","insertTableTitle":"הוספת טבלה","insertTableRowAfterLabel":"הוספת שורה אחרי","buttonCancel":"ביטול","center":"מרכז","deleteTableColumnLabel":"מחיקת עמודה","right":"שמאל","insertTableColumnBeforeLabel":"הוספת עמודה לפני","tableWidth":"רוחב טבלה:‏","buttonInsert":"הוספה","default":"ברירת מחדל","align":"יישור:","insertTableRowBeforeLabel":"הוספת שורה לפני","cellSpacing":"ריווח תאים:","pixels":"פיקסלים","selectTableLabel":"בחירת טבלה","rows":"שורות:","modifyTableTitle":"שינוי טבלה","cellPadding":"ריפוד תאים:","deleteTableRowLabel":"מחיקת שורה","backgroundColor":"צבע רקע:","insertTableColumnAfterLabel":"הוספת עמודה אחרי","left":"ימין","borderThickness":"עובי גבול","columns":"עמודות:","percent":"אחוזים","borderColor":"צבע גבול:"})