({"saveLabel":"Gem","saveSettingdialogTitle":"Gem automatisk","saveSettingdialogButtonOk":"Angiv interval","saveMessageSuccess":"Gemt i ${0}","saveSettingdialogButtonCancel":"Annullér","saveSettingdialogDescription":"Angiv interval for automatisk lagring","saveSettingLabelOn":"Angiv interval for automatisk lagring...","saveSettingdialogParamLabel":"min.","saveSettingdialogParamName":"Interval for automatisk lagring","saveMessageFail":"Ikke gemt i ${0}","saveSettingLabelOff":"Deaktivér automatisk lagring"})