({"emoticonLaughing":"kahkaha","emoticonCool":"havalı","emoticonTongue":"dil çıkaran ifade","emoticonCrying":"ağlayan ifade","emoticonOops":"şaşıran ifade","emoticonFrown":"kaşı çatık ifade","emoticonAngry":"kızgın","emoticonShy":"utangaç","emoticonNo":"hayır","emoticonAngel":"melek","smiley":"İfade Ekle","emoticonIdea":"aklına bir fikir gelmiş","emoticonEyebrow":"kaşı kalkık ifade","emoticonSmile":"gülümseme","emoticonWink":"göz kırpma","emoticonYes":"evet","emoticonGrin":"sırıtma","emoticonGoofy":"ağzı açık ifade","emoticonHalf":"kafası karışık ifade"})