({"conditionBefore":"prima","filterBarDefButton":"Definisci filtro","filterBarMsgNoFilterTemplate":"Nessun filtro applicato","conditionContains":"contiene","clearFilterMsg":"In tal modo il filtro verrà rimosso e verranno mostrati tutti i record disponibili.","all":"tutti","waiValueBoxTemplate":"Immettere il valore da filtrare per la regola ${0}","conditionRange":"intervallo","waiCancelButton":"Annulla questa finestra di dialogo","relationAll":"tutte le regole","conditionLarger":"maggiore di","valueBoxLabel":"Valore","statusTipMsg":"Fare clic sulla barra filtro per filtrare in base ai valori in ${0}.","waiRelAll":"Soddisfa tutte le seguenti regole:","closeFilterBarBtn":"Chiudi barra filtro","a11yFilterBarDefButton":"Filtra...","trueLabel":"True","conditionIsNot":"non è","conditionIs":"è","ruleTitleTemplate":"Regola ${0}","waiRemoveRuleButtonTemplate":"Rimuovi regola ${0}","and":"e","conditionLess":"minore di","waiColumnSelectTemplate":"Colonna per la regola ${0}","anycolumn":"qualunque colonna","filterDefDialogTitle":"Filtro","waiConditionSelectTemplate":"Condizione per la regola ${0}","waiFilterButton":"Inoltra il filtro","statusTipTitle":"Barra filtro","statusTipRelPre":"Soddisfa","anyColumnOption":"Qualunque colonna","conditionSelectLabel":"Condizione","cancelButton":"Annulla","conditionNotStartWith":"non inizia con","conditionEndWith":"termina con","statusTipTitleNoFilter":"Barra filtro","columnSelectLabel":"Colonna","conditionNotContain":"non contiene","clearButton":"Cancella","conditionLargerEqual":"maggiore o uguale a","any":"qualunque","relationMsgFront":"Soddisfa","statusTipHeaderColumn":"Colonna","defaultItemsName":"elementi","conditionEqual":"uguale","relationAny":"qualunque regola","trueLabelEditable":"Selezionato","or":"o","filterBarMsgHasFilterTemplate":"${0} di ${1} ${2} visualizzati.","rangeTo":"a","conditionStartsWith":"inizia con","waiRelAny":"Soddisfa una delle seguenti regole:","statusTipRelPost":"regole.","relationMsgTail":"","filterBarClearButton":"Cancella filtro","incompleteRuleTip":"Questa regola non è completa.","conditionLessEqual":"minore o uguale a","waiFilterBarClearButton":"Cancella il filtro","conditionAfter":"dopo","statusTipHeaderCondition":"Regole","falseLabel":"False","filterButton":"Filtro","clearFilterDialogTitle":"Cancella filtro","statusTipTitleHasFilter":"Filtro","addRuleButton":"Aggiungi regola","waiAddRuleButton":"Aggiungi una nuova regola","waiClearButton":"Cancella il filtro","conditionNotEqual":"diverso da","conditionNotEndWith":"non termina con","rangeTemplate":"da ${0} a ${1}","removeRuleButton":"Rimuovi regola","waiFilterBarDefButton":"Filtra la tabella","conditionIsEmpty":"is empty"})