<table class="Status">
    <tr>
        <td class="Time"><span dojoAttachPoint="timeNode">0.00</span></td>
        <td class="Status"><div dojoAttachPoint="titleNode">Loading...</div></td>
        <td class="Duration"><span dojoAttachPoint="durNode">0.00</span></td>
    </tr>
</table>
