({"set":"‏ادراج‏","text":"الوصف","insertImageTitle":"ادراج صورة","invalidMessage":"نوع ملف صور غير صحيح","prePopuTextBrowse":" أو تصفح الى ملف محلي.","browse":"استعراض...","prePopuTextUrl":"ادخل عنوان URL لصورة","url":"صورة"})