({"replaceDialogText":"التكرارات ${0} المستبدلة.","eofDialogTextFind":"موجودة","eofDialogText":"آخر تكرار ${0}","backwards":"الى الوراء","replaceButton":"استبدال","replaceLabel":"استبدال بالآتي:","matchCase":"مطابقة حالة الحروف","findTooltip":"ادخال نص لايجاد","replaceTooltip":"ادخال نص للاستبدال مع","replaceAllButton":"استبدال كل","eofDialogTextReplace":"مستبدل","findReplace":"ايجاد واستبدال","backwardsTooltip":"البحث الى الوراء عن نص","replaceAllButtonTooltip":"استبدال كل النص","replaceButtonTooltip":"استبدال النص","findLabel":"ايجاد:","findButton":"ايجاد","matchCaseTooltip":"مطابقة حالة الحروف","findButtonTooltip":"ايجاد النص","replaceAll":"كل التكرارات"})