({"valueLabel":"h","hueLabel":"t","saturationLabel":"i","saturationPickerTitle":"Intensiteit selecteren","huePickerTitle":"Tint selecteren","blueLabel":"b","greenLabel":"g","redLabel":"r","hexLabel":"hex","degLabel":"°"})