({"selectContents":"İçindekileri seç","deleteElement":"Öğeyi sil","moveEnd":"İmleci sona taşı","deleteContents":"İçindekileri sil","nodeActions":"${nodeName} Eylemleri","selectElement":"Öğeyi seç","moveStart":"İmleci başa taşı"})