.dojoxGridRtl .dojoxGridSortBtnSingle {
	right: auto;
	left: 0px;
}
.dojoxGridRtl .dojoxGridSortBtnNested {
	right: auto;
	left: 10px;
}

.dojoxGridRtl .dojoxGridCellOver  .dojoxGridSortBtn,
.dj_ie6 dojoxGridRtl .dojoxGridCellOver .dojoxGridSortBtn,
.dojoxGridRtl .dojoxGridCellSortFocus .dojoxGridSortBtn {
	border-left: none;
	border-right: 1px solid #999999;
}

/*Control the title node width*/
.dijitRtl .dojoxGridCellOver .dojoxGridSortNode,
.dijitRtl .dojoxGridCellSortFocus .dojoxGridSortNode,
.dijitRtl .dojoxGridSingleSorted .dojoxGridHeader .dojoxGridCellOver .dojoxGridSortNodeMain,
.dijitRtl .dojoxGridSingleSorted .dojoxGridHeader .dojoxGridCellSortFocus .dojoxGridSortNodeMain {
	margin-right: auto;
	margin-left: 10px;
}
.dijitRtl .dojoxGridSorted .dojoxGridCellOver .dojoxGridSortNode,
.dijitRtl .dojoxGridSorted .dojoxGridCellSortFocus .dojoxGridSortNode {
	margin-right: auto;
	margin-left: 24px;
}
/*html > body used for !dj_ie6, TBD - a better way*/
html.dijitRtl > body .dojoxGridCellOver .dojoxGridSortNode .dojoxGridSortBtnSingle {
	right: auto;
	left: -10px;
}
html.dijitRtl > body .dojoxGridCellSortFocus .dojoxGridSortNode .dojoxGridSortBtnSingle,
html.dijitRtl > body .dojoxGridSorted .dojoxGridCellOver .dojoxGridSortNodeMain .dojoxGridSortBtnSingle,
html.dijitRtl > body .dojoxGridSorted .dojoxGridCellSortFocus .dojoxGridSortNodeMain .dojoxGridSortBtnSingle {
	right: auto;
	left: -10px;
}
html.dijitRtl > body .dojoxGridSorted .dojoxGridCellOver .dojoxGridSortBtnSingle,
html.dijitRtl > body .dojoxGridSorted .dojoxGridCellSortFocus .dojoxGridSortBtnSingle,
html.dijitRtl > body .dojoxGridNestSorted .dojoxGridCellOver .dojoxGridSortNodeMain .dojoxGridSortBtnSingle,
html.dijitRtl > body .dojoxGridNestSorted .dojoxGridCellSortFocus .dojoxGridSortNodeMain .dojoxGridSortBtnSingle {
	right: auto;
	left: -28px;
}
html.dijitRtl > body .dojoxGridSorted .dojoxGridCellOver .dojoxGridSortBtnNested,
html.dijitRtl > body .dojoxGridSorted .dojoxGridCellSortFocus .dojoxGridSortBtnNested {
	right: auto;
	left: -17px;
}
.dojoxGridRtl .dojoxGridCellOver .dojoxGridSortBtnNested,
.dojoxGridRtl .dojoxGridCellSortFocus .dojoxGridSortBtnNested {
	background-position: -159px 5px;
	padding-left:8px;
}
.dojoxGridRtl .dojoxGridCellOver .dojoxGridSortNodeAsc .dojoxGridSortBtnNested,
.dojoxGridRtl .dojoxGridCellSortFocus .dojoxGridSortNodeAsc .dojoxGridSortBtnNested {
	background-position: -139px 5px;
}

.dojoxGridRtl .dojoxGridCellOver .dojoxGridSortNodeDesc .dojoxGridSortBtnNested,
.dojoxGridRtl .dojoxGridCellSortFocus .dojoxGridSortNodeDesc .dojoxGridSortBtnNested {
	background-position: -179px 5px;
}
.dojoxGridRtl .dojoxGridCellOver .dojoxGridSortBtnNested,
.dojoxGridRtl .dojoxGridCellSortFocus .dojoxGridSortBtnNested {
	padding-left: 8px;
	padding-right: 0;
}

.dj_ie6 .dojoxGridRtl .dojoxGridCellOver .dojoxGridSortBtnNested,
.dj_ie6 .dojoxGridRtl .dojoxGridCellSortFocus .dojoxGridSortBtnNested,
.dj_ie6 .dojoxGridRtl .dojoxGridCellOver .dojoxGridSortBtnNested,
.dj_ie6 .dojoxGridRtl .dojoxGridCellSortFocus .dojoxGridSortBtnNested {
	padding-left:0px;
}
