({"saveLabel":"حفظ","saveSettingdialogTitle":"حفظ آلي","saveSettingdialogButtonOk":"تحديد الفترة الزمنية","saveMessageSuccess":"تم الحفظ في ${0}","saveSettingdialogButtonCancel":"الغاء","saveSettingdialogDescription":"تحديد الفترة الزمنية للحفظ الآلي","saveSettingLabelOn":"تحديد الفترة الزمنية للحفظ الآلي...","saveSettingdialogParamLabel":"دقيقة","saveSettingdialogParamName":"الفترة الزمنية للحفظ الآلي","saveMessageFail":"فشل في الحفظ في ${0}","saveSettingLabelOff":"ايقاف الحفظ الآلي"})