/* Baseline - a designer framework, Copyright (C) 2009 Stephane Curzi, ProjetUrbain.com, Creative Commons Attribution-Share Alike 3.0 License */
/* reset */
html,body,div,span,a,img,h1,h2,h3,h4,h5,h6,hgroup,p,dl,dialog,dt,dd,ol,ul,li,abbr,acronym,address,b,big,blockquote,cite,code,del,dfn,em,i,ins,kbd,pre,q,samp,tt,var,small,strong,sub,sup,object,iframe,form,fieldset,label,legend,table,caption,tbody,tfoot,thead,tr,th,td,article,aside,footer,header,nav,section,figure,menu,time,mark,audio,video{font-family:inherit;font-size:100%;font-weight:inherit;font-style:inherit;vertical-align:baseline;white-space:normal;text-align:left;margin:0;padding:0;border:0;outline:0;background:transparent}textarea{font-family:inherit;font-size:100%;font-weight:normal;font-style:normal;white-space:normal;text-align:left;margin:0;padding:0}article,aside,footer,header,nav,section,dialog,figure,hgroup,menu{display:block}h1,h2,h3,h4,h5,h6{font-size:100%;font-weight:normal}del,ins{text-decoration:none}ol,ul{list-style:none}nav ul{list-style-type:none}table{border-collapse:separate;border-spacing:0;background-color:transparent;width:auto;height:auto}:focus{outline:0}blockquote:before,blockquote:after,q:before,q:after{content:""}blockquote,q{quotes:"" ""}applet,basefont,dir,font,isindex,menu,s,strike,u{font-family:inherit;font-size:100%;font-weight:normal;font-style:normal;white-space:normal;vertical-align:baseline;text-decoration:inherit;text-align:left;color:inherit;margin:0;padding:0;border:0;outline:0}dir,menu{list-style:none}nobr{white-space:normal}blink{text-decoration:none}marquee{overflow:visible}
/* base */
body{font-family:helvetica,arial,sans-serif;line-height:1.5;background:white;color:black}h1,h2,h3,h4,h5,h6{line-height:1.2}h4,h5,h6{font-weight:bold}b,strong,caption,th,thead,dt,legend{font-weight:bold}cite,dfn,em,i{font-style:italic}code,kbd,samp,pre,tt,var{font-family:mono-space,monospace}h1,h2,h3,h4,h5,h6{word-spacing:-0.125em}p{word-spacing:0.125em;hyphenate:auto;hyphenate-lines:3}p+p{text-indent:1.5em}p+p.no-indent{text-indent:0}pre{white-space:pre}del{text-decoration:line-through}mark{background:rgba(255, 255, 0, 0.4);padding:0 .25em}ins{color:#f00}small,sup,sub{font-size:80%}big{font-size:125%;line-height:80%}abbr,acronym{font-size:85%;text-transform:uppercase;letter-spacing:.1em}abbr[title],acronym[title],dfn[title]{border-bottom:1px dotted black;cursor:help}sup,sub{line-height:0}sup{vertical-align:super}sub{vertical-align:sub}blockquote{padding:1.5em}hr{border:none;background:#ddd;width:100%}ul,ol{margin-left:1.5em}ul{list-style:disc outside}ol{list-style:decimal outside}input,select,button{cursor:pointer}table{font:inherit;width:100%}article,aside,header,hgroup,nav,section,footer{float:left;display:block}figure{display:block}.debug{outline:solid gold 1px}.debug-background{background:rgba(255, 215, 0, 0.2) !important}
/* type */
body{font-size:75%;line-height:1.5}h1,h2,h3,h4,h5,h6{position:relative}h1,h2{line-height:36px;margin-bottom:18px}h1,h2,h3,h4{margin-top:18px}h3,h4,h5,h6{line-height:18px}h1{font-size:36px;top:5px}h2{font-size:28px;top:8px}h3{font-size:22px;top:1px}h4{font-size:18px;top:2px}h5{font-size:15px;top:4px}h6{font-size:13px;top:5px}h1:first-child,h2:first-child,h3:first-child,h4:first-child{margin-top:0}p,pre,address{font-size:13px;line-height:18px;position:relative;top:5px}abbr,code,kbd,samp,small,var{line-height:15px}ul,ol,dl,dialog{font-size:13px;line-height:18px;position:relative;top:5px;margin-top:18px;margin-bottom:18px}li ul,li ol,ul ul,ol ol{top:0;margin-top:0;margin-bottom:0}li h1,li h2,li h3,li h4,li h5,li h6,li p{top:0}form,legend,label{font-size:13px;line-height:18px}legend{position:relative;top:5px}input,textarea{font-size:12px}table{font-size:13px}caption{font-size:13px;line-height:18px;position:relative}hr{position:relative;height:4px;margin:18px 0 14px 0}
/* table */
table{border-top:solid 3px #000;position:relative;top:-3px;margin-top:18px;padding-bottom:15px}th,td{border-top:solid 1px #000;line-height:18px;padding:9px 18px 8px 0;position:relative}th:first-child,td:first-child{padding-left:0}th:last-child,td:last-child{padding-right:0}tr:nth-child(even) td{}tr:nth-child(odd) td{background:#f6f6f6}caption{top:8px;margin-bottom:18px}
/* form */
form{overflow:auto}legend{padding-bottom:18px}label{padding-top:5px;margin:0 0 11px 18px;float:left;display:block}label.first{margin-left:0}input[type="text"],input[type="password"],input[type="select"],input[type="search"]{border:solid 1px #ccc;height:16px;padding:1px 2px;display:block}input[type="file"]{border:none}textarea{border:solid 1px #ccc;height:16px;padding:0 2px;display:block;line-height:18px}select{display:block;margin-top:2px;margin-bottom:0}input[type="submit"],input[type="reset"],input[type="button"]{margin-left:12px;height:18px;margin-bottom:0}label.width1,label.width2,label.width3,label.width4{float:left}label.width1 input,label.width1 textarea{width:228px}label.width2 input,label.width2 textarea{width:480px}label.width3 input,label.width3 textarea{width:732px}label.width4 input,label.width4 textarea{width:984px}label.width1 select,label.width2 select,label.width3 select,label.width4 select{width:100%}label.unitx1,label.unitx2,label.unitx3,label.unitx4,label.unitx5,label.unitx6,label.unitx7,label.unitx8{float:left}label.unitx1 input,label.unitx1 textarea{width:102px}label.unitx2 input,label.unitx2 textarea{width:228px}label.unitx3 input,label.unitx3 textarea{width:354px}label.unitx4 input,label.unitx4 textarea{width:480px}label.unitx5 input,label.unitx5 textarea{width:606px}label.unitx6 input,label.unitx6 textarea{width:732px}label.unitx7 input,label.unitx7 textarea{width:858px}label.unitx8 input,label.unitx8 textarea{width:984px}label.unitx1 select,label.unitx2 select,label.unitx3 select,label.unitx4 select,label.unitx5 select,label.unitx6 select,label.unitx7 select,label.unitx8 select{width:100%}label.singleline{height:32px;margin-bottom:0}label.singleline input{width:auto;display:inline;margin-left:12px;margin-top:0}label.multilinex2 textarea{height:36px}label.multilinex3 textarea{height:54px}label.multilinex4 textarea{height:72px}label.multilinex5 textarea{height:90px}label.multilinex6 textarea{height:108px}.width1 .radio-text{width:228px}.width2.radio-text{width:480px}.width3.radio-text{width:732px}.width4.radio-text{width:984px}.radio{margin:6px 0 0 1px;float:left;display:block;clear:left}.radio-label{margin:0 0 13px 0;padding-left:18px;float:none;display:block;clear:none}.checkbox{margin:6px 0 0 1px;float:left;display:block;clear:left}.checkbox-label{margin:0 0 13px 0;padding-left:18px;float:none;display:block;clear:none}
/* grid */
#page{width:990px;position:relative}#page:after{content:".";display:block;height:0;clear:both;visibility:hidden}.column{margin-left:18px;display:block;float:left}.colgroup{display:block;float:left}.first{margin-left:0;clear:left}.gutter{margin-left:18px}.no-gutter{margin-left:0}.align-left{float:left}.align-right{float:right;text-align:right}header,section,div{padding-bottom:18px}.leading{margin-bottom:18px}.noleading{margin-bottom:0 !important}.width1{width:234px}.width2{width:486px}.width3{width:738px}.width4{width:990px}.full{width:100%}.unitx1{width:108px}.unitx2{width:234px}.unitx3{width:360px}.unitx4{width:486px}.unitx5{width:612px}.unitx6{width:738px}.unitx7{width:864px}.unitx8{width:990px}.columnsx2{-webkit-column-count:2;-webkit-column-gap:18px;-moz-column-count:2;-moz-column-gap:18px;column-count:2;column-gap:18px}.columnsx4{-webkit-column-count:4;-webkit-column-gap:18px;-moz-column-count:4;-moz-column-gap:18px;column-count:4;column-gap:18px}.table-column th,.table-column td{width:234px}.table-column th.width2,.table-column td.width2{width:486px}.table-column th.width3,.table-column td.width3{width:738px}.table-column th.width4,.table-column td.width4{width:990px}.table-unit th,.table-unit td{width:108px}.table-unit th.unitx2,.table-unit td.unitx2{width:234px}.table-unit th.unitx3,.table-unit td.unitx3{width:360px}.table-unit th.unitx4,.table-unit td.unitx4{width:486px}.table-unit th.unitx5,.table-unit td.unitx5{width:612px}.table-unit th.unitx6,.table-unit td.unitx6{width:738px}.table-unit th.unitx7,.table-unit td.unitx7{width:864px}.table-unit th.unitx8,.table-unit td.unitx8{width:990px}