({"conditionBefore":"前","filterBarDefButton":"定义过滤器","filterBarMsgNoFilterTemplate":"未应用过滤器","conditionContains":"包含","clearFilterMsg":"将除去过滤器并显示所有可用记录。","all":"所有","waiValueBoxTemplate":"为 ${0} 规则向过滤器输入值","conditionRange":"范围","waiCancelButton":"取消此对话","relationAll":"所有规则","conditionLarger":"大于","valueBoxLabel":"值","statusTipMsg":"在此处单击过滤器栏以根据 ${0} 中的值进行过滤。","waiRelAll":"与以下所有规则匹配：","closeFilterBarBtn":"关闭过滤器栏","a11yFilterBarDefButton":"过滤器...","trueLabel":"True","conditionIsNot":"不是","conditionIs":"是","ruleTitleTemplate":"规则 ${0}","waiRemoveRuleButtonTemplate":"除去规则 ${0}","and":"和","conditionLess":"小于","waiColumnSelectTemplate":"${0} 规则列","anycolumn":"任意列","filterDefDialogTitle":"过滤器","waiConditionSelectTemplate":"${0} 规则条件","waiFilterButton":"提交过滤器","statusTipTitle":"过滤器栏","statusTipRelPre":"匹配","anyColumnOption":"任意列","conditionSelectLabel":"条件","cancelButton":"取消","conditionNotStartWith":"不是开始于","conditionEndWith":"结束于","statusTipTitleNoFilter":"过滤器栏","columnSelectLabel":"列","conditionNotContain":"不包含","clearButton":"清除","conditionLargerEqual":"大于等于","any":"任意","relationMsgFront":"匹配","statusTipHeaderColumn":"列","defaultItemsName":"项","conditionEqual":"等于","relationAny":"任意规则","trueLabelEditable":"已检查","or":"或","filterBarMsgHasFilterTemplate":"${0} of ${1} ${2} shown.","rangeTo":"到","conditionStartsWith":"开始于","waiRelAny":"与以下任意规则匹配：","statusTipRelPost":"规则。","relationMsgTail":"","filterBarClearButton":"清除过滤器","incompleteRuleTip":"此规则不完整。","conditionLessEqual":"小于等于","waiFilterBarClearButton":"清除过滤器","conditionAfter":"后","statusTipHeaderCondition":"规则","falseLabel":"False","filterButton":"过滤器","clearFilterDialogTitle":"清除过滤器","statusTipTitleHasFilter":"过滤器","addRuleButton":"添加规则","waiAddRuleButton":"新增规则","waiClearButton":"清除过滤器","conditionNotEqual":"不等于","conditionNotEndWith":"不是结束于","rangeTemplate":"从 ${0} 到 ${1}","removeRuleButton":"除去规则","waiFilterBarDefButton":"过滤表","conditionIsEmpty":"is empty"})