<div style="border:1px solid black;">

	<select dojoType="dijit.form.ComboBox" placeholder="functionName" dojoAttachPoint='combo' style="width:45%;" class="dojoxCalcFuncGenNameBox" dojoAttachEvent='onChange:onSelect'></select>

	<input dojoType="dijit.form.TextBox" placeholder="arguments" class="dojoxCalcFuncGenTextBox" style="width:50%;" dojoAttachPoint='args' />
	<BR>
	<TEXTAREA dojoType="dijit.form.SimpleTextarea" placeholder="function body" class="dojoxCalcFuncGenTextArea" style="text-align:left;width:95%;" rows=10 dojoAttachPoint='textarea' value="" dojoAttachEvent='onClick:readyStatus'></TEXTAREA>
	<BR>
	<input dojoType="dijit.form.Button" class="dojoxCalcFuncGenSave" dojoAttachPoint='saveButton' label="Save" dojoAttachEvent='onClick:onSaved' />
	<input dojoType="dijit.form.Button" class="dojoxCalcFuncGenReset" dojoAttachPoint='resetButton' label="Reset" dojoAttachEvent='onClick:onReset' />
	<input dojoType="dijit.form.Button" class="dojoxCalcFuncGenClear" dojoAttachPoint='clearButton' label="Clear" dojoAttachEvent='onClick:onClear' />
	<input dojoType="dijit.form.Button" class="dojoxCalcFuncGenClose" dojoAttachPoint='closeButton' label="Close" />
	<BR><BR>
	<input dojoType="dijit.form.Button" class="dojoxCalcFuncGenDelete" dojoAttachPoint='deleteButton' label="Delete" dojoAttachEvent='onClick:onDelete' />
	<BR>
	<input dojoType="dijit.form.TextBox" style="width:45%;" dojoAttachPoint='status' class="dojoxCalcFuncGenStatusTextBox" readonly value="Ready" />
</div>
