({"cancel":"Cancel","toDic":"Add to dictionary","skip":"Skip","iMsg":"No spelling suggestions","replaceAll":"Replace All","widgetLabel":"Batch Spell Check","skipAll":"Skip All","suggestions":"Suggestions","iSkipAll":"Skip all like this","replaceWith":"Replace with","msg":"No misspellings found","replace":"Replace","unfound":"Not found","iSkip":"Skip this"})