({"saveLabel":"Сақтау","saveSettingdialogTitle":"Автосақтау","saveSettingdialogButtonOk":"Аралықты орнату","saveMessageSuccess":"${0} сақталды","saveSettingdialogButtonCancel":"Болдырмау","saveSettingdialogDescription":"Автосақтау аралығын көрсету","saveSettingLabelOn":"Автосақтау аралығын орнату...","saveSettingdialogParamLabel":"мин","saveSettingdialogParamName":"Автосақтау аралығы","saveMessageFail":"${0} сақталмады","saveSettingLabelOff":"Автосақтауды өшіру"})