({"replaceDialogText":"${0} дана ауыстырылды.","eofDialogTextFind":"табылды","eofDialogText":"Соңғы дана: ${0}","backwards":"Артқа қарай","replaceButton":"Ауыстыру","replaceLabel":"Келесімен ауыстыру:","matchCase":"Үлкен-кішілігін ескеріп","findTooltip":"Табылатын мәтінді енгізу","replaceTooltip":"Ауыстырылатын мәтінді енгізу","replaceAllButton":"Барлығын ауыстыру","eofDialogTextReplace":"ауыстырылды","findReplace":"Табу және ауыстыру","backwardsTooltip":"Мәтінді табу үшін артқа қарай іздеу","replaceAllButtonTooltip":"Барлық мәтінді ауыстыру","replaceButtonTooltip":"Мәтінді ауыстыру","findLabel":"Табу:","findButton":"Табу","matchCaseTooltip":"Үлкен-кішілігін ескеріп","findButtonTooltip":"Мәтінді табу","replaceAll":"Барлық оқиғалар"})