({"selectContents":"Selecionar Conteúdo","deleteElement":"Excluir Elemento","moveEnd":"Mover Cursor para o Final","deleteContents":"Excluir Conteúdo","nodeActions":"${nodeName} Ações","selectElement":"Selecionar Elemento","moveStart":"Mover Cursor para o Início"})