({"replaceDialogText":"Zamenjanih ${0} pojavitev. ","eofDialogTextFind":"najdeno ","eofDialogText":"Zadnja pojavitev ${0}","backwards":"Nazaj","replaceButton":"Zamenjaj ","replaceLabel":"Zamenjaj z: ","matchCase":"Razlikuj velike in male črke","findTooltip":"Vnesite besedilo za iskanje","replaceTooltip":"Vnesite besedilo za zamenjavo ","replaceAllButton":"Zamenjaj vse","eofDialogTextReplace":"zamenjano ","findReplace":"Najdi in zamenjaj","backwardsTooltip":"Vzvratno iskanje besedila ","replaceAllButtonTooltip":"Zamenjaj celotno besedilo ","replaceButtonTooltip":"Zamenjaj besedilo ","findLabel":"Najdi:","findButton":"Najdi","matchCaseTooltip":"Razlikuj velike in male črke","findButtonTooltip":"Najdi besedilo ","replaceAll":"Vse pojavitve "})