.dijitFileInput {
	position:relative;
	height:1.3em;
	/*padding:2px;*/
}

.dijitFileInputReal {
	position:absolute;
	z-index:2;
	filter:alpha(opacity:0);
	opacity:0;
	cursor:pointer;
}
.dijitFileInputRealBlind {
	right:0;
}
.dijitFileInputReal:hover { cursor:pointer; } 

.dijitFileInputButton,
.dijitFileInputText {
	border:1px solid #333;
	padding:2px 12px 2px 12px; 
	cursor:pointer;
}

.dijitFileInputButton {
	z-index:3;
	visibility:hidden;
}
.dijitFakeInput { position:absolute; top:0; left:0; z-index:1; white-space: nowrap; }

.dijitProgressOverlay {
	display:none;
	width:250px;
	height:1em;
	position:absolute;
	top:0; left:0;
	border:1px solid #333;
	background:#cad2de url('../../../dijit/themes/tundra/images/dijitProgressBarAnim.gif') repeat-x top left;
	padding:2px;	
}

/* tundra */
.tundra .dijitProgressOverlay {
	border:1px solid #84a3d1;
	background-color:#cad2de;
}
.tundra .dijitFakeInput input {
	/*font-size: inherit;*/
	padding: 0;
	background:#fff url("../../../dijit/themes/tundra/images/validationInputBg.png") repeat-x top left;
	border:1px solid #9b9b9b;
	line-height: normal;
}
.tundra .dijitFileInputButton, 
.tundra .dijitFileInputText {
	border:1px solid #9b9b9b;
	padding:0px 12px 0px 12px; /* .3em .4em .2em .4em; */
	background:#e9e9e9 url("../../../dijit/themes/tundra/images/buttonEnabled.png") repeat-x top;
}

/* Soria */
.soria .dijitProgressOverlay {
	border:1px solid #8BA0BD;
	background-color:#cad2de; 
}
.soria .dijitFakeInput input {
	border:1px solid #8BA0BD;
	background:#fff url("../../../dijit/themes/soria/images/validationInputBg.png") repeat-x top left;
	line-height:normal;
	background-position:0 -30px; 
	padding:0.2em 0.3em;
}
.soria .dijitFileInputButton,
.soria .dijitFileInputText {
	border:1px solid #8BA0BD;
	padding:2px 12px 2px 12px;
	background:#b7cdee url('../../../dijit/themes/soria/images/buttonEnabled.png') repeat-x; 
}

/* Nihilo */
.nihilo .dijitProgressOverlay {
	border:1px solid #DEDEDE;
	background-color:#cad2de; 
}
.nihilo .dijitFakeInput input {
	border:1px solid #DEDEDE;
	background:#fff url("../../../dijit/themes/nihilo/images/validationInputBg.png") repeat-x top left;
	line-height:normal;
	background-position:0 -30px; 
	padding:0.2em 0.3em;
}
.nihilo .dijitFileInputButton,
.nihilo .dijitFileInputText {
	border:1px solid #DEDEDE;
	padding:2px 12px 2px 12px;
	background:#b7cdee url('../../../dijit/themes/nihilo/images/buttonEnabled.png') repeat-x; 
}

/* Claro */
.claro .dijitProgressOverlay {
	border:1px solid #769dc0;
	background-color:#769dc0; 
}
.claro .dijitFakeInput input {
	border: 1px solid #bcc8dd;
	background-color: #fff;
	background-repeat: repeat-x;
	background-position: top left;
	background-image:url("../../../dijit/themes/claro/form/images/textBox_back.png");
	line-height:normal;
	padding:0.2em 0.3em;
}

.claro .dijitFileInputButton,
.claro .dijitFileInputText {
	background-image: url("../../../dijit/themes/claro/form/images/button_back_full.png");
	background-position: center top;
	background-repeat: repeat-x;
	background-color: #cde3f6;
	border: 1px solid #799ab7;
	border-radius: 4px;
	-moz-border-radius: 4px;
	-webkit-border-radius: 4px;
	box-shadow:0px 1px 1px rgba(0,0,0,0.2);
	-webkit-box-shadow:0px 1px 1px rgba(0,0,0,0.2);
	-moz-box-shadow: 0px 1px 1px rgba(0,0,0,0.2);
}
