({"selectContents":"Seleccionar conteúdo","deleteElement":"Eliminar elemento","moveEnd":"Mover cursor para o fim","deleteContents":"Eliminar conteúdo","nodeActions":"Acções de ${nodeName}","selectElement":"Seleccionar elemento","moveStart":"Mover cursor para o início"})