({"conditionBefore":"før","filterBarDefButton":"Definer filter","filterBarMsgNoFilterTemplate":"Det er ikke brukt filter","conditionContains":"inneholder","clearFilterMsg":"Dette fjerner filteret og viser alle tilgjengelig poster.","all":"alle","waiValueBoxTemplate":"Angi verdi som skal filtreres for regel ${0}","conditionRange":"område","waiCancelButton":"Avbryt denne dialogboksen","relationAll":"alle regler","conditionLarger":"er større enn","valueBoxLabel":"Verdi","statusTipMsg":"Klikk på filterlinjen her for å filtrere på verdier i ${0}.","waiRelAll":"Samsvar med alle følgende regler:","closeFilterBarBtn":"Lukk filterlinje","a11yFilterBarDefButton":"Filtrer...","trueLabel":"Sann","conditionIsNot":"er ikke","conditionIs":"er","ruleTitleTemplate":"Regel ${0}","waiRemoveRuleButtonTemplate":"Fjern regel ${0}","and":"og","conditionLess":"er mindre enn","waiColumnSelectTemplate":"Kolonne for regel ${0}","anycolumn":"enhver kolonne","filterDefDialogTitle":"Filter","waiConditionSelectTemplate":"Betingelse for regel ${0}","waiFilterButton":"Send filteret","statusTipTitle":"Filterlinje","statusTipRelPre":"Samsvar","anyColumnOption":"Enhver kolonne","conditionSelectLabel":"Betingelse","cancelButton":"Avbryt","conditionNotStartWith":"begynner ikke med","conditionEndWith":"slutter med","statusTipTitleNoFilter":"Filterlinje","columnSelectLabel":"Kolonne","conditionNotContain":"inneholder ikke","clearButton":"Tøm","conditionLargerEqual":"større enn eller lik","any":"minst en","relationMsgFront":"Samsvar","statusTipHeaderColumn":"Kolonne","defaultItemsName":"elementer","conditionEqual":"er lik","relationAny":"minst en regel","trueLabelEditable":"Valgt","or":"eller","filterBarMsgHasFilterTemplate":"${0} av ${1} ${2} vist.","rangeTo":"til","conditionStartsWith":"begynner med","waiRelAny":"Samsvar med minst en av følgende regler:","statusTipRelPost":"regler.","relationMsgTail":"","filterBarClearButton":"Tøm filter","incompleteRuleTip":"Denne regelen er ikke fullstendig.","conditionLessEqual":"mindre enn eller lik","waiFilterBarClearButton":"Tøm filteret","conditionAfter":"etter","statusTipHeaderCondition":"Regler","falseLabel":"Usann","filterButton":"Filter","clearFilterDialogTitle":"Tøm filter","statusTipTitleHasFilter":"Filter","addRuleButton":"Legg til regel","waiAddRuleButton":"Legg til en ny regel","waiClearButton":"Tøm filteret","conditionNotEqual":"er ikke lik","conditionNotEndWith":"slutter ikke med","rangeTemplate":"fra ${0} til ${1}","removeRuleButton":"Fjern regel","waiFilterBarDefButton":"Filtrer tabellen","conditionIsEmpty":"is empty"})