<div class="dijitFileInput">
	<input id="${id}" name="${name}" class="dijitFileInputReal" type="file" dojoAttachPoint="fileInput" />
	<div class="dijitFakeInput" dojoAttachPoint="fakeNodeHolder">
		<input class="dijitFileInputVisible" type="text" dojoAttachPoint="focusNode, inputNode" />
		<div class="dijitInline dijitFileInputText" dojoAttachPoint="titleNode">${label}</div>
		<div class="dijitInline dijitFileInputButton" dojoAttachPoint="cancelNode" dojoAttachEvent="onclick:reset">${cancelText}</div>
	</div>
	<div class="dijitProgressOverlay" dojoAttachPoint="overlay">&nbsp;</div>
</div>
