({"set":"Vstavi ","text":"Opis ","insertImageTitle":"Vstavi sliko","invalidMessage":"Neveljavna vrsta slikovne datoteke ","prePopuTextBrowse":" ali prebrskajte in izberite lokalno datoteko. ","browse":"Prebrskaj ... ","prePopuTextUrl":"Vnesite URL slike","url":"Slika "})