.dojoxGridBorderDIV {
	width:2px;
	background-color: gray;
	font-size:0em;
	position:absolute;
	z-index:9999;
}
.dojoxGridCellBorderDIV {
	position:absolute;
	background-color: transparent;
	border: none;
}
.dojoxGridCellBorderLeftTopDIV {
	position: absolute;
	left: 0;
	top: 0;
	border-style: solid;
	border-width: 2px 0 0 2px;
	border-color: gray transparent transparent gray;
}
.dojoxGridCellBorderRightBottomDIV {
	position: absolute;
	right: 0;
	bottom: 0;
	border-style: solid;
	border-width: 0 2px 2px 0;
	border-color: transparent gray gray transparent;
}
.dojoxGridDnDItemIcon {
	background-image: url("images/sprite_icons.png");
}
.dojoxGridDnDIconRowSingle {
	background-position: -256px 5px;
}
.dojoxGridDnDIconRowMulti {
	background-position: -256px -16px;
}
.dojoxGridDnDIconColSingle {
	background-position: -277px 3px;
}
.dojoxGridDnDIconColMulti {
	background-position: -277px -17px;
}
.dojoxGridDnDIconCellSingle {
	background-position: -235px 5px;
}
.dojoxGridDnDIconCellMulti {
	background-position: -236px -16px;
}
.dojoxGridDndAvatar {
	background-color:white;
	border: 1px solid #CCCCCC;
	padding: 0px;
	-moz-box-shadow: 5px 5px 7px #999;
	-webkit-box-shadow: 5px 5px 7px #999;
	box-shadow: 5px 5px 7px #999;
	z-index: 999;
}
.dojoxGridDndAvatar td {
	padding: 3px;
}
.dojoxGridDnDIcon,
.dojoxGridDnDItemIcon {
	width: 16px;
	height: 16px;
}
.dojoDndMove .dojoxGridDnDIcon {
	background: url(../../../../dojo/resources/images/dndNoMove.png) no-repeat center center;
}
.dojoDndCopy .dojoxGridDnDIcon {
	background: url(../../../../dojo/resources/images/dndNoCopy.png) no-repeat center center;
}
.dojoDndMove .dojoDndAvatarCanDrop .dojoxGridDnDIcon {
	background: url(../../../../dojo/resources/images/dndMove.png) no-repeat center center;
}
.dojoDndCopy .dojoDndAvatarCanDrop .dojoxGridDnDIcon {
	background: url(../../../../dojo/resources/images/dndCopy.png) no-repeat center center;
}
