({"pasteFromWord":"Incolla da Word","paste":"Incolla","cancel":"Annulla","instructions":"Incolla il contenuto da Word nella casella di testo sottostante. Al termine dell'inserimento del contenuto, premere il pulsante Incolla. Per annullare l'inserimento del testo, premere il pulsante Annulla."})