({"emoticonLaughing":"Γέλιο","emoticonCool":"Άνετος","emoticonTongue":"Κοροϊδία","emoticonCrying":"Κλάμα","emoticonOops":"Έκπληξη","emoticonFrown":"Συνοφρυωμένος","emoticonAngry":"Θυμωμένος","emoticonShy":"Ντροπαλός","emoticonNo":"Όχι","emoticonAngel":"Αγγελούδι","smiley":"Εισαγωγή εικονιδίου συναισθήματος","emoticonIdea":"Ιδέα","emoticonEyebrow":"Σηκωμένο φρύδι","emoticonSmile":"Χαμόγελο","emoticonWink":"Κλείσιμο ματιού","emoticonYes":"Ναι","emoticonGrin":"Πλατύ χαμόγελο","emoticonGoofy":"Χαζόφατσα","emoticonHalf":"Μισό"})