({"conditionBefore":"vor","filterBarDefButton":"Filter definieren","filterBarMsgNoFilterTemplate":"Kein Filter angewendet","conditionContains":"enthält","clearFilterMsg":"Dadurch wird der Filter gelöscht und alle verfügbaren Einträge werden angezeigt. ","all":"alle","waiValueBoxTemplate":"Wert zum Filtern für Regel ${0} eingeben","conditionRange":"Bereich","waiCancelButton":"Dieses Dialogfenster abbrechen","relationAll":"alle Regeln","conditionLarger":"größer als","valueBoxLabel":"Wert","statusTipMsg":"Klicken Sie hier auf die Filterleiste, um die Werte in ${0} zu filtern.","waiRelAll":"Übereinstimmung mit allen folgenden Regeln: ","closeFilterBarBtn":"Filterleiste schließen","a11yFilterBarDefButton":"Filtern...","trueLabel":"Wahr","conditionIsNot":"ist nicht","conditionIs":"ist","ruleTitleTemplate":"Regel ${0}","waiRemoveRuleButtonTemplate":"Regel ${0} entfernen","and":"und","conditionLess":"kleiner als","waiColumnSelectTemplate":"Spalte für Regel ${0}","anycolumn":"eine Spalte","filterDefDialogTitle":"Filter","waiConditionSelectTemplate":"Bedingung für Regel ${0}","waiFilterButton":"Filter übergeben","statusTipTitle":"Filterleiste","statusTipRelPre":"Übereinstimmung","anyColumnOption":"Eine Spalte","conditionSelectLabel":"Bedingung ","cancelButton":"Abbrechen","conditionNotStartWith":"beginnt nicht mit","conditionEndWith":"endet mit","statusTipTitleNoFilter":"Filterleiste","columnSelectLabel":"Spalte","conditionNotContain":"enthält nicht","clearButton":"Löschen","conditionLargerEqual":"größer-gleich","any":"ein(e/r)","relationMsgFront":"Übereinstimmung","statusTipHeaderColumn":"Spalte","defaultItemsName":"Elemente ","conditionEqual":"gleich","relationAny":"eine Regel","trueLabelEditable":"Ausgewählt","or":"oder","filterBarMsgHasFilterTemplate":"${0} von ${1} ${2} angezeigt. ","rangeTo":"bis","conditionStartsWith":"beginnt mit","waiRelAny":"Übereinstimmung mit einer der folgenden Regeln: ","statusTipRelPost":"Regeln.","relationMsgTail":"","filterBarClearButton":"Filter löschen","incompleteRuleTip":"Diese Regel ist nicht vollständig. ","conditionLessEqual":"kleiner-gleich","waiFilterBarClearButton":"Filter löschen","conditionAfter":"nach","statusTipHeaderCondition":"Regeln","falseLabel":"Falsch","filterButton":"Filter","clearFilterDialogTitle":"Filter löschen","statusTipTitleHasFilter":"Filter","addRuleButton":"Regel hinzufügen","waiAddRuleButton":"Neue Regel hinzufügen","waiClearButton":"Filter löschen","conditionNotEqual":"ungleich","conditionNotEndWith":"endet nicht mit","rangeTemplate":"von ${0} bis ${1}","removeRuleButton":"Regel entfernen","waiFilterBarDefButton":"Tabelle filtern","conditionIsEmpty":"is empty"})