({"gotoButtonTitle":"特定のページに移動","prevTip":"前のページ","pageCountIndication":" (${0} ページ)","pageStepLabelTemplate":"ページ ${0}","descTemplate":"合計 ${1} ${0} 件の結果のうち、${2} から ${3} 件目を表示しています","lastTip":"最後のページ","nextTip":"次のページ","pageSizeLabelTemplate":"${0} 個の項目をページに表示","dialogCancel":"キャンセル","itemTitle":"項目","dialogConfirm":"実行","firstTip":"最初のページ","allItemsLabelTemplate":"すべての項目","dialogIndication":"ページ番号の指定","dialogTitle":"ページに移動"})