({"set":"Εισαγωγή","text":"Περιγραφή","insertImageTitle":"Εισαγωγή εικόνας","invalidMessage":"Μη έγκυρο είδος αρχείου εικόνας","prePopuTextBrowse":" ή επιλέξτε ένα τοπικό αρχείο.","browse":"Αναζήτηση...","prePopuTextUrl":"Καταχωρήστε τη διεύθυνση URL μιας εικόνας","url":"Εικόνα"})