({"emoticonLaughing":"大笑","emoticonCool":"酷","emoticonTongue":"吐舌頭","emoticonCrying":"哭泣","emoticonOops":"哎喲","emoticonFrown":"皺眉","emoticonAngry":"生氣","emoticonShy":"羞怯","emoticonNo":"不對","emoticonAngel":"守護神","smiley":"插入表情符號","emoticonIdea":"思考","emoticonEyebrow":"揚眉","emoticonSmile":"微笑","emoticonWink":"眨眼","emoticonYes":"對","emoticonGrin":"露齒而笑","emoticonGoofy":"傻了","emoticonHalf":"左右為難"})