<div class="dojoxCal-MY-labels" style="left: 0px;"	
	dojoAttachPoint="myContainer" dojoAttachEvent="onclick: onClick">
		<table cellspacing="0" cellpadding="0" border="0" style="margin: auto;">
				<tbody>
						<tr class="dojoxCal-MY-G-Template">
								<td class="dojoxCal-MY-M-Template">
										<div class="dojoxCalendarMonthLabel"></div>
								</td>
								<td class="dojoxCal-MY-M-Template">
										<div class="dojoxCalendarMonthLabel"></div>
								</td>
								<td class="dojoxCal-MY-Y-Template">
										<div class="dojoxCalendarYearLabel"></div>
								</td>
								<td class="dojoxCal-MY-Y-Template">
										<div class="dojoxCalendarYearLabel"></div>
								</td>
						 </tr>
						 <tr class="dojoxCal-MY-btns">
						 	 <td class="dojoxCal-MY-btns" colspan="4">
						 		 <span class="dijitReset dijitInline dijitButtonNode ok-btn" dojoAttachEvent="onclick: onOk" dojoAttachPoint="okBtn">
						 	 	 	 <button	class="dijitReset dijitStretch dijitButtonContents">OK</button>
								 </span>
								 <span class="dijitReset dijitInline dijitButtonNode cancel-btn" dojoAttachEvent="onclick: onCancel" dojoAttachPoint="cancelBtn">
						 	 		 <button	class="dijitReset dijitStretch dijitButtonContents">Cancel</button>
								 </span>
						 	 </td>
						 </tr>
				</tbody>
		</table>
</div>
