({"descending":"Aflopend","ascending":"Oplopend","nestedSort":"Geneste sorteerbewerking","unsorted":"Deze kolom niet sorteren","singleSort":"Enkele sorteerbewerking","indirectSelectionRadio":"Row ${0}, single selection, radio box","indirectSelectionCheckBox":"Row ${0}, multiple selection, check box","selectAll":"Select all","sortingState":"${0} - ${1}"})