<script type="text/javascript">
    //<![CDATA[
    // Injected by Django Skylark
    var $CF = {
        MEDIA_URL: "{{ cache_url }}{{ prepared_instructions.cache_prefix }}",
        INSTRUMENTED: {{ is_instrumented|lower }}
    }
    //]]>
</script>
