({"gotoButtonTitle":"Siirry tietylle sivulle","prevTip":"Edellinen sivu","pageCountIndication":" (${0} sivua)","pageStepLabelTemplate":"Sivu ${0}","descTemplate":"${2} - ${3} / ${1} ${0}","lastTip":"Viimeinen sivu","nextTip":"Seuraava sivu","pageSizeLabelTemplate":"${0} kohdetta sivulla","dialogCancel":"Peruuta","itemTitle":"kohdetta","dialogConfirm":"Siirry","firstTip":"Ensimmäinen sivu","allItemsLabelTemplate":"Kaikki kohteet","dialogIndication":"Määritä sivunumero","dialogTitle":"Siirry sivulle"})