({"gotoButtonTitle":"Vai a una pagina specifica","prevTip":"Pagina precedente","pageCountIndication":" (${0} pagine)","pageStepLabelTemplate":"Pagina ${0}","descTemplate":"${2} - ${3} di ${1} ${0}","lastTip":"Ultima pagina","nextTip":"Pagina successiva","pageSizeLabelTemplate":"${0} elementi per pagina","dialogCancel":"Annulla","itemTitle":"elementi","dialogConfirm":"Vai","firstTip":"Prima pagina","allItemsLabelTemplate":"Tutti gli elementi","dialogIndication":"Specificare il numero di pagina","dialogTitle":"Vai a pagina"})