({"saveLabel":"Save","saveSettingdialogTitle":"Auto-Save","saveSettingdialogButtonOk":"Set Interval","saveMessageSuccess":"Saved at ${0}","saveSettingdialogButtonCancel":"Cancel","saveSettingdialogDescription":"Specify Auto-Save interval","saveSettingLabelOn":"Set Auto-Save Interval...","saveSettingdialogParamLabel":"min","saveSettingdialogParamName":"Auto-Save Interval","saveMessageFail":"Failed to save at ${0}","saveSettingLabelOff":"Turn off Auto-Save"})