({"pasteFromWord":"Vložit z aplikace Word","paste":"Vložit","cancel":"Storno","instructions":"Vložte obsah z aplikace Word do zobrazeného textového pole. Jakmile jste s vkládaným obsahem spokojeni, stiskněte tlačítko Vložit. Pomocí tlačítka Storno vkládání zrušíte."})