({"saturationPickerTitle":"Επιλογή κορεσμού","valueLabel":"τ","blueLabel":"μ","saturationLabel":"κ","greenLabel":"π","redLabel":"κ","hueLabel":"α","hexLabel":"16-αδικό","huePickerTitle":"Επιλογή απόχρωσης","degLabel":"°"})