({"saveLabel":"Tallenna","saveSettingdialogTitle":"Automaattinen tallennus","saveSettingdialogButtonOk":"Aseta väli","saveMessageSuccess":"Tallennusaika ${0}","saveSettingdialogButtonCancel":"Peruuta","saveSettingdialogDescription":"Määritä automaattisen tallennuksen väli","saveSettingLabelOn":"Aseta automaattisen tallennuksen väli...","saveSettingdialogParamLabel":"min","saveSettingdialogParamName":"Automaattisen tallennuksen väli","saveMessageFail":"Tallennus epäonnistui ${0}","saveSettingLabelOff":"Ota automaattinen tallennus pois käytöstä"})