.feedViewerTable {
	font-size: small;
	width:100%;
	cursor:default;
	padding:2px;
}

.feedViewerTable, .feedViewerTable table {
	border-width: 0;
	margin: 0;
}

.feedViewerTableBody {
	overflow-x: auto;
	overflow-y: auto;
}
