.dijitEditorIconsFindReplace {
	background-image: url('../icons/editorIconsFindReplaceEnabled.png');
	background-repeat: no-repeat;
	width: 18px;
	height: 18px;
	text-align: center;
}

.dijitDisabled .dijitEditorIconsFindReplace {
	background-image: url('../icons/editorIconsFindReplaceDisabled.png');
	background-repeat: no-repeat;
	width: 18px;
	height: 18px;
	text-align: center;
}

.dijitEditorIconFindString { background-position: 0px; }

.dijitEditorIconFind { background-position: -18px; }

.dijitEditorIconReplace { background-position: -36px; }

.dijitEditorIconReplaceAll { background-position: -54px; }

.dijitEditorFindReplaceCheckBox {
	padding-left: 10px;
	padding-right: 10px;
	text-align: center;
}

.dijitEditorFindReplaceTextBox {
	padding-top: 3px;
	padding-bottom: 3px;
	padding-right: 5px;
}

.dijitEditorFindReplaceTextBox .focusTextBox {
	width: 13em;
}

.tundra .dijitEditorIconsFindReplaceClose {
        background-image: url('../icons/tundra/close.gif');
	background-repeat: no-repeat;
	width: 18px;
	height: 18px;
	background-position: center; 
}

.soria .dijitEditorIconsFindReplaceClose {
        background-image: url('../icons/soria/close.gif');
	background-repeat: no-repeat;
	width: 18px;
	height: 18px;
	background-position: center; 
}

.nihilo .dijitEditorIconsFindReplaceClose {
        background-image: url('../icons/nihilo/close.gif');
	background-repeat: no-repeat;
	width: 18px;
	height: 18px;
	background-position: center; 
}

.claro .dijitEditorIconsFindReplaceClose {
        background-image: url('../icons/claro/close.gif');
	background-repeat: no-repeat;
	width: 18px;
	height: 18px;
	background-position: center; 
}

