({"replaceDialogText":"Es wurden ${0} Vorkommen ersetzt.","eofDialogTextFind":"gefunden","eofDialogText":"Letztes Vorkommen ${0}","backwards":"Zurück","replaceButton":"Ersetzen","replaceLabel":"Ersetzen durch:","matchCase":"Groß- und Kleinschreibung abgleichen","findTooltip":"Text zum Suchen eingeben","replaceTooltip":"Text zum Ersetzen eingeben","replaceAllButton":"Global ersetzen","eofDialogTextReplace":"ersetzt","findReplace":"Suchen/Ersetzen","backwardsTooltip":"Rückwärts nach Text suchen","replaceAllButtonTooltip":"Gesamten Text ersetzen","replaceButtonTooltip":"Text ersetzen","findLabel":"Suchbegriff:","findButton":"Suchen","matchCaseTooltip":"Groß- und Kleinschreibung abgleichen","findButtonTooltip":"Text suchen","replaceAll":"Alle Vorkommen"})