<div class="dojoxCalendarYearLabels" style="left: 0px;" dojoAttachPoint="yearContainer">
    <table cellspacing="0" cellpadding="0" border="0" style="margin: auto;" dojoAttachEvent="onclick: onClick">
        <tbody>
            <tr class="dojoxCalendarYearGroupTemplate">
                <td class="dojoxCalendarNextMonth dojoxCalendarYearTemplate">
                    <div class="dojoxCalendarYearLabel">
                    </div>
                </td>
            </tr>
        </tbody>
    </table>
</div>
