({"set":"Indsæt","text":"Beskrivelse","insertImageTitle":"Indsæt billede","invalidMessage":"Ugyldig billedfiltype","prePopuTextBrowse":" eller søg efter en lokal fil.","browse":"Gennemse...","prePopuTextUrl":"Angiv en billed-URL","url":"Billede"})