<div class="Volume" dojoAttachEvent="mousedown:onShowVolume">
	<div class="VolumeSlider" dojoAttachPoint="volumeSlider">
    	<div class="VolumeSliderBack" dojoAttachPoint="volumeSliderBack"></div>
    	<div class="VolumeSliderHandle" dojoAttachPoint="handle" dojoAttachEvent="mousedown:startDrag, mouseup:endDrag, mouseover:handleOver, mouseout:handleOut"></div>	
    </div>
    <div class="icon"></div>
</div>