@import url("../RollingList/RollingList.css");

.dojoxFileInfoPane *{
	white-space: normal;
}

.dojoxFileInfoLabel {
	font-weight: bold;
	white-space: nowrap;
}

.dojoxFileInfoPane {
	width: 20em;
}

.tundra .dojoxFilePickerItem .dijitMenuItemIcon {
	background-image: url("images/tundraFileIcons.gif");
	background-repeat: no-repeat;
}
.tundra .dojoxFilePickerItem .dojoxEmpty {
	background-image: none;
}
.tundra .dojoxFilePickerItem .dojoxDirectoryItemIcon {
	background-position: 0px;
}
.tundra .dojoxFilePickerItemSelected .dojoxDirectoryItemIcon {
	background-position: -16px;
}
.tundra .dojoxFilePickerItem .dojoxFileItemIcon {
	background-position: -32px;
}
.tundra .dojoxFilePickerItemSelected .dojoxFileItemIcon {
	background-position: -48px;
}

.soria .dojoxFilePickerItem .dijitMenuItemIcon {
	background-image: url("images/soriaFileIcons.gif");
	background-repeat: no-repeat;
}
.soria .dojoxFilePickerItem .dojoxEmpty {
	background-image: none;
}
.soria .dojoxFilePickerItem .dojoxDirectoryItemIcon {
	background-position: 0px;
}
.soria .dojoxFilePickerItemSelected .dojoxDirectoryItemIcon {
	background-position: -16px;
}
.soria .dojoxFilePickerItem .dojoxFileItemIcon {
	background-position: -32px;
}
.soria .dojoxFilePickerItemSelected .dojoxFileItemIcon {
	background-position: -48px;
}


.nihilo .dojoxFilePickerItem .dijitMenuItemIcon {
	background-image: url("images/nihiloFileIcons.gif");
	background-repeat: no-repeat;
}
.nihilo .dojoxFilePickerItem .dojoxEmpty {
	background-image: none;
}
.nihilo .dojoxFilePickerItem .dojoxDirectoryItemIcon {
	background-position: 0px;
}
.nihilo .dojoxFilePickerItemSelected .dojoxDirectoryItemIcon {
	background-position: -16px;
}
.nihilo .dojoxFilePickerItem .dojoxFileItemIcon {
	background-position: -32px;
}
.nihilo .dojoxFilePickerItemSelected .dojoxFileItemIcon {
	background-position: -48px;
}

