({"emoticonLaughing":"หัวเราะ","emoticonCool":"เจ๋ง","emoticonTongue":"แลบลิ้น","emoticonCrying":"ร้องไห้","emoticonOops":"oops","emoticonFrown":"หน้าบึ้ง","emoticonAngry":"โกรธ","emoticonShy":"อาย","emoticonNo":"ไม่ใช่","emoticonAngel":"โมโห","smiley":"แทรกไอคอนแสดงอารมณ์","emoticonIdea":"ความคิด","emoticonEyebrow":"คิ้ว","emoticonSmile":"ยิ้ม","emoticonWink":"ขยิบตา","emoticonYes":"ใช่","emoticonGrin":"ยิ้มกว้าง","emoticonGoofy":"โง่","emoticonHalf":"ครึ่งซีก"})