/* Pygments autumn style */

code .comment {color: #aaa; font-style: italic}
code .comment.preproc {color: #4c8317; font-style: normal}
code .comment.special {color: #00a}

code .keyword {color: #00a}
code .keyword.type {color: #0aa}

code .operator.word {color: #00a}

code .name.builtin {color: #0aa}
code .name.function {color: #0a0}
code .name.class {color: #0a0; text-decoration: underline}
code .name.namespace {color: #0aa; text-decoration: underline}
code .name.exception {color: #D2413A; font-weight: bold}
code .name.variable {color: #a00}
code .name.constant {color: #a00}
code .name.entity {color: #800; font-weight: bold}
code .name.attribute {color: #1e90ff}
code .name.tag {color: #1e90ff; font-weight: bold}
code .name.decorator {color: #888}

code .string {color: #a50}
code .string.regex {color: #099}
code .string.symbol {color: #00a}

code .number {color: #099}
