<div class="dojoxGaugeIndicatorDiv">
	<label class="dojoxGaugeIndicatorLabel" for="${title}">${title}:</label>
	<input class="dojoxGaugeIndicatorInput" name="${title}" size="5" value="${value}" dojoAttachPoint="valueNode" dojoAttachEvent="onchange:_update"></input>
</div>