({"pasteFromWord":"Word бағдарламасынан қою","paste":"Қою","cancel":"Болдырмау","instructions":"Мазмұнды Word бағдарламасынан төмендегі мәтін ұясына қойыңыз.  Кірістірілетін мазмұн дұрыс болса, қою түймешігін басыңыз.  Мәтінді кірістіруді доғару үшін болдырмау түймешігін басыңыз."})