<div>
<div dojoAttachPoint="chartsParent" class="dojoxCalcChartHolder"></div>
<span dojoAttachPoint="outerDiv">
<div dojoType="dijit.form.DropDownButton" dojoAttachPoint="windowOptions" class="dojoxCalcDropDownForWindowOptions" title="Window Options">
	<div>Window Options</div>
	<div dojoType="dijit.TooltipDialog" dojoAttachPoint="windowOptionsInside" class="dojoxCalcTooltipDialogForWindowOptions" title="">
		<table class="dojoxCalcGraphOptionTable">
			<tr>
				<td>
					Width:
				</td>
				<td>
					<input dojoType="dijit.form.TextBox" dojoAttachPoint="graphWidth" class="dojoxCalcGraphWidth" value="500" />
				</td>
				<td>
					Height:
				</td>
				<td>
					<input dojoType="dijit.form.TextBox" dojoAttachPoint="graphHeight" class="dojoxCalcGraphHeight" value="500" />
				</td>
			</tr>
			<tr>
				<td>
					X >=
				</td>
				<td>
					<input dojoType="dijit.form.TextBox" dojoAttachPoint="graphMinX" class="dojoxCalcGraphMinX" value="-10" />
				</td>

				<td>
					X <=
				</td>
				<td>
					<input dojoType="dijit.form.TextBox" dojoAttachPoint="graphMaxX" class="dojoxCalcGraphMaxX" value="10" />
				</td>
			</tr>
			<tr>
				<td>
					Y >=
				</td>
				<td>
					<input dojoType="dijit.form.TextBox" dojoAttachPoint="graphMinY" class="dojoxCalcGraphMinY" value="-10" />
				</td>

				<td>
					Y <=
				</td>
				<td>
					<input dojoType="dijit.form.TextBox" dojoAttachPoint="graphMaxY" class="dojoxCalcGraphMaxY" value="10" />
				</td>
			</tr>
		</table>
	</div>
</div>

<BR>

<div class="dojoxCalcGrapherFuncOuterDiv">
	<table class="dojoxCalcGrapherFuncTable" dojoAttachPoint="graphTable">
	</table>
</div>

<div dojoType="dijit.form.DropDownButton" dojoAttachPoint='addFuncButton' class="dojoxCalcDropDownAddingFunction">
	<div>Add Function</div>
	<div dojoType="dijit.TooltipDialog" dojoAttachPoint="addFuncInside" class="dojoxCalcTooltipDialogAddingFunction" title="">
		<table class="dojoxCalcGrapherModeTable">
			<tr>
				<td>
					Mode:
				</td>
				<td>
					<select dojoType="dijit.form.Select" dojoAttachPoint="funcMode" class="dojoxCalcFunctionModeSelector">
						<option value="y=" selected="selected">y=</option>
						<option value="x=">x=</option>
					</select>
				</td>
				<td>
			</tr>
	
			<tr>
				<td>
					<input dojoType="dijit.form.Button" dojoAttachPoint="createFunc" class="dojoxCalcAddFunctionButton" label="Create" />
				</td>
			</tr>
		</table>
	</div>
</div>
<BR>
<BR>
<table class="dijitInline dojoxCalcGrapherLayout">
	<tr>
		<td class="dojoxCalcGrapherButtonContainer">
			<input dojoType="dijit.form.Button" class="dojoxCalcGrapherButton" dojoAttachPoint='selectAllButton' label="Select All" />
		</td>
		<td class="dojoxCalcGrapherButtonContainer">
			<input dojoType="dijit.form.Button" class="dojoxCalcGrapherButton" dojoAttachPoint='deselectAllButton' label="Deselect All" />
		</td>
	</tr>
	<tr>
		<td class="dojoxCalcGrapherButtonContainer">
			<input dojoType="dijit.form.Button" class="dojoxCalcGrapherButton" dojoAttachPoint='drawButton'label="Draw Selected" />
		</td>
		<td class="dojoxCalcGrapherButtonContainer">
			<input dojoType="dijit.form.Button" class="dojoxCalcGrapherButton" dojoAttachPoint='eraseButton' label="Erase Selected" />
		</td>
	</tr>
	<tr>
		<td class="dojoxCalcGrapherButtonContainer">
			<input dojoType="dijit.form.Button" class="dojoxCalcGrapherButton" dojoAttachPoint='deleteButton' label="Delete Selected" />
		</td>
		<td class="dojoxCalcGrapherButtonContainer">
			<input dojoType="dijit.form.Button" class="dojoxCalcGrapherButton" dojoAttachPoint='closeButton' label="Close" />
		</td>
	</tr>
</table>
</span>
</div>
