({"descending":"จากมากไปหาน้อย","ascending":"จากน้อยไปหามาก","nestedSort":"เรียงลำดับที่ซับซ้อน","unsorted":"ห้ามเรียงลำดับคอลัมน์นี้","singleSort":"เรียงลำดับแบบเดี่ยว","indirectSelectionRadio":"Row ${0}, single selection, radio box","indirectSelectionCheckBox":"Row ${0}, multiple selection, check box","selectAll":"Select all","sortingState":"${0} - ${1}"})