/* RadioGroup base Button styling: */
.dojoxRadioButtons {
	margin:0; padding:4px;
	width:100%;
	outline:0;
}

.dojoxRadioButtons tbody  {
	margin:0; padding:0;
	width:100%;
}

.dojoxRadioButton {
	text-align:center;
	cursor:pointer;
	margin:0;
}

.dojoxRadioButtonSelected {
	border:1px solid #ededed;
}
.tundra .dojoxRadioButtonSelected {
	border:1px solid #a0a0a0;
	background:#b7b7b7;
}
.soria .dojoxRadioButtonSelected {
	background:#b7cdee url('../../../dijit/themes/soria/images/gradientTopBg.png') repeat-x top center;
}

.dojoxRadioButtonLabel {
	padding:8px;
	text-align:center;
	display:block;
}

.dojoxRadioGroup {
	overflow:hidden;
	border:0;
	margin:0; padding:0;
}
.dojoxRadioView {
	position:relative;
	overflow:hidden;
	height:100%;
}