<html>
    <head>
        <script type="text/javascript">
    		djConfig = {forceGfxRenderer: "svg"};
    	</script>
        <script type="text/javascript" src="../../../dojo/dojo.js" ></script>
        <script type="text/javascript">
			dojo.require("dojox.gfx");
			dojo.require("dojox.gfx.utils");
        </script>
    </head>
    <body>
    </body>
</html>
