({"set":"Lisää","text":"Kuvaus","insertImageTitle":"Lisää kuva","invalidMessage":"Virheellinen kuvatiedoston laji","prePopuTextBrowse":" tai selaa paikalliseen tiedostoon.","browse":"Selaa...","prePopuTextUrl":"Anna kuvan URL-osoite","url":"Kuva"})