({"cancel":"Annullér","toDic":"Tilføj til ordbog","skip":"Spring over","iMsg":"Ingen forslag til stavning","replaceAll":"Erstat alle","widgetLabel":"Bundtvis stavekontrol","skipAll":"Spring alle over","suggestions":"Forslag","iSkipAll":"Spring alle disse over","replaceWith":"Erstat med","msg":"Ingen stavefejl fundet","replace":"Erstat","unfound":"Ikke fundet","iSkip":"Spring dette over"})