.tundra .dojoxGridSortBtn {
	padding-top: 3px;
}
.tundra .dojoxGridSortBtnNested {
	right: 11px;
}
.tundra .dojoxGridCellOver  .dojoxGridSortBtn,
.tundra .dojoxGridCellSortFocus  .dojoxGridSortBtn {
	background-color: #ccc;
}

/*Control the title node width*/
.tundra .dojoxGridCellOver .dojoxGridSortNode,
.tundra .dojoxGridCellSortFocus .dojoxGridSortNode,
.tundra .dojoxGridSingleSorted .dojoxGridHeader .dojoxGridCellOver .dojoxGridSortNodeMain,
.tundra .dojoxGridSingleSorted .dojoxGridHeader .dojoxGridCellSortFocus .dojoxGridSortNodeMain {
	margin-right: 12px;
}
.tundra .dojoxGridSorted .dojoxGridCellOver .dojoxGridSortNode,
.tundra .dojoxGridSorted .dojoxGridCellSortFocus .dojoxGridSortNode {
	margin-right: 31px;
}

/*ie 6 doesn't need below*/
.tundra .dojoxGridCellOver .dojoxGridSortNode .dojoxGridSortBtnSingle,
.tundra .dojoxGridCellSortFocus .dojoxGridSortNode .dojoxGridSortBtnSingle,
.tundra .dojoxGridSorted .dojoxGridCellOver .dojoxGridSortNodeMain .dojoxGridSortBtnSingle,
.tundra .dojoxGridSorted .dojoxGridCellSortFocus .dojoxGridSortNodeMain .dojoxGridSortBtnSingle  {
	right: -12px;
}
.tundra .dojoxGridSorted .dojoxGridCellOver .dojoxGridSortBtnSingle,
.tundra .dojoxGridSorted .dojoxGridCellSortFocus .dojoxGridSortBtnSingle,
.tundra .dojoxGridNestSorted .dojoxGridCellOver .dojoxGridSortNodeMain .dojoxGridSortBtnSingle,
.tundra .dojoxGridNestSorted .dojoxGridCellSortFocus .dojoxGridSortNodeMain .dojoxGridSortBtnSingle {
	right: -31px;
}
.tundra .dojoxGridSorted .dojoxGridCellOver .dojoxGridSortBtnNested,
.tundra .dojoxGridSorted .dojoxGridCellSortFocus .dojoxGridSortBtnNested {
	right: -20px;
}
/*ie6 hack end*/

/*Control background image*/
.tundra .dojoxGridSortNodeAsc .dojoxGridSortBtnSingle {
	background-position: -119px 8px;
}
.tundra .dojoxGridSortNodeDesc .dojoxGridSortBtnSingle {
	background-position: -99px  8px;
}
/*hover single sort*/
.tundra .dojoxGridCellOver .dojoxGridSortBtnSingle,
.tundra .dojoxGridCellSortFocus .dojoxGridSortBtnSingle {
	background-position: -159px 8px;
}
.tundra .dojoxGridCellOver .dojoxGridSortNodeAsc .dojoxGridSortBtnSingle,
.tundra .dojoxGridCellSortFocus .dojoxGridSortNodeAsc .dojoxGridSortBtnSingle,
.tundra .dojoxGrid .dojoxGridCellOver .dojoxGridSortNode .dojoxGridSortBtnAsc,
.tundra .dojoxGrid .dojoxGridCellSortFocus .dojoxGridSortNode .dojoxGridSortBtnAsc {
	background-position: -139px 8px;
}
.tundra .dojoxGridCellOver .dojoxGridSortNodeDesc .dojoxGridSortBtnSingle, 
.tundra .dojoxGridCellSortFocus .dojoxGridSortNodeDesc .dojoxGridSortBtnSingle, 
.tundra .dojoxGrid .dojoxGridCellOver .dojoxGridSortNode .dojoxGridSortBtnDesc,
.tundra .dojoxGrid .dojoxGridCellSortFocus .dojoxGridSortNode .dojoxGridSortBtnDesc {
	background-position: -179px 8px;
}
/*hover nested sort*/
.tundra .dojoxGridCellOver .dojoxGridSortBtnNested,
.tundra .dojoxGridCellSortFocus .dojoxGridSortBtnNested {
	background-position: -150px 8px;
}
.tundra .dojoxGridCellOver .dojoxGridSortNodeAsc .dojoxGridSortBtnNested,
.tundra .dojoxGridCellSortFocus .dojoxGridSortNodeAsc .dojoxGridSortBtnNested {
	background-position: -130px 8px;
}
.tundra .dojoxGridCellOver .dojoxGridSortNodeDesc .dojoxGridSortBtnNested,
.tundra .dojoxGridCellSortFocus .dojoxGridSortNodeDesc .dojoxGridSortBtnNested {
	background-position: -170px 8px;
}
.tundra .dojoxGridNestSorted .dojoxGridCellOver .dojoxGridSortBtnSingle,
.tundra .dojoxGridNestSorted .dojoxGridCellSortFocus .dojoxGridSortBtnSingle {
	background-position: -159px 8px;
}