<div hidefocus="hidefocus" role="grid" dojoAttachEvent="onmouseout:_mouseOut">
	<div class="dojoxGridMasterHeader" dojoAttachPoint="viewsHeaderNode" role="presentation"></div>
	<div class="dojoxGridMasterView" dojoAttachPoint="viewsNode" role="presentation"></div>
	<div class="dojoxGridMasterMessages" style="display: none;" dojoAttachPoint="messagesNode"></div>
	<span dojoAttachPoint="lastFocusNode" tabindex="0"></span>
</div>
