({"descending":"Descendent","ascending":"Ascendent","nestedSort":"Ordre imbricat","unsorted":"No ordenis aquesta finestra","singleSort":"Ordre únic","indirectSelectionRadio":"Row ${0}, single selection, radio box","indirectSelectionCheckBox":"Row ${0}, multiple selection, check box","selectAll":"Select all","sortingState":"${0} - ${1}"})