({"cancel":"Cancelar","toDic":"Adicionar ao dicionário","skip":"Ignorar","iMsg":"Sem sugestões de ortografia","replaceAll":"Substituir tudo","widgetLabel":"Verificação ortográfica por grupo","skipAll":"Ignorar tudo","suggestions":"Sugestões","iSkipAll":"Ignorar semelhantes","replaceWith":"Substituir por","msg":"Não forem encontrados erros ortográficos","replace":"Substituir","unfound":"Não localizado","iSkip":"Ignorar isto"})