/*
	Copyright (c) 2004-2011, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["dojox.charting.themes.Distinctive"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.themes.Distinctive"] = true;
dojo.provide("dojox.charting.themes.Distinctive");
dojo.require("dojox.charting.Theme");
(function(){
	var dxc=dojox.charting;
	dxc.themes.Distinctive=new dxc.Theme({
		colors: [
			"#497c91",
			"#ada9d6",
			"#768b4e",
			"#eeea99",
			"#b39c53",
			"#c28b69",
			"#815454",
			"#bebebe",
			"#59a0bd",
			"#c9c6e4",
			"#677e13",
			"#f0eebb",
			"#e9c756",
			"#cfb09b",
			"#a05a5a",
			"#d8d8d8",
			"#9dc7d9",
			"#7b78a4",
			"#a8c179",
			"#b7b35c",
			"#ebcf81",
			"#956649",
			"#c99999",
			"#868686",
			"#c7e0e9",
			"#8d88c7",
			"#c0d0a0",
			"#e8e667",
			"#efdeb0",
			"#b17044",
			"#ddc0c0",
			"#a5a5a5"
		
		]
	});
})();

}
