({"emoticonLaughing":"smeško z odprtimi usti","emoticonCool":"smeško je frajer","emoticonTongue":"smeško kaže jezik","emoticonCrying":"smeško joka","emoticonOops":"smeško ga je polomil","emoticonFrown":"smeško ni zadovoljen","emoticonAngry":"smeško je jezen","emoticonShy":"smeško je v zadregi","emoticonNo":"smeško odkimava","emoticonAngel":"smeško je angelček","smiley":"Vstavi čustveni simbol","emoticonIdea":"ideja","emoticonEyebrow":"smeško dviga obrv","emoticonSmile":"smeško","emoticonWink":"smeško pomežikne","emoticonYes":"smeško prikimava","emoticonGrin":"smeško se nasmehne do ušes","emoticonGoofy":"smeško se pači","emoticonHalf":"smeško se mršči"})