({"descending":"降冪","ascending":"遞增","nestedSort":"巢狀排序","unsorted":"本欄不進行排序","singleSort":"單一排序","indirectSelectionRadio":"Row ${0}, single selection, radio box","indirectSelectionCheckBox":"Row ${0}, multiple selection, check box","selectAll":"Select all","sortingState":"${0} - ${1}"})