.dojoxDocTester pre.testCase{
	border:0;
	padding:0.3em;
	padding-left:2em;
}
.dojoxDocTester pre.odd{
	background-color:#E3E3E3;
}
.dojoxDocTester pre.odd.resultOk{
	background-color:#C3FFC3;
}
.dojoxDocTester pre.odd.resultNok{
	background-color:#FFC3C3;
}

.dojoxDocTester pre.even{
        background-color:#F3F3F3;
}
.dojoxDocTester pre.even.resultOk{
	background-color:#DFFFDF;
}
.dojoxDocTester pre.even.resultNok{
	background-color:#FFCFCF;
}

.dojoxDocTester .actualResult{
	display:none;
	font-weight:bold;
}
.dojoxDocTester .actualResult .result{
	color:red;
}
