({"selectContents":"Välj innehåll","deleteElement":"Ta bort element","moveEnd":"Flytta markören till slutet","deleteContents":"Ta bort innehåll","nodeActions":"${nodeName}-åtgärder","selectElement":"Välj element","moveStart":"Flytta markören till början"})