({"saveLabel":"Guardar","saveSettingdialogTitle":"Guardado automático","saveSettingdialogButtonOk":"Definir intervalo","saveMessageSuccess":"Guardado a las ${0}","saveSettingdialogButtonCancel":"Cancelar","saveSettingdialogDescription":"Especificar intervalo de guardado automático","saveSettingLabelOn":"Definir intervalo de guardado automático...","saveSettingdialogParamLabel":"min","saveSettingdialogParamName":"Intervalo de guardado automático","saveMessageFail":"No se ha podido guardar a las ${0}","saveSettingLabelOff":"Desactivar guardado automático"})