({"saturationPickerTitle":"Selettore saturazione","huePickerTitle":"Selettore tonalità","valueLabel":"v","blueLabel":"b","saturationLabel":"s","greenLabel":"g","redLabel":"r","hueLabel":"h","hexLabel":"hex","degLabel":"°"})