({"conditionBefore":"次の値より前","filterBarDefButton":"フィルターの定義","filterBarMsgNoFilterTemplate":"フィルターが適用されていません","conditionContains":"次の値を含む","clearFilterMsg":"これにより、フィルターが削除され、使用可能なすべてのレコードが表示されます。","all":"すべて","waiValueBoxTemplate":"ルール ${0} のフィルターの値を入力","conditionRange":"次の範囲","waiCancelButton":"このダイアログのキャンセル","relationAll":"すべてのルール","conditionLarger":"次の値より大きい","valueBoxLabel":"値","statusTipMsg":"${0} の値をフィルターに掛けるには、ここのフィルター・バーをクリックしてください。","waiRelAll":"次のすべてのルールを突き合わせ:","closeFilterBarBtn":"フィルター・バーを閉じる","a11yFilterBarDefButton":"フィルター...","trueLabel":"True","conditionIsNot":" が  ではありません","conditionIs":"は","ruleTitleTemplate":"ルール ${0}","waiRemoveRuleButtonTemplate":"ルール ${0} の削除","and":" および ","conditionLess":"次の値より小さい","waiColumnSelectTemplate":"ルール ${0} の列","anycolumn":"任意の列","filterDefDialogTitle":"フィルター","waiConditionSelectTemplate":"ルール ${0} の条件","waiFilterButton":"フィルターの送信","statusTipTitle":"フィルター・バー","statusTipRelPre":"次を突き合わせ","anyColumnOption":"任意の列","conditionSelectLabel":"条件","cancelButton":"キャンセル","conditionNotStartWith":"次の値で始まらない","conditionEndWith":"次の値で終わる","statusTipTitleNoFilter":"フィルター・バー","columnSelectLabel":"列","conditionNotContain":"次の値を含まない","clearButton":"クリア","conditionLargerEqual":"次の値以上","any":"任意","relationMsgFront":"次を突き合わせ","statusTipHeaderColumn":"列","defaultItemsName":"項目","conditionEqual":"次の値と等しい","relationAny":"任意のルール","trueLabelEditable":"チェックマークが付いている","or":"または","filterBarMsgHasFilterTemplate":"${1} ${2} 中 ${0} が表示されています。","rangeTo":"比較基準","conditionStartsWith":"次の値で始まる","waiRelAny":"次の任意のルールを突き合わせ:","statusTipRelPost":"ルール。","relationMsgTail":"","filterBarClearButton":"フィルターのクリア","incompleteRuleTip":"このルールは未完成です。","conditionLessEqual":"次の値以下","waiFilterBarClearButton":"フィルターのクリア","conditionAfter":"次の値より後","statusTipHeaderCondition":"ルール","falseLabel":"False","filterButton":"フィルター","clearFilterDialogTitle":"フィルターのクリア","statusTipTitleHasFilter":"フィルター","addRuleButton":"ルールの追加","waiAddRuleButton":"新規ルールの追加","waiClearButton":"フィルターのクリア","conditionNotEqual":"次の値と等しくない","conditionNotEndWith":"次の値で終わらない","rangeTemplate":"${0} から ${1} まで","removeRuleButton":"ルールの削除","waiFilterBarDefButton":"表のフィルター","conditionIsEmpty":"is empty"})