({"conditionBefore":"przed","filterBarDefButton":"Zdefiniuj filtr","filterBarMsgNoFilterTemplate":"Nie zastosowano żadnego filtru","conditionContains":"zawiera","clearFilterMsg":"Spowoduje to usunięcie filtru oraz wyświetlenie wszystkich dostępnych rekordów. ","all":"wszystkie","waiValueBoxTemplate":"Wprowadź wartość filtru dla reguły ${0}","conditionRange":"zakres","waiCancelButton":"Anuluj to okno dialogowe","relationAll":"wszystkie reguły","conditionLarger":"jest większe niż","valueBoxLabel":"Wartość","statusTipMsg":"Kliknij pasek filtru w tym miejscu, aby filtrować wartości znajdujące się w ${0}. ","waiRelAll":"Dopasuj wszystkie następujące reguły:","closeFilterBarBtn":"Zamknij pasek filtru","a11yFilterBarDefButton":"Filtruj...","trueLabel":"Prawda","conditionIsNot":"nie jest","conditionIs":"jest","ruleTitleTemplate":"Reguła ${0}","waiRemoveRuleButtonTemplate":"Usuń regułę ${0}","and":"oraz","conditionLess":"jest mniejsze niż","waiColumnSelectTemplate":"Kolumna reguły ${0}","anycolumn":"dowolna kolumna","filterDefDialogTitle":"Filtr","waiConditionSelectTemplate":"Warunek reguły ${0}","waiFilterButton":"Wyślij filtr","statusTipTitle":"Pasek filtru","statusTipRelPre":"Zgodne","anyColumnOption":"Dowolna kolumna","conditionSelectLabel":"Warunek","cancelButton":"Anuluj","conditionNotStartWith":"nie zaczyna się od","conditionEndWith":"kończy się na","statusTipTitleNoFilter":"Pasek filtru","columnSelectLabel":"Kolumna","conditionNotContain":"nie zawiera","clearButton":"Wyczyść","conditionLargerEqual":"większe lub równe","any":"dowolne","relationMsgFront":"Zgodne","statusTipHeaderColumn":"Kolumna","defaultItemsName":"elementy","conditionEqual":"równe","relationAny":"dowolne reguły","trueLabelEditable":"Sprawdzone","or":"lub","filterBarMsgHasFilterTemplate":"Liczba wyświetlanych elementów ${2}: ${0} z ${1}. ","rangeTo":"do","conditionStartsWith":"zaczyna się od","waiRelAny":"Dopasuj dowolne z następujących reguł:","statusTipRelPost":"reguły. ","relationMsgTail":"","filterBarClearButton":"Wyczyść filtr","incompleteRuleTip":"Ta reguła nie jest kompletna. ","conditionLessEqual":"mniejsze lub równe","waiFilterBarClearButton":"Wyczyść filtr","conditionAfter":"po","statusTipHeaderCondition":"Reguły","falseLabel":"Fałsz","filterButton":"Filtr","clearFilterDialogTitle":"Wyczyść filtr","statusTipTitleHasFilter":"Filtr","addRuleButton":"Dodaj regułę","waiAddRuleButton":"Dodaj nową regułę","waiClearButton":"Wyczyść filtr","conditionNotEqual":"nie jest równe","conditionNotEndWith":"nie kończy się na","rangeTemplate":"od ${0} do ${1}","removeRuleButton":"Usuń regułę","waiFilterBarDefButton":"Filtruj tabelę","conditionIsEmpty":"is empty"})