<div class="dojoxGridFStatusTip">
	<div class="dojoxGridFStatusTipHead">
		<span class="dojoxGridFStatusTipTitle" dojoAttachPoint="statusTitle"></span
		><span class="dojoxGridFStatusTipRelPre" dojoAttachPoint="statusRelPre"></span
		><span class="dojoxGridFStatusTipRel" dojoAttachPoint="statusRel"></span
		><span class="dojoxGridFStatusTipRelPost" dojoAttachPoint="statusRelPost"></span>
	</div>
	<div class="dojoxGridFStatusTipDetail" dojoAttachPoint="statusDetailNode"></div>
</div>
