.dojoxGridRtl .dojoxGridLoading, 
.dojoxGridRtl .dojoxGridError,
.dojoxGridRtl .dojoxGridNoData {
	background-position:right;
	padding-right:25px;
	padding-left:0px;
}
.dojoxGridRtl .dojoxGridHeader {
	margin-left: 0;
	margin-right: -1px;
}

.dojoxGridRtl .dojoxGridCell {
	text-align: right;
}
