({"buttonSet":"Setare","insertTableTitle":"Inserare tabelă","insertTableRowAfterLabel":"Adăugare rând după","center":"centru","deleteTableColumnLabel":"Ştergere coloană","right":"dreapta","insertTableColumnBeforeLabel":"Adăugare coloană înainte","tableWidth":"Lăţime tabelă:","buttonInsert":"Inserare","default":"implicit","align":"Aliniere:","insertTableRowBeforeLabel":"Adăugare rând înainte","cellSpacing":"Spaţiere celulă:","pixels":"pixeli","selectTableLabel":"Selectare tabelă","rows":"Rânduri:","modifyTableTitle":"Modificare tabelă","cellPadding":"Completare celulă:","deleteTableRowLabel":"Ştergere rând","backgroundColor":"Culoare fundal:","insertTableColumnAfterLabel":"Adăugare coloană după","left":"stânga","borderThickness":"Grosime bordură","columns":"Coloane:","percent":"procent","borderColor":"Culoare bordură:","buttonCancel":"Cancel"})