({"buttonSet":"Beállítás","insertTableTitle":"Táblázat beszúrása","insertTableRowAfterLabel":"Sor hozzáadása utána","buttonCancel":"Mégse","center":"középre","deleteTableColumnLabel":"Oszlop törlése","right":"jobbra","insertTableColumnBeforeLabel":"Oszlop hozzáadása előtte","tableWidth":"Táblázat szélessége:","buttonInsert":"Beszúrás","default":"alapértelmezett","align":"Igazítás:","insertTableRowBeforeLabel":"Sor hozzáadása előtte","cellSpacing":"Cella térköz:","pixels":"képpont","selectTableLabel":"Táblázat kijelölése","rows":"Sorok:","modifyTableTitle":"Táblázat módosítása","cellPadding":"Cellakitöltés:","deleteTableRowLabel":"Sor törlése","backgroundColor":"Háttérszín:","insertTableColumnAfterLabel":"Oszlop hozzáadása utána","left":"balra","borderThickness":"Szegély vastagsága","columns":"Oszlopok:","percent":"százalék","borderColor":"Szegélyszín:"})