({"saveLabel":"Αποθήκευση","saveSettingdialogTitle":"Αυτόματη αποθήκευση","saveSettingdialogButtonOk":"Ορισμός διαστήματος","saveMessageSuccess":"Αποθηκεύτηκε στις ${0}","saveSettingdialogButtonCancel":"Ακύρωση","saveSettingdialogDescription":"Ορισμός διαστήματος αυτόματης αποθήκευσης","saveSettingLabelOn":"Ορισμός διαστήματος αυτόματης αποθήκευσης...","saveSettingdialogParamLabel":"λεπτά","saveSettingdialogParamName":"Διάστημα αυτόματης αποθήκευσης","saveMessageFail":"Απέτυχε η αποθήκευση στις ${0}","saveSettingLabelOff":"Απενεργοποίηση αυτόματης αποθήκευσης"})