({"cancel":"Cancelar","toDic":"Añadir al diccionario","skip":"Saltar","iMsg":"No hay sugerencias de ortografía","replaceAll":"Sustituir todo","widgetLabel":"Corrector ortográfico por lotes","skipAll":"Saltar todo","suggestions":"Sugerencias","iSkipAll":"Saltar todos los casos como este","replaceWith":"Sustituir por","msg":"No se han encontrado errores ortográficos","replace":"Sustituir","unfound":"No encontrado","iSkip":"Saltar esto"})