({"emoticonLaughing":"latter","emoticonCool":"kul","emoticonTongue":"tunge","emoticonCrying":"gråt","emoticonOops":"ops","emoticonFrown":"streng","emoticonAngry":"sint","emoticonShy":"sjenert","emoticonNo":"nei","emoticonAngel":"engel","smiley":"Sett inn uttrykksikon","emoticonIdea":"ide","emoticonEyebrow":"øyebryn","emoticonSmile":"smil","emoticonWink":"blunk","emoticonYes":"ja","emoticonGrin":"glis","emoticonGoofy":"tåpelig","emoticonHalf":"halv"})