({"set":"Sett inn","text":"Beskrivelse","insertImageTitle":"Sett inn bilde","invalidMessage":"Ugyldig bildefiltype","prePopuTextBrowse":" eller bla gjennom til en lokal fil.","browse":"Bla gjennom...","prePopuTextUrl":"Angi en bilde-URL","url":"Bilde"})