({"conditionBefore":"πριν","filterBarDefButton":"Ορισμός φίλτρου","filterBarMsgNoFilterTemplate":"Δεν έχει εφαρμοστεί φίλτρο","conditionContains":"περιέχει","clearFilterMsg":"Με την επιλογή αυτή θα αφαιρεθεί το φίλτρο και θα εμφανιστούν όλες οι διαθέσιμες εγγραφές.","all":"όλα","waiValueBoxTemplate":"Καταχωρήστε τιμή φίλτρου για τον κανόνα ${0}","conditionRange":"εύρος","waiCancelButton":"Ακύρωση αυτού του πλαισίου διαλόγου","relationAll":"όλοι οι κανόνες","conditionLarger":"είναι μεγαλύτερο από","valueBoxLabel":"Τιμή","statusTipMsg":"Πατήστε στη γραμμή φίλτρου για φιλτράρισμα με βάση τις τιμές στο ${0}.","waiRelAll":"Αντιστοιχία με όλους τους παρακάτω κανόνες:","closeFilterBarBtn":"Κλείσιμο γραμμής φίλτρου","a11yFilterBarDefButton":"Φιλτράρισμα...","trueLabel":"Αληθές","conditionIsNot":"δεν είναι","conditionIs":"είναι","ruleTitleTemplate":"Κανόνας ${0}","waiRemoveRuleButtonTemplate":"Αφαίρεση κανόνα ${0}","and":"και","conditionLess":"είναι μικρότερο από","waiColumnSelectTemplate":"Στήλη για τον κανόνα ${0}","anycolumn":"οποιαδήποτε στήλη","filterDefDialogTitle":"Φίλτρο","waiConditionSelectTemplate":"Συνθήκη για τον κανόνα ${0}","waiFilterButton":"Υποβολή του φίλτρου","statusTipTitle":"Γραμμή φίλτρου","statusTipRelPre":"Αντιστοιχία","anyColumnOption":"Οποιαδήποτε στήλη","conditionSelectLabel":"Συνθήκη","cancelButton":"Ακύρωση","conditionNotStartWith":"δεν αρχίζει από","conditionEndWith":"τελειώνει σε","statusTipTitleNoFilter":"Γραμμή φίλτρου","columnSelectLabel":"Στήλη","conditionNotContain":"δεν περιέχει","clearButton":"Εκκαθάριση","conditionLargerEqual":"μεγαλύτερο ή ίσο","any":"οποιοδήποτε","relationMsgFront":"Αντιστοιχία","statusTipHeaderColumn":"Στήλη","defaultItemsName":"στοιχεία","conditionEqual":"ίσο","relationAny":"οποιοσδήποτε κανόνας","trueLabelEditable":"Επιλεγμένο","or":"ή","filterBarMsgHasFilterTemplate":"Εμφανίζονται ${0} από ${1} ${2}.","rangeTo":"έως","conditionStartsWith":"αρχίζει από","waiRelAny":"Αντιστοιχία με οποιονδήποτε από τους παρακάτω κανόνες:","statusTipRelPost":"κανόνες.","relationMsgTail":"","filterBarClearButton":"Εκκαθάριση φίλτρου","incompleteRuleTip":"Αυτός ο κανόνας δεν είναι πλήρης.","conditionLessEqual":"μικρότερο ή ίσο","waiFilterBarClearButton":"Εκκαθάριση του φίλτρου","conditionAfter":"μετά","statusTipHeaderCondition":"Κανόνες","falseLabel":"Ψευδές","filterButton":"Φίλτρο","clearFilterDialogTitle":"Εκκαθάριση φίλτρου","statusTipTitleHasFilter":"Φίλτρο","addRuleButton":"Προσθήκη κανόνα","waiAddRuleButton":"Προσθήκη νέου κανόνα","waiClearButton":"Εκκαθάριση του φίλτρου","conditionNotEqual":"όχι ίσο","conditionNotEndWith":"δεν τελειώνει σε","rangeTemplate":"από ${0} έως ${1}","removeRuleButton":"Αφαίρεση κανόνα","waiFilterBarDefButton":"Φιλτράρισμα του πίνακα","conditionIsEmpty":"is empty"})