({"saveLabel":"Salvar","saveSettingdialogTitle":"Salvamento Automático","saveSettingdialogButtonOk":"Configurar Intervalo","saveMessageSuccess":"Salvo em ${0}","saveSettingdialogButtonCancel":"Cancelar","saveSettingdialogDescription":"Especificar Intervalo de Salvamento Automático","saveSettingLabelOn":"Configurar Intervalo de Salvamento Automático...","saveSettingdialogParamLabel":"mín.","saveSettingdialogParamName":"Intervalo de Salvamento Automático","saveMessageFail":"Falha ao salvar em ${0}","saveSettingLabelOff":"Desativar Salvamento Automático"})