({"gotoButtonTitle":"ไปที่หน้าที่ระบุ","prevTip":"หน้าก่อนหน้านี้","pageCountIndication":" (${0} หน้า)","pageStepLabelTemplate":"หน้า ${0}","descTemplate":"${2} - ${3} จาก ${1} ${0}","lastTip":"หน้าสุดท้าย","nextTip":"หน้าถัดไป","pageSizeLabelTemplate":"${0} ไอเท็มต่อหน้า","dialogCancel":"ยกเลิก","itemTitle":"ไอเท็ม","dialogConfirm":"ไปที่","firstTip":"หน้าแรก","allItemsLabelTemplate":"รายการ ทั้งหมด","dialogIndication":"ระบุหมายเลขหน้า","dialogTitle":"ไปที่หน้า"})