<div class="dojoxRollingList ${className}"
	><div class="dojoxRollingListContainer" dojoAttachPoint="containerNode" dojoAttachEvent="onkeypress:_onKey"
	></div
	><div class="dojoxRollingListButtons" dojoAttachPoint="buttonsNode"
        ><button dojoType="dijit.form.Button" dojoAttachPoint="okButton"
				dojoAttachEvent="onClick:_onExecute">${okButtonLabel}</button
        ><button dojoType="dijit.form.Button" dojoAttachPoint="cancelButton"
				dojoAttachEvent="onClick:_onCancel">${cancelButtonLabel}</button
	></div
></div>
