({"cancel":"Annuler","toDic":"Ajouter au dictionnaire","skip":"Ignorer","iMsg":"Aucune suggestion orthographique","replaceAll":"Remplacer tout","widgetLabel":"Vérification orthographique par lots","skipAll":"Ignorer tout","suggestions":"Suggestions","iSkipAll":"Ignorer chaque fois","replaceWith":"Remplacer par","msg":"Aucune faute d'orthographe trouvée","replace":"Remplacer","unfound":"Introuvable","iSkip":"Ignorer une fois"})