({"buttonSet":"Ange","insertTableTitle":"Infoga tabell","insertTableRowAfterLabel":"Lägg till rad efter","buttonCancel":"Avbryt","center":"centrera","deleteTableColumnLabel":"Ta bort kolumn","right":"högerjustera","insertTableColumnBeforeLabel":"Lägg till kolumn före","tableWidth":"Tabellbredd:","buttonInsert":"Infoga","default":"standard","align":"Justera:","insertTableRowBeforeLabel":"Lägg till rad före","cellSpacing":"Cellavstånd:","pixels":"bildpunkter","selectTableLabel":"Välj tabell","rows":"Rader:","modifyTableTitle":"Ändra tabell","cellPadding":"Cellutfyllnad:","deleteTableRowLabel":"Ta bort rad","backgroundColor":"Bakgrundsfärg:","insertTableColumnAfterLabel":"Lägg till kolumn efter","left":"vänsterjustera","borderThickness":"Ramtjocklek","columns":"Kolumner:","percent":"%","borderColor":"Ramfärg:"})