({"cancel":"Cancel·la","toDic":"Afegeix al diccionari","skip":"Salta","iMsg":"No hi ha suggeriments ortogràfics","replaceAll":"Substitueix tot","widgetLabel":"Correcció ortogràfica per lots","skipAll":"Salta tot","suggestions":"Suggeriments","iSkipAll":"Saltar tots els que són com aquest","replaceWith":"Substitueix per","msg":"No s'ha trobat cap error ortogràfic","replace":"Substitueix","unfound":"No trobat","iSkip":"Saltar aquest"})