.claro .dojoxGridFBarInfoTD {
	background: url("../../../resources/images/header_shadow.png") #F2F9FF repeat-x bottom;
	background: -moz-linear-gradient(top, #F2F9FF, #EAF5FF);
	background: -webkit-gradient(linear, left top, left bottom, from(#F2F9FF), to(#EAF5FF));
	filter: progid:DXImageTransform.Microsoft.gradient(startColorstr=#FFF2F9FF, endColorstr=#FFEAF5FF);
}
.claro .dojoxGridFBarHover .dojoxGridFBarInfoTD {
	background: url("../../../resources/images/header_shadow.png") #F2F9FF repeat-x bottom;
	background: -moz-linear-gradient(top, #ECF6FF 2px, #F2F9FF, #EAF5FF 6px);
	background: -webkit-gradient(linear, left top, left bottom, color-stop(0.1, #ECF6FF), color-stop(0.6, #F2F9FF), color-stop(0.8, #EAF5FF));
}
.dj_gecko .claro .dojoxGridFBarInfoTD,
.dj_webkit .claro .dojoxGridFBarInfoTD {
	border-top: 1px solid #BFD6EB;
	border-bottom: 1px solid #BFD6EB;
}
.dj_ie .claro .dojoxGridFBarInner{
	border-top: 1px solid #BFD6EB;
	border-bottom: 1px solid #BFD6EB;
	display:inline-block;
	width: 100%;
}
.claro .dojoxGridFBarClearFilterBtn {
	color: #5B83B6;
}
.claro .dojoxGridFBarInfoTD .dojoxGridFBarClearFilterBtn.dijitButtonHover .dijitButtonText {
	text-decoration: underline;
}
.claro .dojoxGridFBar {
	border-left: 1px solid white;
}
.claro .dojoxGridFStatusTip table {
	border: 1px solid #BCBCBC;
}
.claro .dojoxGridFStatusTip th {
	background-color: #E8EFF4;
	border-bottom: 1px solid #BCBCBC;
}
.claro .dojoxGridFStatusTip th div {
	background-color: #E8EFF4;
	border-left: 1px solid white;
	border-right: 1px solid #BCBCBC;
	border-top: 1px solid white;
	border-bottom: 1px solid white;
}
.claro .dojoxGridFStatusTip th.lastColumn div {
	border-right: 1px solid white;
}
.claro .dojoxGridFStatusTip td {
	border-top: 1px solid white;
	border-bottom: 1px solid white;
}
.claro .dojoxGridFStatusTipRel,
.claro .dojoxGridFStatusTipCondition {
	font-style: italic;
}
.claro .dojoxGridFStatusTipOddRow {
	background-color: #F1F8FF;
}
