/* Pygments native style */

code {background-color: #202020; color: #d0d0d0}

code .comment {color: #999; font-style: italic}
code .comment.preproc {color: #cd2828; font-style: normal; font-weight: bold}
code .comment.special {color: #e50808; font-style: normal; font-weight: bold; background-color: #520000}

code .keyword {color: #6ab825; font-weight: bold}
code .keyword.pseudo {font-weight: normal}

code .operator.word {color: #6ab825; font-weight: bold}

code .name.builtin {color: #24909d}
code .name.function {color: #40ffff}
code .name.class {color: #447fcf; text-decoration: underline}
code .name.namespace {color: #447fcf; text-decoration: underline}
code .name.exception {color: #bbb}
code .name.variable {color: #40ffff}
code .name.constant {color: #40ffff}
code .name.attribute {color: #bbb}
code .name.tag {color: #6ab825; font-weight: bold}
code .name.decorator {color: #ffa500}

code .string {color: #ed9d13}
code .string.other {color: #ffa500}

code .number {color: #3677a9}
