/* dojox.image.SlideShow */

.slideShowWrapper { 
	position:relative; 
	background:#fff;
	padding:8px; 
	border:1px solid #333;	
	padding-bottom:20px;
	overflow:hidden;
	-moz-border-radius:3pt;
	-webkit-border-radius:4pt;
	-webkit-drop-shadow:#ccc 4pt; 
}
.slideShowNav { 
	position:absolute;
	bottom:-18px;	
	left:0px;
	padding:0px 3px 2px 0px;
	background-color:#fff; 
	width:100%;
}
.slideShowNavWrapper { float:right;  }
.slideShowTitle {
	float:left;
	color:#333;
	font-size:10pt;
}
.slideShowTitle .slideShowCounterText {
	font-size:6pt; color:#666; 
}
.slideShowHidden {
	position:absolute;
	display: none;
	height: 1px;
	width: 1px;
}
.slideShowImageWrapper {
	position:relative;
	text-align: center;
	margin-top: -42px;
	float: left;
	width: 100%;
}
.slideShowImageWrapper img {
	border: 0px none;
}
.slideShowNotifier {
	background-color: red;
	width: 100px;
	height: 5px;
	font-size: 1%;/*IE hack to get around the Empty-Div bug*/
}
.slideShowSlideShow {
	position:absolute;
	top:30px; 
	padding: 0 5px;
	border: 0px;
	text-decoration: none; 
	color: #2e6ab1;
}
.slideShowLoading { background-color: #fad66a; }
.slideShowLoaded { background-color: transparent; }
/*
.sprite-arrowbottom { background-position: 0 -30px; } 
.sprite-arrowtop { background-position: 0 -430px; } 
*/
.slideShowCtrlPrev {
	background-position: -96px 0px; 
	float: left;
}
.slideShowCtrlNext {
	background-position: -144px 0px;
	float: right;
}
.slideShowCtrlPlay {
	background-position: -190px 0px;
	position: absolute;
}
.slideShowPaused .slideShowCtrlPlay {
	background-position: -236px 0px;
	position: absolute;
}
.slideShowCtrl span.slideShowCtrlHide {
	background-image: url("../../../dojo/resources/blank.gif");
	cursor: auto;
}

.slideShowCtrl {
	height: 50px;
	width: 100%;
	position: relative;
	z-index:999;
	float: left;
}
.slideShowCtrl span {
	width: 50px;
	height: 100%;
	background-image: url("images/buttons.png");
	cursor: pointer;
}
.dj_ie .slideShowCtrl span {
	background-image: url("images/buttons.gif");
}

.dj_ie6 .slideShowPager li.currentpage,
.dj_ie6 .pagination li.disablepage{
    /*IE 6 and below. Adjust non linked LIs slightly to account for bugs*/
    margin-right: 5px;
    padding-right: 0;
}

