({"replaceDialogText":"Se han sustituido ${0} apariciones.","eofDialogTextFind":"encontrado","eofDialogText":"Última aparición ${0}","backwards":"Hacia atrás","replaceButton":"Sustituir","replaceLabel":"Sustituir por:","matchCase":"Coincidir mayúsculas y minúsculas","findTooltip":"Especifique el texto que desee buscar","replaceTooltip":"Especifique el texto por el que se debe sustituir","replaceAllButton":"Sustituir todo","eofDialogTextReplace":"sustituido","findReplace":"Conmutar Buscar/Sustituir","backwardsTooltip":"Buscar texto hacia atrás","replaceAllButtonTooltip":"Sustituir todo el texto","replaceButtonTooltip":"Sustituir el texto","findLabel":"Buscar:","findButton":"Buscar","matchCaseTooltip":"Coincidir mayúsculas y minúsculas","findButtonTooltip":"Buscar el texto","replaceAll":"Todas las apariciones"})