({"selectContents":"Valitse sisältö","deleteElement":"Poista elementti","moveEnd":"Siirrä kohdistin loppuun","deleteContents":"Poista sisältö","nodeActions":"${nodeName} - Toiminnot","selectElement":"Valitse elementti","moveStart":"Siirrä kohdistin alkuun"})