({"selectContents":"เลือกเนื้อหา","deleteElement":"ลบอิลิเมนต์","moveEnd":"ย้ายเคอร์เซอร์ไปยังจุดสิ้นสุด","deleteContents":"ลบเนื้อหา","nodeActions":"${nodeName} แอ็คชัน","selectElement":"เลือกอิลิเมนต์","moveStart":"ย้ายเคอร์เซอร์ไปยังจุดเริ่มต้น"})