({"gotoButtonTitle":"跳至特定的頁面","prevTip":"上一頁","pageCountIndication":" （${0} 個頁面）","pageStepLabelTemplate":"頁面 ${0}","descTemplate":"${2} - ${1} ${0} 之 ${3}","lastTip":"末頁","nextTip":"下一頁","pageSizeLabelTemplate":"每頁 ${0} 個項目","dialogCancel":"取消","itemTitle":"項目","dialogConfirm":"執行","firstTip":"首頁","allItemsLabelTemplate":"所有項目","dialogIndication":"指定頁碼","dialogTitle":"跳至頁面"})