({"emoticonLaughing":"笑い","emoticonCool":"無愛想","emoticonTongue":"舌を出す","emoticonCrying":"泣く","emoticonOops":"おっと","emoticonFrown":"眉をひそめる","emoticonAngry":"怒り","emoticonShy":"はにかんだ","emoticonNo":"いいえ","emoticonAngel":"エンジェル","smiley":"顔文字の挿入","emoticonIdea":"アイデア","emoticonEyebrow":"眉毛","emoticonSmile":"微笑","emoticonWink":"ウィンク","emoticonYes":"はい","emoticonGrin":"笑顔","emoticonGoofy":"おろかな","emoticonHalf":"半分"})