.dijitEditorIconAutoSave {
	background-image: url(../icons/autoSave.png);
	background-repeat: no-repeat;
	width: 18px;
	height: 18px;
}

.dijitEditorIconAutoSaveDefault {
	background-position: -18px 0px;
}

.dijitDisabled .dijitEditorIconAutoSaveDefault {
	background-position: -54px 0px;
}

.dijitEditorIconAutoSaveSetting {
	background-position: 0px 0px;
}

.dijitEditorAutoSaveSettingDialog {
	width: 21em;
}

.dijitEditorAutoSaveSettingInputArea {
	margin-top: 0.3em;
	margin-left: 2em;
}

.dijitEditorAutoSaveSettingInputArea .textBox {
	width: 2em;
}

.dijitEditorAutoSaveSettingInputArea .boxLabel {
	margin: 0em 0em 0em 0.3em;
}

.dijitEditorAutoSaveSettingButtonArea {
	text-align: right;
	margin: 0.3em 0em 0em 0em;
}

.lucid .dijitEditorAutoSaveSettingDialog {
	width: 22em;
}

.lucid .dijitEditorAutoSaveSettingInputArea .textBox {
	width: 2.5em;
}
