({"conditionBefore":"před","filterBarDefButton":"Definovat filtr","filterBarMsgNoFilterTemplate":"Není použit žádný filtr","conditionContains":"obsahuje","clearFilterMsg":"Tato akce odebere filtr a zobrazí se všechny dostupné záznamy.","all":"vše","waiValueBoxTemplate":"Zadat hodnotu filtru pro pravidlo ${0}","conditionRange":"rozsah","waiCancelButton":"Zrušit toto dialogové okno","relationAll":"všechna pravidla","conditionLarger":"je větší než","valueBoxLabel":"Hodnota","statusTipMsg":"Klepnutím sem na panel filtru provedete filtrování podle hodnoty pro ${0}.","waiRelAll":"Vyhovovat všem následujícím pravidlům:","closeFilterBarBtn":"Zavřít panel filtru","a11yFilterBarDefButton":"Filtrovat...","trueLabel":"Pravda","conditionIsNot":"není","conditionIs":"je","ruleTitleTemplate":"Pravidlo ${0}","waiRemoveRuleButtonTemplate":"Odebrat pravidlo ${0}","and":"a","conditionLess":"je menší než","waiColumnSelectTemplate":"Sloupec pravidla ${0}","anycolumn":"libovolný sloupec","filterDefDialogTitle":"Filtr","waiConditionSelectTemplate":"Podmínka pravidla ${0}","waiFilterButton":"Odeslat tento filtr","statusTipTitle":"Panel filtru","statusTipRelPre":"Shoda","anyColumnOption":"Libovolné sloupec","conditionSelectLabel":"Podmínka","cancelButton":"Storno","conditionNotStartWith":"nezačíná na","conditionEndWith":"končí na","statusTipTitleNoFilter":"Panel filtru","columnSelectLabel":"Sloupec","conditionNotContain":"neobsahuje","clearButton":"Vymazat","conditionLargerEqual":"větší nebo rovno","any":"libovolné","relationMsgFront":"Shoda","statusTipHeaderColumn":"Sloupec","defaultItemsName":"položek","conditionEqual":"rovná se","relationAny":"libovolné pravidlo","trueLabelEditable":"Zaškrtnuto","or":"nebo","filterBarMsgHasFilterTemplate":"Zobrazeno ${0} z ${1} ${2}.","rangeTo":"do","conditionStartsWith":"začíná na","waiRelAny":"Vyhovovat libovolnému z následujících pravidel:","statusTipRelPost":"pravidel.","relationMsgTail":"","filterBarClearButton":"Vymazat filtr","incompleteRuleTip":"Toto pravidlo není úplné.","conditionLessEqual":"menší nebo rovno","waiFilterBarClearButton":"Vymazat tento filtr","conditionAfter":"po","statusTipHeaderCondition":"Pravidla","falseLabel":"Nepravda","filterButton":"Filtrovat","clearFilterDialogTitle":"Vymazat filtr","statusTipTitleHasFilter":"Filtr","addRuleButton":"Přidat pravidlo","waiAddRuleButton":"Přidat nové pravidlo","waiClearButton":"Vymazat tento filtr","conditionNotEqual":"nerovná se","conditionNotEndWith":"nekončí na","rangeTemplate":"od ${0} do ${1}","removeRuleButton":"Odebrat pravidlo","waiFilterBarDefButton":"Filtrovat tabulku","conditionIsEmpty":"is empty"})