({"cancel":"取消","toDic":"添加到字典","skip":"跳过","iMsg":"没有拼写建议","replaceAll":"全部替换","widgetLabel":"批处理拼写检查","skipAll":"全部跳过","suggestions":"建议","iSkipAll":"跳过所有与此处相同的地方","replaceWith":"替换为","msg":"未找到拼写错误","replace":"替换","unfound":"未找到","iSkip":"跳过此处"})