/* === Search Button ==*/
.mblDomButtonSearch_2 {
	background-image: url(compat/search-button.png);
	background-repeat: no-repeat;
}
.mblDomButtonSearch_2 DIV {
	display: none;
}

.dj_ie6 .mblDomButtonSearch_2 {
	background-image: url(compat/search-button.gif);
}
