({"saveLabel":"儲存","saveSettingdialogTitle":"自動儲存","saveSettingdialogButtonOk":"設定間隔","saveMessageSuccess":"已儲存於 ${0}","saveSettingdialogButtonCancel":"取消","saveSettingdialogDescription":"指定自動儲存間隔","saveSettingLabelOn":"設定自動儲存間隔...","saveSettingdialogParamLabel":"分鐘","saveSettingdialogParamName":"自動儲存間隔","saveMessageFail":"無法儲存於 ${0}","saveSettingLabelOff":"關閉自動儲存"})