({"saveLabel":"저장","saveSettingdialogTitle":"자동 저장","saveSettingdialogButtonOk":"간격 설정","saveMessageSuccess":"${0}에 저장됨","saveSettingdialogButtonCancel":"취소","saveSettingdialogDescription":"자동 저장 간격 지정","saveSettingLabelOn":"자동 저장 간격 설정...","saveSettingdialogParamLabel":"최소","saveSettingdialogParamName":"자동 저장 간격","saveMessageFail":"${0}에 저장 실패","saveSettingLabelOff":"자동 저장 끄기"})