({"buttonSet":"Definer","insertTableTitle":"Sett inn tabell","insertTableRowAfterLabel":"Legg til rad etter","buttonCancel":"Avbryt","center":"midtstill","deleteTableColumnLabel":"Slett kolonne","right":"høyre","insertTableColumnBeforeLabel":"Legg til kolonne foran","tableWidth":"Tabellbredde:","buttonInsert":"Sett inn","default":"standard","align":"Juster:","insertTableRowBeforeLabel":"Legg til rad foran","cellSpacing":"Celleavstand:","pixels":"piksler","selectTableLabel":"Velg tabell","rows":"Rader:","modifyTableTitle":"Endre tabell","cellPadding":"Celleutfylling:","deleteTableRowLabel":"Slett rad","backgroundColor":"Bakgrunnsfarge:","insertTableColumnAfterLabel":"Legg til kolonne etter","left":"venstre","borderThickness":"Kanttykkelse","columns":"Kolonner:","percent":"prosent","borderColor":"Kantlinjefarge:"})