({"pasteFromWord":"Paste From  Word","paste":"Paste","cancel":"Cancel","instructions":"Paste the content from Word into the text box below.  Once you are satisfied with the content to insert, press the paste button.  To abort inserting text, press the cancel button."})