({"emoticonLaughing":"ضاحك","emoticonCool":"حسنا","emoticonTongue":"لسان","emoticonCrying":"يبكي","emoticonOops":"عفوا","emoticonFrown":"عبوس","emoticonAngry":"غضبان","emoticonShy":"خجول","emoticonNo":"لا","emoticonAngel":"ملاك","smiley":"ادراج شكل متحرك","emoticonIdea":"فكرة","emoticonEyebrow":"حاجب","emoticonSmile":"ابتسامة","emoticonWink":"غمزة","emoticonYes":"نعم","emoticonGrin":"تكشير","emoticonGoofy":"أبله","emoticonHalf":"نصف"})