/*
	Copyright (c) 2004-2011, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["dojox.drawing.library.greek"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.drawing.library.greek"] = true;
dojo.provide("dojox.drawing.library.greek");

dojox.drawing.library.greek = {
	// summary:
	//		Greek characters used by typesetter and greekPalette.
	// description:
	//		These are used to convert between the character and
	//		the written version of greek letters.  Any character
	//		can be included here and it will automatically be added
	//		to the palette and converted by typesetter
        
        "alpha": 945,   //alpha,     U+03B1 ISOgrk3 -->
        "beta": 946,   //beta, U+03B2 ISOgrk3 -->
        "gamma": 947,   //gamma,  U+03B3 ISOgrk3 -->
        "delta": 948,   //delta,        U+03B4 ISOgrk3 -->
        "epsilon": 949,   //epsilon,    U+03B5 ISOgrk3 -->
        "zeta": 950,   //zeta, U+03B6 ISOgrk3 -->
        "eta": 951,   //eta, U+03B7 ISOgrk3 -->
        "theta": 952,   //theta,   U+03B8 ISOgrk3 -->
        "iota": 953,   //iota, U+03B9 ISOgrk3 -->
        "kappa": 954,   //kappa,   U+03BA ISOgrk3 -->
        "lambda": 955,   //lambda,    U+03BB ISOgrk3 -->
        "mu": 956,   //mu, U+03BC ISOgrk3 -->
        "nu": 957,   //nu, U+03BD ISOgrk3 -->
        "xi": 958,   //xi, U+03BE ISOgrk3 -->
        "omicron": 959,   //omicron, U+03BF NEW -->
        "pi": 960,   //pi, U+03C0 ISOgrk3 -->
        "rho": 961,   //rho, U+03C1 ISOgrk3 -->
        "sigmaf": 962,   //final sigma,  U+03C2 ISOgrk3 -->
        "sigma": 963,   //sigma,  U+03C3 ISOgrk3 -->
        "tau": 964,   //tau, U+03C4 ISOgrk3 -->
        "upsilon": 965,   //upsilon,   U+03C5 ISOgrk3 -->
        "phi": 966,   //phi, U+03C6 ISOgrk3 -->
        "chi": 967,   //chi, U+03C7 ISOgrk3 -->
        "psi": 968,   //psi, U+03C8 ISOgrk3 -->
        "omega": 969,   //omega,      U+03C9 ISOgrk3 -->
        "thetasym": 977,   //theta symbol,  U+03D1 NEW -->
        "upsih": 978,     // upsilon with hook symbol,  U+03D2 NEW -->
        "piv": 982,     // greek pi symbol, U+03D6 ISOgrk3 -->
        "Alpha": 913,    // alpha, U+0391 -->
        "Beta": 914,      // beta, U+0392 -->
        "Gamma": 915,    //gamma, U+0393 ISOgrk3 -->
        "Delta": 916,    //delta,           U+0394 ISOgrk3 -->
        "Epsilon": 917,    //epsilon, U+0395 -->
        "Zeta": 918,    //zeta, U+0396 -->
        "Eta": 919,    //eta, U+0397 -->
        "Theta": 920,    //theta,           U+0398 ISOgrk3 -->
        "Iota": 921,    //iota, U+0399 -->
        "Kappa": 922,    //kappa, U+039A -->
        "Lambda": 923,    //lambda,      U+039B ISOgrk3 -->
        "Mu": 924,    //mu, U+039C -->
        "Nu": 925,    //nu, U+039D -->
        "Xi": 926,    //xi, U+039E ISOgrk3 -->
        "Omicron": 927,    //omicron, U+039F -->
        "Pi": 928,    //pi, U+03A0 ISOgrk3 -->
        "Rho": 929,    //rho, U+03A1 -->
        "Sigma": 931,    //sigma,     U+03A3 ISOgrk3 -->
        "Tau": 932,    //tau, U+03A4 -->
        "Upsilon": 933,    //upsilon,   U+03A5 ISOgrk3 -->
        "Phi": 934,    //phi, U+03A6 ISOgrk3 -->
        "Chi": 935,    //chi, U+03A7 -->
        "Psi": 936,    //psi,   U+03A8 ISOgrk3 -->
        "Omega": 937    //omega,     U+03A9 ISOgrk3 -->
};
    
    

}
