<div dojoAttachPoint="paginatorBar">
	<table cellpadding="0" cellspacing="0"  class="dojoxGridPaginator">
		<tr>
			<td dojoAttachPoint="descriptionTd" class="dojoxGridDescriptionTd">
				<div dojoAttachPoint="descriptionDiv" class="dojoxGridDescription" />
			</td>
			<td dojoAttachPoint="sizeSwitchTd"></td>
			<td dojoAttachPoint="pageStepperTd" class="dojoxGridPaginatorFastStep">
				<div dojoAttachPoint="pageStepperDiv" class="dojoxGridPaginatorStep"></div>
			</td>
		</tr>
	</table>
</div>
