({"hueLabel":"t","saturationPickerTitle":"Sélecteur de saturation","huePickerTitle":"Sélecteur de teinte","greenLabel":"v","valueLabel":"v","blueLabel":"b","saturationLabel":"s","redLabel":"r","hexLabel":"hex","degLabel":"°"})