({"pasteFromWord":"הדבקה מתוך Word","paste":"הדבקה","cancel":"ביטול","instructions":"הדביקו את התוכן מתוך Word לתוך תיבת התמליל למטה. לאחר שתהיו מרוצים מהתוכן להוספה, לחצו על לחצן ההדבקה. כדי לבטל את הוספת התמליל, לחצו על לחצן הביטול. "})