{% for css in prepared_instructions.css %}
    {% if css.ieversion %}
    <!--[if {{ css.ieversion }}]>
    {% endif %}
    {% if css.location %}
        <link rel="stylesheet{% if css.process == 'lessjs' %}/less{% endif %}" type="text/css" href="{{ css.location }}" media="{{ css.media|default:"screen" }}" />
    {% else %}
        <style rel="stylesheet{% if css.process == 'lessjs' %}/less{% endif %}" type="text/css" media="{{ css.media|default:"screen" }}">
            {{ css.source }}
        </style>
    {% endif %}
    {% if css.ieversion %}
    <![endif]-->
    {% endif %}
{% endfor %}
