({"conditionBefore":"ennen","filterBarDefButton":"Määritä suodatin","filterBarMsgNoFilterTemplate":"Ei suodatinta käytössä","conditionContains":"sisältää","clearFilterMsg":"Poistaa suodattimen ja näyttää kaikki käytettävissä olevat tietueet.","all":"kaikki","waiValueBoxTemplate":"Anna suodatusarvo säännölle ${0}","conditionRange":"sallittu alue","waiCancelButton":"Peruuta tämä valintaikkuna","relationAll":"kaikki säännöt","conditionLarger":"on suurempi kuin","valueBoxLabel":"Arvo","statusTipMsg":"Napsauta suodatinpalkkia tässä ja suodata arvot kohteessa ${0}.","waiRelAll":"Vastaa kaikkia seuraavia sääntöjä:","closeFilterBarBtn":"Sulje suodatinpalkki","a11yFilterBarDefButton":"Suodata...","trueLabel":"Tosi","conditionIsNot":"ei ole","conditionIs":"on","ruleTitleTemplate":"Sääntö ${0}","waiRemoveRuleButtonTemplate":"Poista sääntö ${0}","and":"ja","conditionLess":"on pienempi kuin","waiColumnSelectTemplate":"Sarake säännölle ${0}","anycolumn":"mikä tahansa sarake","filterDefDialogTitle":"Suodatin","waiConditionSelectTemplate":"Ehto säännölle ${0}","waiFilterButton":"Lähetä suodatin","statusTipTitle":"Suodatinpalkki","statusTipRelPre":"Vastaa","anyColumnOption":"Mikä tahansa sarake","conditionSelectLabel":"Ehto","cancelButton":"Peruuta","conditionNotStartWith":"ei ala merkillä","conditionEndWith":"loppuu merkkiin","statusTipTitleNoFilter":"Suodatinpalkki","columnSelectLabel":"Sarake","conditionNotContain":"ei sisällä","clearButton":"Tyhjennä","conditionLargerEqual":"suurempi tai yhtä suuri","any":"mikä tahansa","relationMsgFront":"Vastaa","statusTipHeaderColumn":"Sarake","defaultItemsName":"kohteet","conditionEqual":"on yhtä suuri","relationAny":"mitkä tahansa säännöt","trueLabelEditable":"Tarkistettu","or":"tai","filterBarMsgHasFilterTemplate":"${0} / ${1} ${2} näkyy.","rangeTo":"-","conditionStartsWith":"alkaa merkillä","waiRelAny":"Vastaa jotakin seuraavista säännöistä:","statusTipRelPost":"sääntöä.","relationMsgTail":"","filterBarClearButton":"Tyhjennä suodatin","incompleteRuleTip":"Tämä sääntö ei ole valmis.","conditionLessEqual":"pienempi tai yhtä suuri","waiFilterBarClearButton":"Tyhjennä suodatin","conditionAfter":"jälkeen","statusTipHeaderCondition":"Säännöt","falseLabel":"Epätosi","filterButton":"Suodatin","clearFilterDialogTitle":"Tyhjennä suodatin","statusTipTitleHasFilter":"Suodatin","addRuleButton":"Lisää sääntö","waiAddRuleButton":"Lisää uusi sääntö","waiClearButton":"Tyhjennä suodatin","conditionNotEqual":"ei ole yhtä suuri","conditionNotEndWith":"ei lopu merkkiin","rangeTemplate":"${0} - ${1}","removeRuleButton":"Poista sääntö","waiFilterBarDefButton":"Suodata taulukko","conditionIsEmpty":"is empty"})