({"set":"Inserisci","text":"Descrizione","insertImageTitle":"Inserisci immagine","invalidMessage":"Tipo di file immagine non valido","prePopuTextBrowse":" o individuare un file locale.","browse":"Sfoglia...","prePopuTextUrl":"Immettere un URL immagine","url":"Immagine"})