({"selectContents":"Selectare conţinut","deleteElement":"Ştergere element","moveEnd":"Mutare cursor la sfârşit","deleteContents":"Ştergere conţinut","nodeActions":"Acţiuni ${nodeName}","selectElement":"Selectare element","moveStart":"Mutare cursor la început"})