({"cancel":"İptal","toDic":"Sözlüğe ekle","skip":"Atla","iMsg":"Yazım önerisi yok","replaceAll":"Tümünü Değiştir","widgetLabel":"Toplu Yazım Denetimi","skipAll":"Tümünü Atla","suggestions":"Öneriler","iSkipAll":"Buna benzeyenlerin tümünü atla","replaceWith":"Şununla Değiştir","msg":"Yazım hatası bulunamadı","replace":"Değiştir","unfound":"Bulunamadı","iSkip":"Bunu atla"})