({"emoticonLaughing":"күлу","emoticonCool":"салқын","emoticonTongue":"тіл","emoticonCrying":"жылау","emoticonOops":"ой","emoticonFrown":"қабағы түйілу","emoticonAngry":"ашулы","emoticonShy":"ұялшақ","emoticonNo":"ешбір","emoticonAngel":"періште","smiley":"Эмограмма енгізу","emoticonIdea":"ой","emoticonEyebrow":"қас","emoticonSmile":"күлімсіреу","emoticonWink":"көз қысу","emoticonYes":"иә","emoticonGrin":"ақситу","emoticonGoofy":"ақымақ","emoticonHalf":"жарты"})