({"saveLabel":"Enregistrer","saveSettingdialogTitle":"Enregistrement automatique","saveSettingdialogButtonOk":"Définir l'intervalle","saveMessageSuccess":"Enregistré à ${0}","saveSettingdialogButtonCancel":"Annuler","saveSettingdialogDescription":"Spécifiez l'intervalle d'enregistrement automatique","saveSettingLabelOn":"Définir l'intervalle d'enregistrement automatique...","saveSettingdialogParamLabel":"min","saveSettingdialogParamName":"Intervalle d'enregistrement automatique","saveMessageFail":"Echec de l'enregistrement à ${0}","saveSettingLabelOff":"Désactiver l'enregistrement automatique"})