({"cancel":"취소","toDic":"사전에 추가","skip":"건너뛰기","iMsg":"맞춤법 제안 없음","replaceAll":"모두 바꾸기","widgetLabel":"맞춤법 검사 일괄처리","skipAll":"모두 건너뛰기","suggestions":"제안","iSkipAll":"다음과 같이 모두 건너뛰기","replaceWith":"다음으로 바꾸기","msg":"맞춤법 오류 없음","replace":"바꾸기","unfound":"찾을 수 없음","iSkip":"이 항목 건너뛰기"})