({"saveLabel":"Mentés","saveSettingdialogTitle":"Automatikus mentés","saveSettingdialogButtonOk":"Időköz beállítása","saveMessageSuccess":"Mentés: ${0}","saveSettingdialogButtonCancel":"Mégse","saveSettingdialogDescription":"Automatikus mentés időközének megadása","saveSettingLabelOn":"Automatikus mentés időközének beállítása...","saveSettingdialogParamLabel":"perc","saveSettingdialogParamName":"Automatikus mentés időköze","saveMessageFail":"Sikertelen mentés: ${0}","saveSettingLabelOff":"Automatikus mentés kikapcsolása"})