({"saveLabel":"Salva","saveSettingdialogTitle":"Salvataggio automatico","saveSettingdialogButtonOk":"Imposta intervallo","saveMessageSuccess":"Salvato alle ${0}","saveSettingdialogButtonCancel":"Annulla","saveSettingdialogDescription":"Specifica intervallo di salvataggio automatico","saveSettingLabelOn":"Imposta intervallo di salvataggio automatico...","saveSettingdialogParamLabel":"min","saveSettingdialogParamName":"Intervallo di salvataggio automatico","saveMessageFail":"Salvataggio alle ${0} non riuscito","saveSettingLabelOff":"Disattiva salvataggio automatico"})