/*
 This is directly from @import "../plugins/resources/css/InsertEntity.css";
 It's here because certain details may change over time.  The classes
 remain the same so applying the original will be easy in case of change.
*/

.dojoxEntityPalette {
	/* outer node of the dropdown */
	border: 1px solid #999;
	background: #fff;
	-moz-border-radius: 3pt;
}

.dojoxEntityPaletteCell {
	/* individual cell of the drop down */
	border: 1px dotted gray;
	width: 20px;	/* todo: don't hardcode width/height; it's neither nececessary nor a11y safe */
	line-height: 18px;
	overflow: hidden;
	z-index: 10;
	text-align: center;
}

.dojoxEntityPaletteCellHover, .dojoxEntityPaletteCellActive, .dojoxEntityPaletteCellFocused {
	width: 18px;
        line-height: 16px;
	overflow: hidden;
	cursor: default;
	border:1px dashed #000;
	outline:1px dashed #dedede;
}


.dojoxEntityPalettePreviewTable {
	table-layout: auto;
	font-size: 1em;
	width: 100%;
}

.dojoxEntityPalettePreviewHeader {
	font-size: .8em;
	padding: 3px 3px 3px 3px;
}

.dojoxEntityPalettePreviewDetailEntity {
	font-size: .8em;
	font-weight: bold;
}

.dojoxEntityPalettePreviewDetail {
	font-size: .8em;
	padding: 3px 3px 3px 3px;
}

.dijit_a11y .dojoxEntityPaletteCell {
	background-color:transparent !important;
}
