<div class="dijit dijitReset dijitInline" dojoAttachEvent="onmousedown:_onMouseDown,onclick:focus"
	><select class="${baseClass}Select" multiple="true" dojoAttachPoint="containerNode,focusNode"></select
	><div dojoAttachPoint="wrapperDiv"></div
></div>