({"cancel":"Annulla","toDic":"Aggiungi al dizionario","skip":"Ignora","iMsg":"Nessun suggerimento ortografico","replaceAll":"Sostituisci tutto","widgetLabel":"Controllo ortografico batch","skipAll":"Ignora tutto","suggestions":"Suggerimenti","iSkipAll":"Ignora tutto","replaceWith":"Sostituisci con","msg":"Nessun errore di ortografia trovato","replace":"Sostituisci","unfound":"Non trovato","iSkip":"Ignora"})