.dijitAdditionalEditorIconShowBlockNodes {
	background-image: url(../icons/showBlockNodes.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 18px;
	height: 18px;
}

.dijitDisabled .dijitAdditionalEditorIconShowBlockNodes {
	background-image: url(../icons/showBlockNodesDisabled.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 18px;
	height: 18px;
}


