({"set":"Insereix","text":"Descripció","insertImageTitle":"Insereix imatge","invalidMessage":"Tipus de fitxer d'imatge no vàlid","prePopuTextBrowse":" o navegueu fins un fitxer local.","browse":"Navega...","prePopuTextUrl":"Especifiqueu un URL d'imatge","url":"Imatge"})