<div dojoAttachPoint="domNode" class="dojoxDocTester">
	<div dojoAttachPoint="containerNode"></div>
	<button dojoType="dojox.form.BusyButton" busyLabel="Testing..." dojoAttachPoint="runButtonNode">Run tests</button>
	<button dojoType="dijit.form.Button" dojoAttachPoint="resetButtonNode" style="display:none;">Reset</button>
	<span>
		<span dojoAttachPoint="numTestsNode">0</span> tests,
		<span dojoAttachPoint="numTestsOkNode">0</span> passed,
		<span dojoAttachPoint="numTestsNokNode">0</span> failed
	</span>
</div>