({"replaceDialogText":"${0} occurrence(s) remplacée(s)","eofDialogTextFind":"trouvé","eofDialogText":"Dernière occurrence ${0}","backwards":"Vers l'arrière","replaceButton":"Remplacer","replaceLabel":"Remplacer par :","matchCase":"Respecter la casse","findTooltip":"Entrez le texte à rechercher","replaceTooltip":"Entrez le texte de remplacement","replaceAllButton":"Remplacer tout","eofDialogTextReplace":"remplacé","findReplace":"Rechercher et remplacer","backwardsTooltip":"Recherchez le texte vers l'arrière","replaceAllButtonTooltip":"Remplacez tout le texte","replaceButtonTooltip":"Remplacez le texte","findLabel":"Rechercher :","findButton":"Rechercher","matchCaseTooltip":"Respecter la casse","findButtonTooltip":"Recherchez le texte","replaceAll":"Toutes les occurrences"})