.dijitAdditionalEditorIconPasteFromWord {
	background-image: url(../icons/pasteFromWord.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 18px;
	height: 18px;
}

.dijitDisabled .dijitAdditionalEditorIconPasteFromWord {
	background-image: url(../icons/pasteFromWordDisabled.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 18px;
	height: 18px;
}

.tundra .dijitPasteFromWordEmbeddedRTE .dijitEditor,
.nihilo .dijitPasteFromWordEmbeddedRTE .dijitEditor {
	border-color: #BFBFBF;
	border-style: solid;
	border-width: 1px;
}

.soria .dijitPasteFromWordEmbeddedRTE .dijitEditor {
	border-color: #CCCCCC;
	border-style: solid;
	border-width: 1px;
}


