({"saveLabel":"שמירה","saveSettingdialogTitle":"שמירה אוטומטית","saveSettingdialogButtonOk":"הגדרת מרווח  ","saveMessageSuccess":"נשמר ${0}","saveSettingdialogButtonCancel":"ביטול","saveSettingdialogDescription":"ציינו מרווח שמירה אוטומטית ","saveSettingLabelOn":"הגדרת מרווח שמירה אוטומטית...‏","saveSettingdialogParamLabel":"דות","saveSettingdialogParamName":"מרווח שמירה אוטומטית ","saveMessageFail":"כשל בשמירה ${0}","saveSettingLabelOff":"השבתת שמירה אוטומטית"})