({"saturationPickerTitle":"Metningsvelger","huePickerTitle":"Nyansevelger","valueLabel":"v","blueLabel":"b","saturationLabel":"s","greenLabel":"g","redLabel":"r","hueLabel":"h","hexLabel":"hex","degLabel":"°"})