({"replaceDialogText":"Erstattet ${0} forekomster.","eofDialogTextFind":"fundet","eofDialogText":"Sidste forekomst ${0}","backwards":"Tilbage","replaceButton":"Erstat","replaceLabel":"Erstat med:","matchCase":"Store/små bogstaver","findTooltip":"Indtast tekst, der skal søges efter","replaceTooltip":"Indtast tekst, der skal erstattes med","replaceAllButton":"Erstat alle","eofDialogTextReplace":"erstattet","findReplace":"Søg og erstat","backwardsTooltip":"Søg baglæns efter tekst","replaceAllButtonTooltip":"Erstat alle forekomster i teksten","replaceButtonTooltip":"Erstat teksten","findLabel":"Søg efter:","findButton":"Søg","matchCaseTooltip":"Store/små bogstaver","findButtonTooltip":"Find teksten","replaceAll":"Alle forekomster"})