({"saveLabel":"Opslaan","saveSettingdialogTitle":"Automatisch opslaan","saveSettingdialogButtonOk":"Interval instellen","saveMessageSuccess":"Opgeslagen op ${0}","saveSettingdialogButtonCancel":"Annuleren","saveSettingdialogDescription":"Geef het interval voor Automatisch opslaan op.","saveSettingLabelOn":"Interval voor automatisch opslaan instellen...","saveSettingdialogParamLabel":"min","saveSettingdialogParamName":"Interval voor Automatisch opslaan","saveMessageFail":"Opslaan mislukt op ${0}","saveSettingLabelOff":"Automatisch opslaan uitschakelen"})