({"gotoButtonTitle":"Belli bir sayfaya git","prevTip":"Önceki Sayfa","pageCountIndication":" (${0} sayfa)","pageStepLabelTemplate":"Sayfa ${0}","descTemplate":"${2} - ${3} / ${1} ${0}","lastTip":"Son Sayfa","nextTip":"Sonraki Sayfa","pageSizeLabelTemplate":"Sayfa başına ${0} öğe","dialogCancel":"İptal","itemTitle":"öğeler","dialogConfirm":"Git","firstTip":"İlk Sayfa","allItemsLabelTemplate":"Tüm öğeler","dialogIndication":"Sayfa numarasını belirt","dialogTitle":"Sayfaya Git"})