({"cancel":"Prekliči","toDic":"Dodaj v slovar ","skip":"Preskoči","iMsg":"Ni predlogov za črkovanje ","replaceAll":"Zamenjaj vse","widgetLabel":"Paketno preverjanje črkovanja ","skipAll":"Preskoči vse","suggestions":"Predlogi ","iSkipAll":"Preskoči vse vnose, kot je ta ","replaceWith":"Zamenjaj z","msg":"Najdenih ni bilo nobenih napačnih črkovanj ","replace":"Zamenjaj ","unfound":"Ni najdeno ","iSkip":"Preskoči to "})