({"conditionBefore":"antes","filterBarDefButton":"Definir filtro","filterBarMsgNoFilterTemplate":"Nenhum filtro aplicado","conditionContains":"contém","clearFilterMsg":"Este procedimento irá remover o filtro e apresentar todos os registos disponíveis.","all":"tudo","waiValueBoxTemplate":"Introduzir valor para filtrar para a regra ${0}","conditionRange":"intervalo","waiCancelButton":"Cancelar esta caixa de diálogo","relationAll":"todas as regras","conditionLarger":"é maior do que","valueBoxLabel":"Valor","statusTipMsg":"Faça clique na barra de filtro para filtrar os valores em ${0}.","waiRelAll":"Corresponder a todas as seguintes regras:","closeFilterBarBtn":"Fechar barra de filtro","a11yFilterBarDefButton":"Filtrar...","trueLabel":"True","conditionIsNot":"não é","conditionIs":"é","ruleTitleTemplate":"Regra ${0}","waiRemoveRuleButtonTemplate":"Remover regra ${0}","and":"and","conditionLess":"é menor do que","waiColumnSelectTemplate":"Coluna para a regra ${0}","anycolumn":"qualquer coluna","filterDefDialogTitle":"Filtro","waiConditionSelectTemplate":"Condição para a regra ${0}","waiFilterButton":"Submeter o filtro","statusTipTitle":"Barra do filtro","statusTipRelPre":"Correspondência","anyColumnOption":"Qualquer coluna","conditionSelectLabel":"Condição","cancelButton":"Cancelar","conditionNotStartWith":"não começa com","conditionEndWith":"termina com","statusTipTitleNoFilter":"Barra do filtro","columnSelectLabel":"Coluna","conditionNotContain":"não contém","clearButton":"Limpar","conditionLargerEqual":"maior ou igual","any":"qualquer","relationMsgFront":"Corresponder","statusTipHeaderColumn":"Coluna","defaultItemsName":"itens","conditionEqual":"igual","relationAny":"quaisquer regras","trueLabelEditable":"Verificado","or":"or","filterBarMsgHasFilterTemplate":"${0} de ${1} ${2} apresentado(s).","rangeTo":"a","conditionStartsWith":"começa com","waiRelAny":"Corresponder a qualquer uma das seguintes regras:","statusTipRelPost":"regras.","relationMsgTail":"","filterBarClearButton":"Limpar filtro","incompleteRuleTip":"Esta regra não está completa.","conditionLessEqual":"menor ou igual","waiFilterBarClearButton":"Limpar o filtro","conditionAfter":"após","statusTipHeaderCondition":"Regras","falseLabel":"False","filterButton":"Filtro","clearFilterDialogTitle":"Limpar filtro","statusTipTitleHasFilter":"Filtro","addRuleButton":"Adicionar regra","waiAddRuleButton":"Adicionar uma nova regra","waiClearButton":"Limpar o filtro","conditionNotEqual":"não é igual","conditionNotEndWith":"não termina com","rangeTemplate":"de ${0} a ${1}","removeRuleButton":"Remover regra","waiFilterBarDefButton":"Filtrar a tabela","conditionIsEmpty":"is empty"})