({"conditionBefore":"ก่อน","filterBarDefButton":"กำหนดตัวกรอง","filterBarMsgNoFilterTemplate":"ไม่นำตัวกรองไปใช้","conditionContains":"ประกอบด้วย","clearFilterMsg":"ซึ่งจะลบตัวกรองออกและแสดงเร็กคอร์ดที่พร้อมใช้งานทั้งหมด","all":"ทั้งหมด","waiValueBoxTemplate":"ป้อนค่าให้กับตัวกรองสำหรับกฏ ${0}","conditionRange":"ช่วง","waiCancelButton":"ยกเลิกไดอะล็อกนี้","relationAll":"กฏทั้งหมด","conditionLarger":"มากกว่า","valueBoxLabel":"ค่า","statusTipMsg":"คลิกที่แถบตัวกรองที่นี่เพื่อกรองค่าใน ${0}","waiRelAll":"ตรงกับกฏทั้งหมดต่อไปนี้:","closeFilterBarBtn":"ปิดแถบตัวกรอง","a11yFilterBarDefButton":"ตัวกรอง...","trueLabel":"จริง","conditionIsNot":"ไม่","conditionIs":"เป็น","ruleTitleTemplate":"กฏ ${0}","waiRemoveRuleButtonTemplate":"ลบกฏ ${0}","and":"และ","conditionLess":"น้อยกว่า","waiColumnSelectTemplate":"คอลัมน์สำหรับกฏ ${0}","anycolumn":"คอลัมน์ใดๆ","filterDefDialogTitle":"ตัวกรอง","waiConditionSelectTemplate":"เงื่อนไขสำหรับกฏ ${0}","waiFilterButton":"ส่งตัวกรอง","statusTipTitle":"แถบตัวกรอง","statusTipRelPre":"ตรงกับ","anyColumnOption":"คอลัมน์ใดๆ","conditionSelectLabel":"เงื่อนไข","cancelButton":"ยกเลิก","conditionNotStartWith":"ไม่เริ่มต้นด้วย","conditionEndWith":"ลงท้ายด้วย","statusTipTitleNoFilter":"แถบตัวกรอง","columnSelectLabel":"คอลัมน์","conditionNotContain":"ไม่ประกอบด้วย","clearButton":"ลบ","conditionLargerEqual":"มากกว่าหรือเท่ากับ","any":"ใด","relationMsgFront":"ตรงกับ","statusTipHeaderColumn":"คอลัมน์","defaultItemsName":"ไอเท็ม","conditionEqual":"เท่ากับ","relationAny":"กฏใดๆ","trueLabelEditable":"ได้รับการตรวจสอบ","or":"หรือ","filterBarMsgHasFilterTemplate":"${0} ของ ${1} ${2} จะถูกแสดง","rangeTo":"ถึง","conditionStartsWith":"เริ่มต้นด้วย","waiRelAny":"ตรงกับกฏใดๆต่อไปนี้:","statusTipRelPost":"กฏ","relationMsgTail":"","filterBarClearButton":"ลบตัวกรอง","incompleteRuleTip":"กฏนี้ยังไม่สมบูรณ์","conditionLessEqual":"น้อยกว่าหรือเท่ากับ","waiFilterBarClearButton":"ลบตัวกรอง","conditionAfter":"หลัง","statusTipHeaderCondition":"กฏ","falseLabel":"เท็จ","filterButton":"ตัวกรอง","clearFilterDialogTitle":"ลบตัวกรอง","statusTipTitleHasFilter":"ตัวกรอง","addRuleButton":"เพิ่มกฏ","waiAddRuleButton":"เพิ่มกฏใหม่","waiClearButton":"ลบตัวกรอง","conditionNotEqual":"ไม่เท่ากับ","conditionNotEndWith":"ไม่ลงท้ายด้วย","rangeTemplate":"จาก ${0} ถึง ${1}","removeRuleButton":"ลบกฏ","waiFilterBarDefButton":"กรองตาราง","conditionIsEmpty":"is empty"})