({"cancel":"ביטול","toDic":"הוספה למילון","skip":"דילוג","iMsg":"אין הצעות איות ","replaceAll":"החלפת הכל","widgetLabel":"בדיקת איות באצווה ","skipAll":"דילוג על הכל","suggestions":"הצעות ","iSkipAll":"דילוג על כל הערכים הדומים לערך זה ","replaceWith":"החלפה בערך ","msg":"לא נמצאו שגיאות איות ","replace":"החלפה ","unfound":"לא נמצא ","iSkip":"דילוג על ערך זה "})