// Generic events
dojo.provide('ChirpTools.Mvc.Events');

chirp.makeEvents('ChirpTools.Mvc.Events', [ 
    'ViewDestruction',
    'History.RestoreInitialState',
    'New.Controller',
    'New.View'
]);
