({"set":"Кірістіру","text":"Сипаттама","insertImageTitle":"Сурет кірістіру","invalidMessage":"Кескін файлының түрі дұрыс емес","prePopuTextBrowse":" немесе жергілікті файлға өтіңіз.","browse":"Шолу...","prePopuTextUrl":"Кескіннің URL мекен-жайын енгізіңіз","url":"Кескін"})