.feedViewerEntry {
        background: #F0F0F0;
        padding: 2px;
        overflow-x: auto;
        overflow-y: auto;
        cursor: pointer;   
}

.feedViewerEntry:hover {
        background: #a6c2e7;
        padding: 2px;
}

.feedViewerEntrySelected {
        color: #000000;
        background: #C0C0C0;
        text-align: left;
        vertical-align: top;
        padding-bottom: 2px;
        padding: 2px;
}

.feedViewerEntrySelected:hover {
        background: #a6c2e7;
        padding: 2px;
}


.feedViewerEntryUpdated {
        text-align: left;
        vertical-align: top;
        color: #909090;
        padding-bottom: 2px;
        padding: 2px;
}

.feedViewerEntryUpdatedSelected {
        text-align: left;
        vertical-align: top;
        color: #000000;
        padding-bottom: 2px;
        padding: 2px;
}


.feedViewerEntryTitle {
        color:#000000;
        text-align: left;
        text-decoration:none;
        padding-left: 4px;
        padding-bottom: 2px;
}
.feedViewerEntryTitle:hover {
        color:#000000;
        text-align: left;
        text-decoration:underline;
        padding-left: 4px;
        padding-bottom: 2px;
}

.feedViewerDeleteButton {
        cursor: pointer;
        color:#0000FF;
        text-align: left;
        font-size:  small;
}

.feedViewerDeleteButton:hover {
        text-decoration: underline;
}
