({"conditionBefore":"之前","filterBarDefButton":"定義過濾器","filterBarMsgNoFilterTemplate":"未套用任何過濾器","conditionContains":"包含","clearFilterMsg":"這將會移除過濾器並顯示所有可用的記錄。","all":"全部","waiValueBoxTemplate":"輸入要過濾規則 ${0} 的值","conditionRange":"範圍","waiCancelButton":"取消此對話框","relationAll":"所有規則","conditionLarger":"為大於","valueBoxLabel":"值","statusTipMsg":"按一下過濾器列以過濾 ${0} 中的值。","waiRelAll":"符合下列的所有規則：","closeFilterBarBtn":"關閉過濾器列","a11yFilterBarDefButton":"過濾...","trueLabel":"True","conditionIsNot":"為不是","conditionIs":"為","ruleTitleTemplate":"規則 ${0}","waiRemoveRuleButtonTemplate":"移除規則 ${0}","and":"及","conditionLess":"為小於","waiColumnSelectTemplate":"規則 ${0} 的直欄","anycolumn":"任何直欄","filterDefDialogTitle":"過濾器","waiConditionSelectTemplate":"規則 ${0} 的條件","waiFilterButton":"提交過濾器","statusTipTitle":"過濾器列","statusTipRelPre":"符合","anyColumnOption":"任何直欄","conditionSelectLabel":"條件","cancelButton":"取消","conditionNotStartWith":"不起始於","conditionEndWith":"結束於","statusTipTitleNoFilter":"過濾器列","columnSelectLabel":"直欄","conditionNotContain":"不包含","clearButton":"清除","conditionLargerEqual":"大於或等於","any":"任何","relationMsgFront":"符合","statusTipHeaderColumn":"直欄","defaultItemsName":"項目","conditionEqual":"等於","relationAny":"任何規則","trueLabelEditable":"已勾選","or":"或","filterBarMsgHasFilterTemplate":"顯示 ${1} ${2} 之 ${0}。","rangeTo":"到","conditionStartsWith":"開始於","waiRelAny":"符合下列的任何規則：","statusTipRelPost":"規則。","relationMsgTail":"","filterBarClearButton":"清除過濾器","incompleteRuleTip":"此規則不完整。","conditionLessEqual":"小於或等於","waiFilterBarClearButton":"清除過濾器","conditionAfter":"之後","statusTipHeaderCondition":"規則","falseLabel":"False","filterButton":"過濾器","clearFilterDialogTitle":"清除過濾器","statusTipTitleHasFilter":"過濾器","addRuleButton":"新增規則","waiAddRuleButton":"新增規則","waiClearButton":"清除過濾器","conditionNotEqual":"不等於","conditionNotEndWith":"不結束於","rangeTemplate":"從 ${0} 到 ${1}","removeRuleButton":"移除規則","waiFilterBarDefButton":"過濾表格","conditionIsEmpty":"is empty"})