({"emoticonLaughing":"skrattar","emoticonCool":"cool","emoticonTongue":"räcker ut tungan","emoticonCrying":"gråter","emoticonOops":"hoppsan","emoticonFrown":"rynkar pannan","emoticonAngry":"arg","emoticonShy":"blyg","emoticonNo":"nej","emoticonAngel":"ängel","smiley":"Infoga känslolägesikon","emoticonIdea":"idé","emoticonEyebrow":"lyfter på ena ögonbrynet","emoticonSmile":"ler","emoticonWink":"blinkar med ena ögat","emoticonYes":"ja","emoticonGrin":"ler stort","emoticonGoofy":"knasig","emoticonHalf":"halvt leende"})