({"emoticonLaughing":"ler","emoticonCool":"sej","emoticonTongue":"tungen ud","emoticonCrying":"græder","emoticonOops":"ups","emoticonFrown":"rynker panden","emoticonAngry":"vred","emoticonShy":"genert","emoticonNo":"nej","emoticonAngel":"engel","smiley":"Indsæt humørikon","emoticonIdea":"idé","emoticonEyebrow":"øjenbryn","emoticonSmile":"smil","emoticonWink":"blink","emoticonYes":"ja","emoticonGrin":"grin","emoticonGoofy":"skør","emoticonHalf":"halv"})