({"saveLabel":"Guardar","saveSettingdialogTitle":"Gravação automática","saveSettingdialogButtonOk":"Definir intervalo","saveMessageSuccess":"Guardado às ${0}","saveSettingdialogButtonCancel":"Cancelar","saveSettingdialogDescription":"Especificar intervalo de gravação automática","saveSettingLabelOn":"Definir intervalo de gravação automática...","saveSettingdialogParamLabel":"mín.","saveSettingdialogParamName":"Intervalo de gravação automática","saveMessageFail":"Falha ao guardar às ${0}","saveSettingLabelOff":"Desactivar gravação automática"})