({"set":"Vložit","text":"Popis","insertImageTitle":"Vložit obrázek","invalidMessage":"Neplatný typ souboru obrázku","prePopuTextBrowse":" nebo vyhledejte lokální soubor.","browse":"Procházet...","prePopuTextUrl":"Zadejte adresu URL obrázku","url":"Obrázek"})