.dojoxColorPicker {
	padding:8px;
	-moz-border-radius:4pt;
	-webkit-border-radius:5pt;
	-webkit-drop-shadow:3pt;
}

.dojoxColorPickerRightPad {
	padding-right: 8px;
}

.tundra .dojoxColorPicker {
	background:#ededed;
	border:1px solid #ccc;
}

.dojoxColorPickerBox {
	position:relative;
	width:150px;	
	height:150px;	
	margin:0; 
	padding:0;
}

.dojoxColorPickerUnderlay {
	position:relative; 
	top:0; left:0;
	width:150px; 
	height:150px;
	z-index:1;
}
.tundra .dojoxColorPickerUnderlay {
	border:1px solid #a0a0a0;
}

.claro .dojoxColorPicker {
	background:#ededed;
	border:1px solid #cdcdcd;
}

.claro .dojoxColorPickerUnderlay {
	border:1px solid #cccccc;
}

.dojoxHuePickerUnderlay {
	position:relative;
	top:0; left:0;
	height:150px;
	width:20px;
	z-index:1;
	text-align: center;
}

.dojoxHuePicker { position:relative; top: 0px; left: 0px; padding: 0px;}

.dojoxHuePickerPoint {
	position:absolute;
	top:0; left:0;
	width:20px;
	height:8px;
	z-index:3; 
	cursor:move;
}

.dojoxColorPickerPoint {
	position:absolute;
	width:10px; 
	height:10px;
	border:0;
	z-index:3; 
	cursor:move; 
}

.dojoxColorPickerPreview {
	display:block;	
	width:45px;	
	height:45px;
	border:1px solid #333;
	background-color:#fff; 
	position:relative;
	top: 0px;
	left: 0px; 
}
.dojoxColorPickerWebSafePreview {
	display:block;
	width:25px; 
	height:25px; 
	position:relative;
	top: 0px;
	left: 0px; 
	border:1px solid #333; 
}

.dojoxColorPickerOptional {
	position:relative;
	top: 0px;
	left: 0px;
	height: 100%;
}

.dojoxColorPickerOptional table {
	border-spacing: 4px;
}

.dojoxColorPickerPreviewContainer table {
	border-spacing: 6px 0px;
}

.dojoxColorPickerOptional input {
	border:1px solid #a7a7a7;
	width:25px;
	padding:1px 3px 1px 3px;
	line-height:1.1em;
}

.dojoxColorPickerHex input {
	width:55px;	
}
