({"selectContents":"Velg innhold","deleteElement":"Slett element","moveEnd":"Flytt markør til slutt","deleteContents":"Slett innhold","nodeActions":"${nodeName} Handlinger","selectElement":"Velg element","moveStart":"Flytt markør til start"})