({"cancel":"Ακύρωση","toDic":"Προσθήκη στο λεξικό","skip":"Παράλειψη","iMsg":"Δεν υπάρχουν προτάσεις ορθογραφίας","replaceAll":"Αντικατάσταση όλων","widgetLabel":"Συνολικός ορθογραφικός έλεγχος","skipAll":"Παράλειψη όλων","suggestions":"Προτάσεις","iSkipAll":"Παράλειψη όλων των όμοιων","replaceWith":"Αντικατάσταση με","msg":"Δεν βρέθηκαν ορθογραφικά λάθη","replace":"Αντικατάσταση","unfound":"Δεν εντοπίστηκε","iSkip":"Παράλειψη τρέχοντος"})