({"set":"Вставить","text":"Описание","insertImageTitle":"Вставить изображение","invalidMessage":"Недопустимый тип файла изображения","prePopuTextBrowse":" или выберите локальный файл.","browse":"Обзор...","prePopuTextUrl":"Введите URL изображения","url":"Изображение"})