({"emoticonLaughing":"riso","emoticonCool":"com estilo","emoticonTongue":"língua de fora","emoticonCrying":"a chorar","emoticonOops":"ups","emoticonFrown":"triste","emoticonAngry":"zangado","emoticonShy":"tímido","emoticonNo":"não","emoticonAngel":"anjo","smiley":"Inserir ícone emotivo","emoticonIdea":"ideia","emoticonEyebrow":"desconfiado","emoticonSmile":"sorriso","emoticonWink":"piscadela","emoticonYes":"sim","emoticonGrin":"gargalhada","emoticonGoofy":"engraçado","emoticonHalf":"céptico"})