({"set":"挿入","text":"説明","insertImageTitle":"イメージの挿入","invalidMessage":"無効なイメージ・ファイル・タイプです","prePopuTextBrowse":"ローカル・ファイルを参照してください。","browse":"参照...","prePopuTextUrl":"イメージ URL を入力するか、","url":"イメージ"})