({"cancel":"Cancel","toDic":"เพิ่งลงในพจนานุกรม","skip":"ข้าม","iMsg":"ไม่มีการแนะนำการสะกด","replaceAll":"แทนที่ทั้งหมด","widgetLabel":"ตรวจสอบการสะกดคำแบบแบตช์","skipAll":"ข้ามทั้งหมด","suggestions":"การแนะนำ","iSkipAll":"ข้ามที่เหมือนนี้","replaceWith":"แทนที่ด้วย","msg":"ไม่พบคำที่สะกดผิด","replace":"แทนที่","unfound":"ไม่พบ","iSkip":"ข้ามนี้"})