/**********************************oaf.base.dnd************************************/
.dragNode{
	z-index:10000;
	cursor:move;
}
.dragHandle{
	cursor:move;
}
.dropIndicator{
	position:relative;
	height:50px;
	border:1px dashed #FFF;
}
.dropIndicator > div{
	position:absolute;
	left:0;top:0;right:0;bottom:0;	
	background-color:#000;
	opacity:0.17; 
	filter:alpha(opacity=17); 
	-khtml-opacity:0.17; 
	-moz-opacity:0.17;
}
.dj_ie6 .dropIndicator div{
	position:relative;
	height:100%;
	background-color:#000;
	filter:alpha(opacity=17); 
}
.dj_ie6 .dropIndicator{
	_margin:10px;
}
.dndCover{
	background-image:url(../../layout/resources/icons/pixel.gif);
	position:absolute;
	left:0;
	top:0;
	width:100%;
	height:1000px;
	z-index:9999;
	cursor:move;
}

.dndCover2{
	z-index: 10001;
}
.dragNode{
	z-index:10000;
	cursor:move;
}
.dragHandle{
	cursor:move;
}
.dropIndicator{
	position:relative;
	height:50px;
	border:1px dashed #FFF;
}
.dropIndicator > div{
	position:absolute;
	left:0;top:0;right:0;bottom:0;	
	background-color:#000;
	opacity:0.17; 
	filter:alpha(opacity=17); 
	-khtml-opacity:0.17; 
	-moz-opacity:0.17;
}
.dj_ie6 .dropIndicator div{
	position:relative;
	height:100%;
	background-color:#000;
	filter:alpha(opacity=17); 
}
.dj_ie6 .dropIndicator{
	_margin:10px;
}
.dndCover{
	background-image:url(data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw%3D%3D);
	position:absolute;
	left:0;
	top:0;
	width:100%;
	height:1000px;
	z-index:9999;
	cursor:move;
}

.dj_ie6 .orange .dndCover, .dj_ie7 .orange .dndCover{
	background-image:url(../../images/oaf/pixel.gif);
}

.orange .dndCover2{
	z-index: 10001;
}