({"cancel":"الغاء","toDic":"اضافة الى القاموس","skip":"تخطي","iMsg":"لا توجد اقتراحات للهجاء","replaceAll":"استبدال كل","widgetLabel":"فحص هجاء دفعي","skipAll":"تخطي كل","suggestions":"اقتراحات","iSkipAll":"تخطي كل المماثل لهذا","replaceWith":"استبدال مع","msg":"لا يوجد أخطاء في الهجاء","replace":"استبدال","unfound":"غير موجودة","iSkip":"تخطي هذا"})