({"buttonSet":"Ορισμός","insertTableTitle":"Εισαγωγή πίνακα","insertTableRowAfterLabel":"Προσθήκη γραμμής μετά","buttonCancel":"Ακύρωση","center":"κέντρο","deleteTableColumnLabel":"Διαγραφή στήλης","right":"δεξιά","insertTableColumnBeforeLabel":"Προσθήκη στήλης πριν","tableWidth":"Πλάτος πίνακα:","buttonInsert":"Εισαγωγή","default":"προεπιλογή","align":"Στοίχιση:","insertTableRowBeforeLabel":"Προσθήκη γραμμής πριν","cellSpacing":"Απόσταση κελιών","pixels":"εικονοστοιχεία","selectTableLabel":"Επιλογή πίνακα","rows":"Γραμμές:","modifyTableTitle":"Τροποποίηση πίνακα","cellPadding":"Περιθώριο κελιών:","deleteTableRowLabel":"Διαγραφή γραμμής","backgroundColor":"Χρώμα φόντου:","insertTableColumnAfterLabel":"Προσθήκη στήλης μετά","left":"αριστερά","borderThickness":"Πάχος περιγράμματος","columns":"Στήλες:","percent":"ποσοστό","borderColor":"Χρώμα περιγράμματος:"})