({"cancel":"Mégse","toDic":"Hozzáadás a szótárhoz","skip":"Kihagyja","iMsg":"Nincsenek helyesírási javaslatok","replaceAll":"Mindent lecserél","widgetLabel":"Kötegelt helyesírás-ellenőrzés","skipAll":"Mindet kihagyja","suggestions":"Javaslatok","iSkipAll":"Összes hasonlót kihagyja","replaceWith":"Csere erre","msg":"Nem található helyesírási hiba","replace":"Csere","unfound":"Nem található","iSkip":"Ezt kihagyja"})