({"replaceDialogText":"${0} ocorrências substituídas.","eofDialogTextFind":"localizado","eofDialogText":"Última ocorrência ${0}","backwards":"Retroceder","replaceButton":"Substituir","replaceLabel":"Substituir por:","matchCase":"Coincidir maiúscula/minúscula","findTooltip":"Inserir texto a ser localizado","replaceTooltip":"Inserir texto a ser substituído por","replaceAllButton":"Substituir Todos","eofDialogTextReplace":"substituído","findReplace":"Localizar e Substituir","backwardsTooltip":"Procurar texto para trás","replaceAllButtonTooltip":"Substituir todo o texto","replaceButtonTooltip":"Substituir o texto","findLabel":"Localizar:","findButton":"Localizar","matchCaseTooltip":"Coincidir maiúscula/minúscula","findButtonTooltip":"Localizar o texto","replaceAll":"Todas as Ocorrências"})