/* Claro styles for DataGrid */
@import url("Grid.css");

.claro .dojoxGrid {
	margin:0px;
	padding:0px;
	border-collapse:collapse;	
	background-color:#fff;	
}
/*  messages */
.claro .dojoxGridMasterMessages {
	background-color: #fefefe;
}
.claro .dojoxGridLoading, 
.claro .dojoxGridError {
	background-position:left center;
	background-repeat: no-repeat;
	padding-left:25px;
}
.claro .dojoxGridLoading {
	background-image:url("../../../dijit/themes/claro/images/loadingAnimation.gif");
}
.claro .dojoxGridError {
	background-image: url('../../../dijit/icons/images/commonIconsObjActEnabled.png'); 
	background-position: -496px;
	width: 16px;
	height: 16px;
}

/* header */
.claro .dojoxGridHeader {
	background-color:#FFF;
}
.claro .dojoxGridHeader .dojoxGridCell { 
	background:url("images/header.png") #e5edf4 repeat-x top;
	border-style:solid;
	border-width:1px;
	border-color:#BCBCBC #BCBCBC #BCBCBC transparent;
}
.dj_ie6 .claro .dojoxGridHeader .dojoxGridCell { 
	border-color:#BCBCBC #BCBCBC #BCBCBC #e5edf4;
}

/* header sorting arrow */
.claro .dojoxGridSortNode {
	background:url("images/header_shadow.png") #e5edf4 repeat-x bottom;
	text-decoration:none;
	display:block;
	padding:4px 6px 5px 6px;
	border:1px solid #e0eefb;
}
.dj_ie6 .claro .dojoxGridSortNode {
	background-image:none;
}
.claro .dojoxGridCellOver .dojoxGridSortNode {
	background-color:#9dcfff;
}
.claro .dojoxGridArrowButtonChar {
	display:none;
	float:right;
}
.claro .dojoxGridArrowButtonNode {
	background:transparent url("../../../dijit/themes/claro/images/spriteArrows.png") no-repeat scroll left center;
	display:block;
	float:right;
	height:1em;
	margin:2px 4px 0 5px;
	padding-left:0;
	width:7px;
}
.claro .dojoxGridSortUp .dojoxGridArrowButtonNode {
	background-position:-21px 50%;
}

/* header rows */
.dj_ie .claro .dojoxGridHeader .dojoxGridRowTable {
	border-collapse:separate;
}

/* Grid view content */
.claro .dojoxGridScrollbox {
	background-color: #fefefe;
}

/* rowbar - row headers */
.claro .dojoxGridRowbar { 
	background:url("images/header.png") #e5edf4 repeat-x top;
	border:none;
	border-right:1px solid #BCBCBC;
}
.claro .dojoxGridRowbarTable { 
	background:transparent url("images/header_shadow.png") repeat-x scroll center bottom;
}
.dj_ie6 .claro .dojoxGridRowbar,
.dj_ie6 .claro .dojoxGridRowbarTable { 
	background-image:none;
}
.claro .dojoxGridRowbarInner {
	border:none;
	border-bottom:1px solid #ccc;
}
.claro .dojoxGridRowbarOver .dojoxGridRowbarTable {
	background-color:#abd5fd;
}
.claro .dojoxGridRowBarActive .dojoxGridRowbarTable {
	background-color:#91c9fe;
}
.claro .dojoxGridRowbarSelected {
	background-color:#abd5fd;
	border-right:1px solid #ccc;
}

/* rows */
.claro .dojoxGridRow {
	border:none;
	background-color:#fff;
}
.dj_ie .claro .dojoxGridMasterView .dojoxGridRowTable {
	border-collapse:separate;/*IE, separate is must to show the border of tr and td*/
}
.claro .dojoxGridRowTable tr {
	background:url("images/row_back.png") #fff repeat-x;
}
.claro .dojoxGridRowOdd .dojoxGridRowTable tr {
	background-color:#f7fcff;
}
.claro .dojoxGridRowSelected .dojoxGridRowTable tr {
	background-color:#cee6fa;
}

/* cells */
.claro .dojoxGridCell {
	padding:0px;
	border:1px solid transparent;
}
.dj_ie6 .claro .dojoxGridCell {
	border-color:#fff
}
.dj_ie6 .claro .dojoxGridRowOdd .dojoxGridCell {
	border-left-color:#f4f9fd;
	border-right-color:#f4f9fd;
}
.dj_ie6 .claro .dojoxGridRowSelected .dojoxGridCell {
	border-left-color:#d3e9fb;
	border-right-color:#d3e9fb;
}
.claro .dojoxGridRowSelected .dojoxGridCell {
	border-top:1px solid #BFD6EB;
	border-bottom:1px solid #BFD6EB;
}
.claro .dojoxGridCellFocus {
	border:1px dashed darkblue !important;
}
.claro .dojoxGridCellContent {
	padding:4px 6px 4px 6px;
}

/*  Single Affordance Hover Effect */
.claro .dojoxGridRowOver .dojoxGridCell {
	background:url("images/row_back.png") #ABD6FF repeat-x;
	border-top:1px solid #769DC0;
	border-bottom:1px solid #769DC0;
}
.dj_ie6 .claro .dojoxGridRowOver .dojoxGridCell,
.dj_ie7 .claro .dojoxGridRowOver .dojoxGridCell {
	border-left:1px solid #ABD6FF;
	border-right:1px solid #ABD6FF;
}
.claro .dojoxGridRowActive .dojoxGridCell {
	background:url("images/td_button_down.png") #7DBEFA repeat-x;
}
.dj_ie6 .claro .dojoxGridRowActive .dojoxGridCell,
.dj_ie7 .claro .dojoxGridRowActive .dojoxGridCell {
	border-left:1px solid #7DBEFA;
	border-right:1px solid #7DBEFA;	
}

/*  Double Affordance Hover Effect */
.claro .dojoxGridDoubleAffordance .dojoxGridRowOver .dojoxGridCellOver {
	border:solid 1px #769dc0;
	background-color:#93cafe;
	border-collapse:separate;/*FF*/
}
.claro .dojoxGridDoubleAffordance .dojoxGridRowActive .dojoxGridCell{	
	background-image:url("images/row_back.png");
}
.claro .dojoxGridDoubleAffordance .dojoxGridRowActive .dojoxGridCellActive {	
	background:url("images/td_button_down.png") #93cafe repeat-x;
}
.dj_ie6 .claro .dojoxGridCell {
	background-image:none !important;
}

/* editing */
.claro .dojoxGridRowEditing td {
	/* background-color: #F4FFF4; */
	background-color: #cee6fa; 
	/* padding: 0px 3px 0px 3px; */
}
.claro .dojoxGridRow-inserting td {
	background-color: #F4FFF4;
}
.claro .dojoxGridRow-inflight td {
	background-color: #F2F7B7;
}
.claro .dojoxGridRow-error td {
	background-color: #F8B8B6;
}

/* Drag and Drop */
.claro .dojoxGrid .dojoDndItemBefore {
	border-left-color: #3559ac;
}
.claro .dojoxGrid .dojoDndItemAfter {
	border-right-color: #3559ac;
}

/* Tree Grid */
.claro .dojoxGridExpando {
	float: left;
	height: 18px;
	width: 18px;
	text-align: center;
	margin-top: -3px;
}
.dijitRtl .claro .dojoxGridExpando {
	float: right;
}
.claro .dojoxGridExpandoCell {
	padding-top: 5px;
	background-position: left top !important;
}
.claro .dojoxGridExpandoNode {
	background-image: url('../../../dijit/themes/claro/images/treeExpandImages.png');
    width: 16px;
    height: 16px;
	background-position: 1px 0px; /* for closed state */
}
.dj_ie6 .claro .dojoxGridExpandoNode {
	background-image: url('../../../dijit/themes/claro/images/treeExpandImages8bit.png');
}
.claro .dojoxGridRowOver .dojoxGridExpandoNode {
	background-position: -17px 0px;
}
.claro .dojoxGridExpandoOpened .dojoxGridExpandoNode {
	background-position: -35px 0px;
}
.claro .dojoxGridRowOver .dojoxGridExpandoOpened .dojoxGridExpandoNode {
	background-position: -53px 0px;
}
.claro .dojoxGridExpandoLoading .dojoxGridExpandoNode {
	background-image: url('../../../dijit/themes/claro/images/loadingAnimation.gif');
}
.claro .dojoxGridTreeModel .dojoxGridNoChildren .dojoxGridExpando {
	visibility: visible !important;
	width: 18px !important;
}
.claro .dojoxGridTreeModel .dojoxGridNoChildren .dojoxGridExpandoNode,
.dj_ie6 .claro .dojoxGridTreeModel .dojoxGridNoChildren .dojoxGridExpandoNode {
	background-image:none;
}
.claro .dojoxGridExpandoNodeInner {
	visibility: hidden;
}
.dijit_a11y .dojoxGridExpandoNodeInner {
	visibility: visible;
}

.claro .dojoxGridSummaryRow .dojoxGridCell {
	border:1px solid transparent;
}
.dj_ie6 .claro .dojoxGridSummaryRow .dojoxGridCell {
	border-color:#fff
}
.claro tr.dojoxGridSubRowAlt {
	background-color:#f4f9fd;
}
.claro .dojoxGridRowOdd tr.dojoxGridSubRowAlt {
	background-color:#fff;
}
.claro .dojoxGridRow .dojoxGridRowTable tr.dojoxGridRowSelected {
	background-color:#cee6fa;
}
