({"conditionBefore":"antes","filterBarDefButton":"Definir filtro","filterBarMsgNoFilterTemplate":"Nenhum filtro aplicado","conditionContains":"contém","clearFilterMsg":"Isso removerá o filtro e mostrará todos os registros disponíveis.","all":"todos","waiValueBoxTemplate":"Insira o valor para filtragem da regra ${0}","conditionRange":"intervalo","waiCancelButton":"Cancelar este diálogo","relationAll":"todas as regras","conditionLarger":"é maior que","valueBoxLabel":"Valor","statusTipMsg":"Clique na barra de filtragem aqui para filtrar os valores de ${0}.","waiRelAll":"Corresponder a todas as seguintes regras:","closeFilterBarBtn":"Fechar a barra de filtragem","a11yFilterBarDefButton":"Filtrar...","trueLabel":"Verdadeiro","conditionIsNot":"não é","conditionIs":"é","ruleTitleTemplate":"Regra ${0}","waiRemoveRuleButtonTemplate":"Remover regra ${0}","and":"e","conditionLess":"é menor que","waiColumnSelectTemplate":"Coluna para a regra ${0}","anycolumn":"qualquer coluna","filterDefDialogTitle":"Filtrar","waiConditionSelectTemplate":"Condição para a regra ${0}","waiFilterButton":"Enviar o filtro","statusTipTitle":"Barra de Filtragem","statusTipRelPre":"Corresponder","anyColumnOption":"Qualquer Coluna","conditionSelectLabel":"Condição","cancelButton":"Cancelar","conditionNotStartWith":"não inicia com","conditionEndWith":"termina com","statusTipTitleNoFilter":"Barra de Filtragem","columnSelectLabel":"Coluna","conditionNotContain":"não contém","clearButton":"Limpar","conditionLargerEqual":"maior ou igual a","any":"qualquer um","relationMsgFront":"Corresponder","statusTipHeaderColumn":"Coluna","defaultItemsName":"itens","conditionEqual":"igual","relationAny":"qualquer regra","trueLabelEditable":"Verificado","or":"ou","filterBarMsgHasFilterTemplate":"${0} de ${1} ${2} mostrados.","rangeTo":"a","conditionStartsWith":"inicia com","waiRelAny":"Corresponder a qualquer uma das seguintes regras:","statusTipRelPost":"regras.","relationMsgTail":"","filterBarClearButton":"Limpar filtro","incompleteRuleTip":"Esta regra não está completa.","conditionLessEqual":"menor ou igual a","waiFilterBarClearButton":"Limpar o filtro","conditionAfter":"depois","statusTipHeaderCondition":"Regras","falseLabel":"Falso","filterButton":"Filtrar","clearFilterDialogTitle":"Limpar Filtro","statusTipTitleHasFilter":"Filtrar","addRuleButton":"Incluir Regra","waiAddRuleButton":"Incluir uma nova regra","waiClearButton":"Limpar o filtro","conditionNotEqual":"não é igual","conditionNotEndWith":"não termina com","rangeTemplate":"de ${0} a ${1}","removeRuleButton":"Remover Regra","waiFilterBarDefButton":"Filtrar a tabela","conditionIsEmpty":"is empty"})