({"conditionBefore":"avant","filterBarDefButton":"Définir le filtre","filterBarMsgNoFilterTemplate":"Aucun filtre appliqué","conditionContains":"contient","clearFilterMsg":"Le filtre sera supprimé et tous les enregistrements disponibles affichés.","all":"tout","waiValueBoxTemplate":"Entrez une valeur pour filtrer la règle ${0}","conditionRange":"plage","waiCancelButton":"Annuler cette boîte de dialogue","relationAll":"toutes les règles","conditionLarger":"supérieur à","valueBoxLabel":"Valeur","statusTipMsg":"Cliquez sur cette barre de filtre pour filtrer les valeurs de ${0}.","waiRelAll":"Répondre à toutes les règles suivantes :","closeFilterBarBtn":"Fermer la barre de filtre","a11yFilterBarDefButton":"Filtrer...","trueLabel":"Vrai","conditionIsNot":"n'est pas","conditionIs":"est","ruleTitleTemplate":"Règle ${0}","waiRemoveRuleButtonTemplate":"Supprimer la règle ${0}","and":"et","conditionLess":"inférieur à","waiColumnSelectTemplate":"Colonne pour la règle ${0}","anycolumn":"toute colonne","filterDefDialogTitle":"Filtrer","waiConditionSelectTemplate":"Condition pour la règle ${0}","waiFilterButton":"Soumettre le filtre","statusTipTitle":"Barre de filtre","statusTipRelPre":"Correspondance","anyColumnOption":"Toute colonne","conditionSelectLabel":"Condition","cancelButton":"Annuler","conditionNotStartWith":"ne commence pas par","conditionEndWith":"se termine par","statusTipTitleNoFilter":"Barre de filtre","columnSelectLabel":"Colonne","conditionNotContain":"ne contient pas","clearButton":"Effacer","conditionLargerEqual":"supérieur ou égal à","any":"n'importe lequel","relationMsgFront":"Répondre à","statusTipHeaderColumn":"Colonne","defaultItemsName":"éléments","conditionEqual":"égal à","relationAny":"n'importe quelle règle","trueLabelEditable":"Vérifié","or":"ou","filterBarMsgHasFilterTemplate":"${0} sur ${1} ${2} montrés.","rangeTo":"à","conditionStartsWith":"commence par","waiRelAny":"Répondre à l'une des règles suivantes :","statusTipRelPost":"règles.","relationMsgTail":"","filterBarClearButton":"Effacer le filtre","incompleteRuleTip":"Cette règle n'est pas complète.","conditionLessEqual":"inférieur ou égal à","waiFilterBarClearButton":"Effacer le filtre","conditionAfter":"après","statusTipHeaderCondition":"Règles","falseLabel":"Faux","filterButton":"Filtrer","clearFilterDialogTitle":"Effacer le filtre","statusTipTitleHasFilter":"Filtrer","addRuleButton":"Ajouter une règle","waiAddRuleButton":"Ajouter une nouvelle règle","waiClearButton":"Effacer ce filtre","conditionNotEqual":"différent de","conditionNotEndWith":"ne se termine pas","rangeTemplate":"de ${0} à ${1}","removeRuleButton":"Supprimer une règle","waiFilterBarDefButton":"Filtrer la table","conditionIsEmpty":"is empty"})