html, body{
	width:100%;
	height:100%;
	font-family:sans-serif;
}

.textAnnotation{
	position:absolute;
	border:1px solid #ccc;
	background-color:#ffff00;
	color:#0000ff;
	padding:3px;
	font-family:sans-serif;
	font-size:12px;
	margin-top:-10px;
	z-index:501;
}

#conEdit{
	border:0;
	/*background:#ccc;
	border:1px solid #ff0000;
	outline:none;*/
	padding:3px;
	margin:0;
}

.drawingTooltip{
	font-family:sans-serif;
	font-family:sans-serif;
	font-weight:normal;
	padding:5px;
}
.drawingTipTitle{
	font-size:12px;
	font-weight:bold;
}
.drawingTipDesc{
	font-size:11px;
	padding-left:10px;
}