({"saveLabel":"Uložit","saveSettingdialogTitle":"Automatické uložení","saveSettingdialogButtonOk":"Nastavit interval","saveMessageSuccess":"Uloženo v ${0}","saveSettingdialogButtonCancel":"Storno","saveSettingdialogDescription":"Určit interval pro automatické uložení","saveSettingLabelOn":"Nastavit interval pro automatické uložení","saveSettingdialogParamLabel":"min","saveSettingdialogParamName":"Interval pro automatické uložení","saveMessageFail":"Selhalo uložení v ${0}","saveSettingLabelOff":"Vypnout automatické uložení"})