({"saveLabel":"Speichern","saveSettingdialogTitle":"Automatisch speichern","saveSettingdialogButtonOk":"Intervall festlegen","saveMessageSuccess":"Gespeichert um ${0}","saveSettingdialogButtonCancel":"Abbrechen","saveSettingdialogDescription":"Intervall für automatisches Speichern angeben","saveSettingLabelOn":"Intervall für automatisches Speichern festlegen","saveSettingdialogParamLabel":"Min.","saveSettingdialogParamName":"Intervall für automatisches Speichern","saveMessageFail":"Konnte nicht um ${0} gespeichert werden","saveSettingLabelOff":"Automatisches Speichern inaktivieren"})