.mblBlueMinusButton {
	background-image: url(compat/small-blue-button-bg.png);
}
.mblDarkBlueMinusButton {
	background-image: url(compat/small-darkblue-button-bg.png);
}
.mblRedMinusButton {
	background-image: url(compat/small-red-button-bg.png);
}
.mblBluePlusButton {
	background-image: url(compat/small-blue-button-bg.png);
}
.mblDarkBluePlusButton {
	background-image: url(compat/small-darkblue-button-bg.png);
}
.mblRedPlusButton {
	background-image: url(compat/small-red-button-bg.png);
}
.mblCheckOnButton {
	background-image: url(compat/check-on-button.png);
}
.mblCheckOnButton DIV, .mblCheckOnButton P {
	display: none;
}
.mblCheckOffButton {
	background-image: url(compat/check-off-button.png);
}
.mblCheckOffButton DIV, .mblCheckOffButton P {
	display: none;
}
