.dojoxListInput {
	border:1px solid #ccc;
	background-color:#efefef;
	height:auto !important;
	max-height:60px;
	min-height:20px;
	overflow:auto;
	position:relative
}
.dojoxListInput ul {
	margin:0;
	padding:0
}
.dijitDialogCloseIcon {
	display:none;
}
.dojoxListInputClosable .dijitDialogCloseIcon{
	display:inline 
}
.dojoxListInputClosable {
	padding-right:18px !important
}
.dojoxListInputItem {
	float:left;
	list-style-type:none;
	margin:1px 5px 1px 1px;
	padding:0
}
.dojoxListInputItem .closeText {
	display:none;
	position:absolute;
}
.dojoxListInputItem .dijitDialogCloseIcon{
	right:auto !important;
	border:0 !important;
	padding:0!important;
}
.dojoxListInputMatch {
	border :1px solid #5EB55E;
	background-color:#efffef
}
.dojoxListInputMismatch {
	border :1px solid #B55E5E;
	background-color:#ffefef
}
.dojoxListInput.dojoxListInputFocused {
	border:1px solid #000;
	border-right:1px solid #ccc;
	border-bottom:1px solid #ccc;
	background-color:#fff
}
.dojoxListInputNode {
	cursor:text;
}
.dojoxListInput .dijitTextBox {
	background:#efefef !important;

	border:0 !important;
}
.dojoxListInputFocused .dijitTextBox {
	background: #fff !important;
	border:0 !important;
}
.dojoxListInputItem .dijitTextBox {
	width:auto !important;
	border:0 !important;
}
.dojoxListInputNode {
	float:none;
}
.dojoxListInputItemEdited{
	display:none !important
}
.dojoxListInputItem .dijitInline{
	display:inline !important
}
