<div class="dijitReset dijitInline dojoxCalc"
><table class="dijitReset dijitInline dojoxCalcLayout" dojoAttachPoint="calcTable" rules="none" cellspacing=0 cellpadding=0 border=0>
	<tr
		><td colspan="4" class="dojoxCalcInputContainer"
			><input dojoType="dijit.form.TextBox" dojoAttachEvent="onBlur:onBlur,onKeyPress:onKeyPress" dojoAttachPoint='textboxWidget'
		/></td
	></tr>
	<tr>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="seven" label="7" value='7' dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="eight" label="8" value='8' dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="nine" label="9" value='9' dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="divide" label="/" value='/' dojoAttachEvent='onClick:insertOperator' />
		</td>
	</tr>
	<tr>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="four" label="4" value='4' dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="five" label="5" value='5' dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="six" label="6" value='6' dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="multiply" label="*" value='*' dojoAttachEvent='onClick:insertOperator' />
		</td>
	</tr>
	<tr>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="one" label="1" value='1' dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="two" label="2" value='2' dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="three" label="3" value='3' dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="add" label="+" value='+' dojoAttachEvent='onClick:insertOperator' />
		</td>
	</tr>
	<tr>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="decimal" label="." value='.' dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="zero" label="0" value='0' dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="equals" label="x=y" value='=' dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcMinusButtonContainer">
			<span dojoType="dijit.form.ComboButton" dojoAttachPoint="subtract" label='-' value='-' dojoAttachEvent='onClick:insertOperator'>

				<div dojoType="dijit.Menu" style="display:none;">
					<div dojoType="dijit.MenuItem" dojoAttachEvent="onClick:insertMinus">
						(-)
					</div>
				</div>
			</span>
		</td>
	</tr>
	<tr>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="clear" label="Clear" dojoAttachEvent='onClick:clearText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="sqrt" label="&#x221A;" value="&#x221A;" dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="power" label="^" value="^" dojoAttachEvent='onClick:insertOperator' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="comma" label="," value=',' dojoAttachEvent='onClick:insertText' />
		</td>
	</tr>
	<tr>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="AnsButton" label="Ans" value="Ans" dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="LeftParenButton" label="(" value="(" dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="RightParenButton" label=")" value=")" dojoAttachEvent='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button dojoType="dijit.form.Button" dojoAttachPoint="enter" label="Enter" dojoAttachEvent='onClick:parseTextbox' />
		</td>
	</tr>
</table>
<span dojoAttachPoint="executor" dojoType="dojox.calc._Executor" dojoAttachEvent="onLoad:executorLoaded"></span>
</div>
