({"cancel":"Abbrechen","toDic":"Zum Wörterbuch hinzufügen","skip":"Überspringen","iMsg":"Keine Rechtschreibvorschläge","replaceAll":"Global ersetzen","widgetLabel":"Batchrechtschreibprüfung","skipAll":"Alle überspringen","suggestions":"Vorschläge","iSkipAll":"Alle entsprechenden Funde überspringen","replaceWith":"Ersetzen durch","msg":"Keine Rechtschreibfehler gefunden","replace":"Ersetzen","unfound":"Nicht gefunden","iSkip":"Diesen Fund überspringen"})