({"saveLabel":"บันทึก","saveSettingdialogTitle":"บันทึกอัตโนมัติ","saveSettingdialogButtonOk":"ตั้งช่วงเวลา","saveMessageSuccess":"ถูกบันทึกเมื่อ ${0}","saveSettingdialogButtonCancel":"ยกเลิก","saveSettingdialogDescription":"ระบุช่วงเวลาบันทึกอัตโนมัติ","saveSettingLabelOn":"ตั้งช่วงเวลาบันทึกอัตโนมัติ...","saveSettingdialogParamLabel":"นาที","saveSettingdialogParamName":"ช่วงเวลาบันทึกอัตโนมัติ","saveMessageFail":"ล้มเหลวในการบันทึกเมื่อ ${0}","saveSettingLabelOff":"ปิดบันทึกอัตโนมัติ"})