.dojoxCollapsibleToolbarContainer {
	border-collapse: collapse;
	padding: 0px;
	margin: 0px;
	border-style: none;
}

.dojoxCollapsibleToolbarCollapse {
	cursor: pointer;
}

.dojoxCollapsibleToolbarExpand {
	cursor: pointer;
}

.claro .dojoxCollapsibleToolbarCollapse {
	background-image: url(../images/claro/collapse.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 12px;
	height: 12px;
}

.claro .dojoxCollapsibleToolbarExpand {
	background-image: url(../images/claro/expand.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 12px;
	height: 12px;
}

.claro .dojoxCollapsibleToolbarContainer {
	background-color: #F1F1F1;
}

.claro .dojoxCollapsibleToolbarControl {
	vertical-align: top;
	border-bottom: 1px solid #ccc;
	border-left: 1px solid #ccc;
	border-right: 1px solid #ccc;
        margin-bottom: 1px;
}


.tundra .dojoxCollapsibleToolbarCollapse {
	background-image: url(../images/tundra/collapse.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 12px;
	height: 12px;
}

.tundra .dojoxCollapsibleToolbarExpand {
	background-image: url(../images/tundra/expand.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 12px;
	height: 12px;
}

.tundra .dojoxCollapsibleToolbarContainer {
	background-image: url(../images/tundra/sidebar.gif);
	background-repeat: repeat-x;
	background-position: top left;
	background-color: #CCCCCC;
}

.tundra .dojoxCollapsibleToolbarControl {
	vertical-align: top;
	border-bottom: 1px solid #ccc;
	border-left: 1px solid #ccc;
	border-right: 1px solid #ccc;
}

.soria .dojoxCollapsibleToolbarCollapse {
	background-image: url(../images/soria/collapse.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 12px;
	height: 12px;
}

.soria .dojoxCollapsibleToolbarExpand {
	background-image: url(../images/soria/expand.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 12px;
	height: 12px;
}

.soria .dojoxCollapsibleToolbarContainer {
	background-image: url(../images/soria/sidebar.gif);
	background-repeat: repeat-x;
	background-position: top left;
	background-color: #CCCCCC;
}

.soria .dojoxCollapsibleToolbarControl {
	vertical-align: top;    
	border-bottom: 1px solid #ccc;
	border-left: 1px solid #ccc;
	border-right: 1px solid #ccc;
}

.nihilo .dojoxCollapsibleToolbarCollapse {
	background-image: url(../images/nihilo/collapse.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 12px;
	height: 12px;
}

.nihilo .dojoxCollapsibleToolbarExpand {
	background-image: url(../images/nihilo/expand.gif);
	background-repeat: no-repeat;
	background-position: top left;
	width: 12px;
	height: 12px;
}

.nihilo .dojoxCollapsibleToolbarContainer {
	background-image: url(../images/nihilo/sidebar.gif);
	background-repeat: repeat-x;
	background-position: top left;
	background-color: #CCCCCC;
}

.nihilo .dojoxCollapsibleToolbarControl {
	vertical-align: top;
	border-bottom: 1px solid #ccc;
	border-left: 1px solid #ccc;
	border-right: 1px solid #ccc;
}

.dijit_a11y .dojoxCollapsibleToolbarCollapse {
	border-style: outset;
	border-width: 3px;
	border-color: black;
	width: 1.5em;
	height: 100%;
	text-align: center;
	vertical-align: middle;
}

.dijit_a11y .dojoxCollapsibleToolbarExpand {
	border-style: outset;
	border-width: 3px;
	border-color: black;
	width: 1.5em;
	height: 100%;
	text-align: center;
	vertical-align: middle;
}

.dojoxCollapsibleToolbarCollapseText {
	display: none;
}

.dojoxCollapsibleToolbarExpandText {
	display: none;
}

.dijit_a11y .dojoxCollapsibleToolbarCollapseText {
	display: inline;
	padding: 3px;
	font-weight: bold;
}

.dijit_a11y .dojoxCollapsibleToolbarExpandText {
	display: inline;
	padding: 3px;
	font-weight: bold; 
}
