({"cancel":"Avbryt","toDic":"Legg til i ordliste","skip":"Hopp over","iMsg":"Ingen staveforslag","replaceAll":"Erstatt alle","widgetLabel":"Satsvis stavekontroll","skipAll":"Hopp over alle","suggestions":"Forslag","iSkipAll":"Hopp over alle slike","replaceWith":"Erstatt med","msg":"Ingen stavefeil funnet","replace":"Erstatt","unfound":"Ikke funnet","iSkip":"Hopp over dette"})