.dojoxGrid {
	border:1px solid #DBDBDB;
}
.dojoxGridCellContent {
	padding:3px;
}
.dojoxGridSortNode {
	padding:3px;
}
.dojoxGridInProgress {
	cursor:progress;
}
/* Indirect Selection */
.dojoxGridRowSelectorStatusText{
	visibility:hidden;
}
.dijit_a11y .dojoxGridRowSelected {
	opacity:0.8 !important;
}
.dijit_a11y .dojoxGridBorderDIV {
	border:2px solid #000 !important;
}
.dijit_a11y .dojoxGridRowSelector {
	height:100% !important;
}
.dijit_a11y .dojoxGridRowSelectorStatusText{
	font-size:larger !important;
	visibility:visible;
}
.dijit_a11y .dijitCheckBox .dojoxGridRowSelectorStatusText{
	font-weight:bolder !important;
	font-size:x-large !important;
}
.dijit_a11y .dijitCheckBoxChecked .dojoxGridRowSelectorStatusText{
	font-size:small !important;
}
