({"buttonSet":"Nastaviť","insertTableTitle":"Vložiť tabuľku","insertTableRowAfterLabel":"Pridať riadok za","center":"na stred","deleteTableColumnLabel":"Vymazať stĺpec","right":"vpravo","insertTableColumnBeforeLabel":"Pridať stĺpec pred","tableWidth":"Šírka tabuľky:","buttonInsert":"Vložiť","default":"štandardne","align":"Zarovnať:","insertTableRowBeforeLabel":"Pridať riadok pred","cellSpacing":"Rozstupy buniek:","pixels":"pixely","selectTableLabel":"Vybrať tabuľku","rows":"Riadky:","modifyTableTitle":"Upraviť tabuľku","cellPadding":"Odsadenie obsahu buniek:","deleteTableRowLabel":"Vymazať riadok","backgroundColor":"Farba pozadia:","insertTableColumnAfterLabel":"Pridať stĺpec za","left":"vľavo","borderThickness":"Hrúbka rámčeka","columns":"Stĺpce:","percent":"percentá","borderColor":"Farba rámčeka:","buttonCancel":"Cancel"})