({"saveLabel":"Kaydet","saveSettingdialogTitle":"Otomatik Kaydet","saveSettingdialogButtonOk":"Aralığı Ayarla","saveMessageSuccess":"${0} konumuna kaydedildi","saveSettingdialogButtonCancel":"İptal","saveSettingdialogDescription":"Otomatik Kaydetme Aralığını Belirt","saveSettingLabelOn":"Otomatik Kaydetme Aralığını Ayarla...","saveSettingdialogParamLabel":"dak","saveSettingdialogParamName":"Otomatik Kaydetme Aralığı","saveMessageFail":"${0} konumuna kaydedilemedi","saveSettingLabelOff":"Otomatik Kaydetmeyi Kapat"})