dojo.provide('ChirpTools.TestRunner.Events');

chirp.makeEvents('ChirpTools.TestRunner.Events', [
    'TestCount',
    'TestFinished',
    'Channel.RemoteReady',
    'SubjectFrame.Ready',
    'Display.RunClick',
    'Display.RunOneClick',
    'Running',
    'Navigate'
]);
