({"cancel":"Cancelar","toDic":"Incluir no dicionário","skip":"Ignorar","iMsg":"Nenhuma sugestão de ortografia","replaceAll":"Substituir Todos","widgetLabel":"Verificação Ortográfica em Lote","skipAll":"Ignorar Todos","suggestions":"Sugestões","iSkipAll":"Ignorar todos iguais a este","replaceWith":"Substituir por","msg":"Nenhum erro de ortografia encontrado","replace":"Substituir","unfound":"Não localizado","iSkip":"Ignorar isto"})