({"descending":"יורד","ascending":"עולה","nestedSort":"מיון מקונן","unsorted":"לא למיין עמודה זו","singleSort":"מיון יחיד","indirectSelectionRadio":"Row ${0}, single selection, radio box","indirectSelectionCheckBox":"Row ${0}, multiple selection, check box","selectAll":"Select all","sortingState":"${0} - ${1}"})