({"replaceDialogText":"${0} előfordulás cseréje megtörtént.","eofDialogTextFind":"találat","eofDialogText":"Legutóbbi előfordulás: ${0}","backwards":"Visszafelé","replaceButton":"Csere","replaceLabel":"Csere:","matchCase":"Kis-nagybetűk egyeznek","findTooltip":"Adja meg a keresett szöveget","replaceTooltip":"Adja meg a cseréhez az új szöveget","replaceAllButton":"Mindent lecserél","eofDialogTextReplace":"cserélve","findReplace":"Keresés és csere","backwardsTooltip":"Szöveg keresése visszafelé","replaceAllButtonTooltip":"Minden szöveg cseréje","replaceButtonTooltip":"Szöveg cseréje","findLabel":"Keresés:","findButton":"Keresés","matchCaseTooltip":"Kis-nagybetűk egyeznek","findButtonTooltip":"Szöveg keresése","replaceAll":"Minden előfordulás"})