({"saveLabel":"保存","saveSettingdialogTitle":"自动保存","saveSettingdialogButtonOk":"设置时间间隔","saveMessageSuccess":"已保存到 ${0}","saveSettingdialogButtonCancel":"取消","saveSettingdialogDescription":"指定自动保存时间间隔","saveSettingLabelOn":"设置自动保存时间间隔...","saveSettingdialogParamLabel":"分钟","saveSettingdialogParamName":"自动保存时间间隔","saveMessageFail":"未能保存到 ${0}","saveSettingLabelOff":"关闭自动保存"})