/* filter bar */
.dojoxGridRtl .dojoxGridFBarStatus {
	margin-left: 0;
	margin-right: 9px;
}
.dojoxGridRtl .dojoxGridFBarClearFilterBtn {
	margin-left: 0;
	margin-right: 10px;
}
.dojoxGridRtl .dojoxGridFBarCloseBtn {
	margin: 0 0 0 4px;
	float: left;
}
.dj_ie .dojoxGridRtl .dojoxGridFBarCloseBtn {
	left: 2px;
	right: auto;
}
/* filter def dialog */
.dijitRtl .dojoxGridFDPaneRelation {
	margin: 0 4px 3px 0;
}  
.dijitRtl .dojoxGridFDPane .dijitAccordionTitleFocus {
	margin-right: 0;
	margin-left: 14px;
}
.dijitRtl .dojoxGridFDPane .dijitAccordionTitle .dijitButton {
	left: 3px;
	right: auto;
}
.dj_ie6 .dijitRtl .dojoxGridFDPane .dijitAccordionTitle .dijitButton{
	left: -290px;
}
.dijitRtl .dojoxGridFDPaneBtns {
	left: 0;
	right: auto;
}
.dijitRtl .dojoxGridFDPaneBtns .dijitButton,
.dijitRtl .dojoxGridClearFilterBtns .dijitButton,
.dijitRtl .dojoxGridFCBoxEndValue {
	float: left;
}
.dijitRtl .dojoxGridFCBoxRangeValueTxt {
	margin-left: 0;
	margin-right: 4px;
}

/* status tip */
.dijitRtl .dojoxGridFStatusTipTitle {
	margin-left: 1em;
	margin-right: 0;
}
.dijitRtl .dojoxGridFStatusTipHandle {
	padding-left: 16px;
	padding-right: 0;
}
.dijitRtl .dojoxGridFStatusTipDetail .dijitButton {
	left: 0;
	right: auto;
}

/* bool value box */
.dijitRtl .dojoxGridTrueBox,
.dijitRtl .dojoxGridFalseBox {
	float: right;
}

/* For Claro */
.claro .dojoxGridRtl .dojoxGridFBar {
	border-right: 1px solid white;
}

.claro .dijitRtl .dojoxGridFStatusTip th div {
	border-left: 1px solid #BCBCBC;
	border-right: 1px solid white;
}
.claro .dijitRtl .dojoxGridFStatusTip th.lastColumn div {
	border-left: 1px solid #BCBCBC;
}
