/*
	Copyright (c) 2004-2011, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


/*=====
// Supplemental summaries for those hard-to-doc places your conventional doc parser can't reach.
// Where possible, these summaries should appear inline in the code.
//
// this is "package level documentation"

dojo.cldr = {
	// summary: transformation of relevant pieces of the Unicode.org Common Locale Data Repository
	// (see http://unicode.org/cldr) to JSON from the original XML with associated utility classes
};

dojo.data = {
	// summary: A uniform data access layer
};

dojo.dnd = {
	// summary: Drag and Drop resources
};

dojo.io = {
	// summary: Additional I/O transports (Ajax)
};

dojo.rpc = {
	// summary: Dojo remote-procedure-call resources
};

// "variables"

dojo.baseUrl = {
	// summary: The root relative path to dojo.js (as a string)
	// example:
	//	if(typeof dojo != "undefined"){ console.log(dojo.baseUrl); }
};

=====*/
