<div class="dijit dijitReset dijitInlineTable dijitLeft"
	id="widget_${id}"
	role="combobox" tabIndex="-1"
	><div style="overflow:hidden;"
		><div class='dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton'
			dojoAttachPoint="downArrowNode,_buttonNode,_popupStateNode" role="presentation"
			><div class="dijitArrowButtonInner">&thinsp;</div
			><div class="dijitArrowButtonChar">&#9660;</div
		></div
		><div class="dijitReset dijitValidationIcon"><br></div
		><div class="dijitReset dijitValidationIconText">&Chi;</div
		><div class="dijitReset dijitInputField"
			><input type="text" autocomplete="off" ${!nameAttrSetting} class='dijitReset'
				dojoAttachEvent='onkeypress:_onKey' 
				dojoAttachPoint='textbox,focusNode' role="textbox" aria-haspopup="true" aria-autocomplete="list"
		/></div
	></div
></div>
