({"set":"แทรก","text":"รายละเอียด","insertImageTitle":"แทรกรูปภาพ","invalidMessage":"ชนิดของไฟล์รูปภาพไม่ถูกต้อง","prePopuTextBrowse":" หรือเรียกดูโลคัลไฟล์","browse":"เรียกดู...","prePopuTextUrl":"ป้อน URL ของรูปภาพ","url":"รูปภาพ"})