/* Editor Table Dialog */
.EditorTableDialog{

}
.EditorTableDialog .etdTable{
	border:none;
	table-layout:fixed;
	border-collapse:collapse;
}
.EditorTableDialog .etdTable td{
	border:#CCCCCC 0px solid;
	padding:5px;
}
.EditorTableDialog .etdTable td.inner{
	padding:0px;
}
.EditorTableDialog .dijitTextBox{
	width:50px;
	padding-left:3px;
}
.EditorTableDialog .dijitComboBox{
	width:85px;
	padding-left:5px;
}
.EditorTableDialog label{
	float:right;
	margin:0;
	margin-right:5px;
}
.EditorTableDialog .cellpad{
	background:url(images/cellpad.png) no-repeat 5px;
}
.EditorTableDialog .cellspace{
	background:url(images/cellspace.png) no-repeat 5px;
}
.dialogButtonContainer{
	text-align:right;
	margin-top:10px;
	cursor:pointer;
}

.colorSwatchBtn{
	display:block;
	border:#666666 2px solid;
	background-color:#CCCCCC;
	width:16px;
	height:16px;
	cursor:pointer;
}
.colorSwatchBtn:hover{
	border:#0099FF 2px solid;
}
.colorSwatchBtn div{
	background-color:#CCCCCC;
	width:16px;
	height:16px;
	cursor:pointer;
}

/* Editor Buttons */
.editorIcon{
	background-image:url(images/tableIcons.png);
	background-repeat: no-repeat;
	width: 18px;
	height: 18px;
	text-align: center;
}
.editorIconUploadImage{
	background:url(images/uploadImageIcon.gif) no-repeat 0px 4px;
}

.editorIconInsertTable{				
	background-position:0px 0px;	}
.dijitDisabled .editorIconInsertTable{				
	background-position:-162px 0px;	}
.editorIconInsertTableRowBefore{ 	
	background-position:-36px 0px;	}
.dijitDisabled .editorIconInsertTableRowBefore{ 	
	background-position:-198px 0px;	}
	
.editorIconInsertTableRowAfter{ 	
	background-position:-54px 0px;	}
.dijitDisabled .editorIconInsertTableRowAfter{ 	
	background-position:-216px 0px;	}

.editorIconInsertTableColumnBefore{ 	
	background-position:-72px 0px;	}
.dijitDisabled .editorIconInsertTableColumnBefore{ 	
	background-position:-234px 0px;	}
	
.editorIconInsertTableColumnAfter{ 	
	background-position:-90px 0px;	}
.dijitDisabled .editorIconInsertTableColumnAfter{ 	
	background-position:-252px 0px;	}
	
.editorIconDeleteTableRow{ 	
	background-position:-108px 0px;	}
.dijitDisabled .editorIconDeleteTableRow{ 	
	background-position:-270px 0px;	}
	
.editorIconDeleteTableColumn{ 	
	background-position:-126px 0px;	}
.dijitDisabled .editorIconDeleteTableColumn{ 	
	background-position:-288px 0px;	}
	
.editorIconColorTableCell{ 	
	background-position:-144px 0px;	}
.dijitDisabled .editorIconColorTableCell{ 	
	background-position:-306px 0px;	}

.editorIconModifyTable{ 	
	background-position:-18px 0px;	}
.dijitDisabled .editorIconModifyTable{ 	
	background-position:-180px 0px;	}




.RichTextEditable .dijitTextBox, 
.RichTextEditable .dijitComboBox,
.RichTextEditable  .dijitSpinner {
	width: 5em;
}

.dojoxDropDownSelect {
	background:#FFFFFF url(images/dropBk.png) repeat-x scroll left top;
	border: 1px solid;
	border-top-color:#cccccc;
	border-right-color:#cccccc;
	border-left-color:#999999;
	border-bottom-color:#ffffff;
}
.dojoxDropDownSelect.dojoxDropDownSelectHover{
	background-image:url(images/dropBkOver.png);
}
.dojoxDropDownSelect button{
	line-height:16px;
	height:16px;
	width:100px;
	text-align:left;
	color:#8397b9;
}

.dijitButtonNode .dijitArrowButtonInner{
	/* fixing the dijit drop arrows - can't figure why this broke */
	background:url(../../../../dijit/themes/tundra/images/spriteArrows.png) no-repeat left top;
	width:7px;
}

.dojoxEditorUploadNorm{
	font-family:Arial;
	font-size:12px;
	
	
	vertical-align: middle;
	
	margin:0;
	line-height:normal;
	text-align:center;
	white-space: nowrap;
	
	cursor: pointer;
	background: #ccc url("images/uploadImageIcon.gif") no-repeat 2px 2px;
}

.dojoxEditorUploadHover{
	background: #ccc url("images/uploadImageIcon.gif") no-repeat 2px 0px;
}
.dojoxEditorUploadActive{
	background: #ccc url("images/uploadImageIcon.gif") no-repeat 2px 4px;
}
.dojoxEditorUploadDisabled{
	background: #ccc url("images/uploadImageIcon.gif") no-repeat 2px 2px;
}
