({"buttonSet":"Configurar","insertTableTitle":"Inserir Tabela","insertTableRowAfterLabel":"Incluir Linha Depois","buttonCancel":"Cancelar","center":"centro","deleteTableColumnLabel":"Excluir Coluna","right":"direita","insertTableColumnBeforeLabel":"Incluir Coluna Antes","tableWidth":"Largura da Tabela:","buttonInsert":"Inserir","default":"default","align":"Alinhar:","insertTableRowBeforeLabel":"Incluir Linha Antes","cellSpacing":"Espaçamento de Célula:","pixels":"pixels","selectTableLabel":"Selecionar Tabela","rows":"Linhas:","modifyTableTitle":"Modificar Tabela","cellPadding":"Preenchimento de Célula:","deleteTableRowLabel":"Excluir Linha","backgroundColor":"Cor do Plano de Fundo:","insertTableColumnAfterLabel":"Incluir Coluna Depois","left":"esquerda","borderThickness":"Grossura da Borda","columns":"Colunas:","percent":"percentual","borderColor":"Cor da Borda:"})