({"gotoButtonTitle":"Перейти на указанную страницу","prevTip":"Предыдущая страница","pageCountIndication":" (${0} страниц)","pageStepLabelTemplate":"Страница ${0}","descTemplate":"${2} - ${3} из ${1} ${0}","lastTip":"Последняя страница","nextTip":"Следующая страница","pageSizeLabelTemplate":"${0} элементов на странице","dialogCancel":"Отменить","itemTitle":"элементов","dialogConfirm":"Перейти","firstTip":"Первая страница","allItemsLabelTemplate":"Все элементы","dialogIndication":"Укажите номер страницы","dialogTitle":"Перейти на страницу"})