/* DOM Buttons */
@import url("domButtons/Common-compat.css");
@import url("domButtons/DomButtonPlus-compat.css");
@import url("domButtons/DomButtonUpArrow-compat.css");
@import url("domButtons/DomButtonDownArrow-compat.css");
@import url("domButtons/DomButtonSearch-compat.css");
