({"cancel":"Storno","toDic":"Přidat do slovníku","skip":"Přeskočit","iMsg":"Žádné návrhy pravopisu","replaceAll":"Nahradit vše","widgetLabel":"Dávková kontrola pravopisu","skipAll":"Přeskočit vše","suggestions":"Návrhy","iSkipAll":"Přeskočit všechny podobné výskyty","replaceWith":"Text pro nahrazení","msg":"Nebyly nalezeny žádné chyby pravopisu.","replace":"Nahradit","unfound":"Nenalezeno","iSkip":"Přeskočit tento výskyt"})