({"gotoButtonTitle":"Pojdi na specifično stran","prevTip":"Prejšnja stran","pageCountIndication":" (${0} strani) ","pageStepLabelTemplate":"Stran ${0}","descTemplate":"${2} - ${3} od ${1} ${0}","lastTip":"Zadnja stran","nextTip":"Naslednja stran","pageSizeLabelTemplate":"${0} postavk na stran ","dialogCancel":"Prekliči","itemTitle":"postavk","dialogConfirm":"Pojdi","firstTip":"Prva stran","allItemsLabelTemplate":"Vse postavke ","dialogIndication":"Podajte številko strani ","dialogTitle":"Pojdi na stran "})