({"conditionBefore":"före","filterBarDefButton":"Definiera filter","filterBarMsgNoFilterTemplate":"Inga filter har använts","conditionContains":"innehåller","clearFilterMsg":"Filtret kommer att tas bort och alla tillgängliga poster kommer att visas.","all":"alla","waiValueBoxTemplate":"Ange värdet för regeln ${0}","conditionRange":"intervall","waiCancelButton":"Stäng den här dialogrutan","relationAll":"alla regler","conditionLarger":"är större än","valueBoxLabel":"Värde","statusTipMsg":"Om du vill filtret efter värden i ${0} klickar du på filterfältet.","waiRelAll":"Matcha följande regler:","closeFilterBarBtn":"Stäng filterfältet","a11yFilterBarDefButton":"Filter...","trueLabel":"Sant","conditionIsNot":"är inte","conditionIs":"är","ruleTitleTemplate":"Regeln ${0}","waiRemoveRuleButtonTemplate":"Ta bort regeln ${0}","and":"och","conditionLess":"är mindre än","waiColumnSelectTemplate":"Kolumn för regeln ${0}","anycolumn":"någon kolumn","filterDefDialogTitle":"Filter","waiConditionSelectTemplate":"Villkor för regeln ${0}","waiFilterButton":"Lämna in filtret","statusTipTitle":"Filterfält","statusTipRelPre":"Matchar","anyColumnOption":"Någon kolumn","conditionSelectLabel":"Villkor","cancelButton":"Avbryt","conditionNotStartWith":"börjar inte med","conditionEndWith":"slutar med","statusTipTitleNoFilter":"Filterfält","columnSelectLabel":"Kolumn","conditionNotContain":"innehåller inte","clearButton":"Rensa","conditionLargerEqual":"är större än eller lika med","any":"någon","relationMsgFront":"Matcha","statusTipHeaderColumn":"Kolumn","defaultItemsName":"objekt","conditionEqual":"är lika med","relationAny":"någon regel","trueLabelEditable":"Markerat","or":"eller","filterBarMsgHasFilterTemplate":"${0} av ${1} ${2} visas","rangeTo":"till","conditionStartsWith":"börjar med","waiRelAny":"Matcha någon av följande regler:","statusTipRelPost":"regler.","relationMsgTail":"","filterBarClearButton":"Rensa filtret","incompleteRuleTip":"Den här regeln är inte fullständig","conditionLessEqual":"är mindre än eller lika med","waiFilterBarClearButton":"Rensa filtret","conditionAfter":"efter","statusTipHeaderCondition":"Regler","falseLabel":"Falskt","filterButton":"Filter","clearFilterDialogTitle":"Rensa filtret","statusTipTitleHasFilter":"Filter","addRuleButton":"Lägg till regel","waiAddRuleButton":"Lägg till en ny regel","waiClearButton":"Rensa filtret","conditionNotEqual":"är inte lika med","conditionNotEndWith":"slutar inte med","rangeTemplate":"från ${0} till ${1}","removeRuleButton":"Ta bort regel","waiFilterBarDefButton":"Filtrera tabellen","conditionIsEmpty":"is empty"})