({"buttonSet":"Nastavi","insertTableTitle":"Vstavi tabelo","insertTableRowAfterLabel":"Dodaj vrstico za","buttonCancel":"Prekliči","center":"na sredini","deleteTableColumnLabel":"Izbriši stolpec","right":"desno","insertTableColumnBeforeLabel":"Dodaj stolpec pred","tableWidth":"Širina tabele:","buttonInsert":"Vstavi","default":"privzeto","align":"Poravnaj:","insertTableRowBeforeLabel":"Dodaj vrstico pred","cellSpacing":"Razmik celic:","pixels":"pikslov","selectTableLabel":"Izberi tabelo","rows":"Vrstice:","modifyTableTitle":"Spremeni tabelo","cellPadding":"Polnjenje celic:","deleteTableRowLabel":"Izbriši vrstico","backgroundColor":"Barva ozadja:","insertTableColumnAfterLabel":"Dodaj stolpec za","left":"levo","borderThickness":"Debelina obrobe","columns":"Stolpci:","percent":"odstotkov","borderColor":"Barva obrobe:"})