/* filter bar */
.dojoxGridFBar {
	width: 100%;
	cursor: pointer;
}
.dojoxGridFBar .dojoxGridFBarBtn {
	margin: 0;
}
.dojoxGridFBarBtnTD {
	width: 38px;
}
.dojoxGridFBar .dojoxGridFBarBtn .dijitButtonNode {
	-moz-border-radius: 1px;
	-webkit-border-radius: 1px;
	padding-top: 0;
	padding-bottom: 0;
	padding-right: 2px;
}
.dojoxGridFBarDefFilterBtnIcon {
	background: url("images/sprite_icons.png") no-repeat;
	background-position: -100px -18px;
	width: 14px;
	height: 14px;
}
.dj_ie .dojoxGridFBarInner{
	display: inline-block;
	width: 100%;
}
.dojoxGridFBarStatus {
	margin-left: 9px;
	float: left;
}
.dojoxGridFBarClearFilterBtn {
	margin: 0 0 0 10px;
	vertical-align: top;
	float: left;
}
.dojoxGridFBarInfoTD .dojoxGridFBarClearFilterBtn .dijitButtonNode,
.dojoxGridFBarInfoTD .dojoxGridFBarCloseBtn .dijitButtonNode,
.dojoxGridFDPane .dijitAccordionTitle .dijitButtonNode,
.dojoxGridFStatusTipDetail .dijitButton .dijitButtonNode {
	-moz-border-radius: 0;
	-moz-box-shadow: none;
	-webkit-border-radius: 0;
	-webkit-box-shadow: none;
	background-image: none;
	background-color: transparent;
	margin: 0;
	padding: 0;
	border: none;
}
.dj_ie .dojoxGridFBarInner {
	position: relative;
}
.dojoxGridFBarCloseBtn {
	margin: 0 4px 0 0;
	float: right;
}
.dj_ie .dojoxGridFBarCloseBtn {
	float: none;
	position: absolute;
	right: 0;
}
.dojoxGridFBarCloseBtnIcon {
	background: url("images/sprite_icons.png") no-repeat;
	background-position: -119px -20px;
	width: 14px;
	height: 14px;
}
.dijitButtonHover .dojoxGridFBarCloseBtnIcon {
	background-position: -140px -20px;
}
.dijitButtonActive .dojoxGridFBarCloseBtnIcon {
	background-position: -160px -20px;
}

/* filter def dialog */
.dojoxGridFDTitlePane {
	width: 316px;
	height: 330px;
}
.dijit_a11y .dojoxGridFDTitlePane .dijitArrowButtonInner{
	/* This should be fixed in dijit */
	display:none !important;
}
.dojoxGridFDTitlePane .dijitDialogPaneContent {
	padding: 7px 5px 9px 5px;
	height: 290px !important;
}
.dojoxGridFDTitlePane .dijitTitlePaneTitle {
	cursor: move;
}
.dojoxGridFDPaneRelation {
	margin: 0 0 3px 4px;
}
.dojoxGridFDPane {
	width: 100%;
	height: 100%;
	position: relative;
}
.dojoxGridFDPaneRulePane {
	height: 222px;
	overflow: auto;
	/* for ie6/7 only */
	position: relative;
}
.dj_ie6 .dojoxGridFDPaneRulePane {
	width: 100%;
}
.dojoxGridFDPane .dijitAccordionContainer .dijitContentPane {
	padding: 6px 9px;
}
.dojoxGridFDPane .dijitAccordionTitle {
	position: relative;
	/* for non-claro theme, this can ensure a correct height change when adding/removing child */
	min-height: 17px;
}
.dojoxGridFDPane .dijitAccordionTitle .dijitButton {
	position: absolute;
	margin: 0;
	right: 3px;
	top: 4px;
}
.dj_ie6 .dojoxGridFDPane .dijitAccordionTitle .dijitButton {
	top: 2px;
}
.dojoxGridFDPane .dijitAccordionTitleFocus {
	margin-right: 14px;
}
.dojoxGridFDPane .dijitAccordionText {
	display: inline-block;
	position: relative;
}
.dojoxGridFCBoxRemoveCBoxBtnIcon {
	background-image: url("images/sprite_icons.png");
	background-position: -198px -18px;
	width: 16px;
	height: 16px;
}
.dijitButtonHover .dojoxGridFCBoxRemoveCBoxBtnIcon {
	background-position: -198px 2px;
}
.dojoxGridRuleTitleCondition {
	font-style: italic;
}
.dojoxGridFDPaneModes .dijitSelect {
	position: relative;
}
.dojoxGridFDPaneBtns {
	position: absolute;
	right: 0;
	bottom: 0;
}
.dj_ie7 .dojoxGridFDPaneBtns,
.dj_ie6 .dojoxGridFDPaneBtns {
	z-index: -1;
}
.dojoxGridFDPaneBtns .dijitButton {
	float: right;
}
.dojoxGridFDPaneAddCBoxBtn {
	margin-top: 9px;
}
.dojoxGridFDPaneAddCBoxBtnIcon {
	background-image: url("images/sprite_icons.png");
	background-position: -218px 2px;
	width: 16px;
	height: 16px;
}
.dijitButtonDisabled .dojoxGridFDPaneAddCBoxBtnIcon {
	background-position: -218px -18px;
}

/* criteria box */
.dojoxGridFCBox {
	height: 125px;
}
.dojoxGridFCBoxSelCol, 
.dojoxGridFCBoxCondition {
	height: 40px;
}
.dojoxGridFCBox .dojoxGridFCBoxColSelect, 
.dojoxGridFCBox .dojoxGridFCBoxCondSelect, 
.dojoxGridFCBox .dojoxGridFCBoxValueBox {
	margin: 5px 0 0 0;
	width: 100%;
	/* inherited is inline-block this will cause width "error" in non IE for html tables */
	display: inline-table;
}
.dojoxGridFCBoxCondSelectAlt {
	width: 100%;
	padding: 5px 0 0 0;
	font-weight: bold;
}
.dojoxGridFCBox .dijitSelect .dijitArrowButton {
	width: 18px;
}
.dojoxGridFCBoxStartValue {
	width: 45%;	
}
.dojoxGridFCBoxEndValue {
	float: right;
	width: 45%;
}
.dojoxGridFCBoxRangeValueTxt {
	margin-left: 4px;
}

/* status tip */
.dojoxGridFStatusTipDialog .dijitTooltipContainer {
	padding: 9px 5px;
}
.dojoxGridFStatusTip thead,
.dojoxGridFStatusTip tr {
	height: 19px;
}
.dojoxGridFStatusTip th {
	max-width: 150px;
	width: expression(this.clientWidth > 150 ? "150px" : "auto");
}
.dojoxGridFStatusTip th div {
	padding: 2px 5px;
}
.dojoxGridFStatusTip td {
	padding: 2px 6px;
	max-width: 150px;
	overflow: hidden;
	width: expression(this.clientWidth > 150 ? "150px" : "auto");
}
.dojoxGridFStatusTipHead {
	margin-bottom: 9px;
}
.dojoxGridFStatusTipTitle {
	font-weight: bold;
	margin-right: 1em;
}
.dojoxGridFStatusTipHandle {
	position: relative;
	padding-right: 16px;
}
.dojoxGridFStatusTipDetail .dijitButton {
	position: absolute;
	margin: 0;
	padding: 0;
	top: -1px;
	right:0;
}
.dojoxGridFStatusTipDelRuleBtnIcon {
	background-image: url("images/sprite_icons.png");
	background-position: -198px -18px;
	height: 16px;
	width: 16px;
}
.dijitButtonHover .dojoxGridFStatusTipDelRuleBtnIcon {
	background-position: -198px 2px;
}

/* clear dlg */
.dj_ie7 .dojoxGridClearFilterConfirm,
.dj_ie6 .dojoxGridClearFilterConfirm {
	width: 300px;
}
.dojoxGridClearFilterBtns{
	padding: 10px;
	height: 18px;
}
.dojoxGridClearFilterBtns .dijitButton {
	float: right;
}

/* bool value box */
.dojoxGridTrueBox {
	float: left;
	width: 49%;
}
.dojoxGridFalseBox {
	float: left;
	width: 49%;
}
.dojoxGridBoolValueBox .dijitRadio {
	vertical-align: middle;
} 
.dojoxGridTrueLabel,
.dojoxGridFalseLabel {
	width: 116px;
	display:inline-block;
	vertical-align: middle;
}
.dj_ie7 .dojoxGridTrueLabel,
.dj_ie7 .dojoxGridFalseLabel {
	display: inline;
}
