({"conditionBefore":"önce","filterBarDefButton":"Süzgeç tanımla","filterBarMsgNoFilterTemplate":"Süzgeç uygulanmadı","conditionContains":"içerir","clearFilterMsg":"Bu işlem sonucunda süzgeç kaldırılacak ve kullanılabilir tüm kayıtlar gösterilecek.","all":"tümü","waiValueBoxTemplate":"${0} kuralını süzmek için değer girin","conditionRange":"aralık","waiCancelButton":"Bu iletişim kutusunu iptal et","relationAll":"tüm kurallar","conditionLarger":"büyüktür","valueBoxLabel":"Değer","statusTipMsg":"${0} öğesindeki değerleri süzmek için burada süzgeç çubuğunu tıklatın.","waiRelAll":"Aşağıdaki kuralların tümüyle eşleştir:","closeFilterBarBtn":"Süzgeç çubuğunu kapat","a11yFilterBarDefButton":"Süz...","trueLabel":"Doğru","conditionIsNot":"değildir","conditionIs":"eşittir","ruleTitleTemplate":"Kural ${0}","waiRemoveRuleButtonTemplate":"${0} kuralını kaldır","and":"ve","conditionLess":"küçüktür","waiColumnSelectTemplate":"${0} kuralı için sütun","anycolumn":"herhangi bir sütun","filterDefDialogTitle":"Süzgeç","waiConditionSelectTemplate":"${0} kuralı için koşul","waiFilterButton":"Süzgeci gönder","statusTipTitle":"Süzgeç Çubuğu","statusTipRelPre":"Eşleştir","anyColumnOption":"Herhangi Bir Sütun","conditionSelectLabel":"Koşul","cancelButton":"İptal","conditionNotStartWith":"ile başlamaz","conditionEndWith":"ile biter","statusTipTitleNoFilter":"Süzgeç Çubuğu","columnSelectLabel":"Sütun","conditionNotContain":"içermez","clearButton":"Temizle","conditionLargerEqual":"büyüktür ya da eşittir","any":"herhangi biri","relationMsgFront":"Eşleştir","statusTipHeaderColumn":"Sütun","defaultItemsName":"öğeler","conditionEqual":"eşittir","relationAny":"kurallardan herhangi biri","trueLabelEditable":"Denetlendi","or":"veya","filterBarMsgHasFilterTemplate":"${0} /f ${1} ${2} gösteriliyor.","rangeTo":"-","conditionStartsWith":"ile başlar","waiRelAny":"Aşağıdaki kurallardan herhangi biriyle eşleştir:","statusTipRelPost":"kurallar.","relationMsgTail":"","filterBarClearButton":"Süzgeci kaldır","incompleteRuleTip":"Bu kural tamamlanmamış.","conditionLessEqual":"küçüktür ya da eşittir","waiFilterBarClearButton":"Süzgeci kaldır","conditionAfter":"sonra","statusTipHeaderCondition":"Kurallar","falseLabel":"Yanlış","filterButton":"Süzgeç","clearFilterDialogTitle":"Süzgeci Kaldır","statusTipTitleHasFilter":"Süzgeç","addRuleButton":"Kural Ekle","waiAddRuleButton":"Yeni kural ekle","waiClearButton":"Süzgeci kaldır","conditionNotEqual":"eşit değildir","conditionNotEndWith":"ile bitmez","rangeTemplate":"${0} - ${1}","removeRuleButton":"Kural Kaldır","waiFilterBarDefButton":"Tabloyu süz","conditionIsEmpty":"is empty"})