({"gotoButtonTitle":"Gå til en bestemt side","prevTip":"Forrige side","pageCountIndication":" (${0} sider)","pageStepLabelTemplate":"Side ${0}","descTemplate":"${2} - ${3} av ${1} ${0}","lastTip":"Siste side","nextTip":"Neste side","pageSizeLabelTemplate":"${0} elementer per side","dialogCancel":"Avbryt","itemTitle":"elementer","dialogConfirm":"OK","firstTip":"Første side","allItemsLabelTemplate":"Alle elementer","dialogIndication":"Angi sidetall","dialogTitle":"Gå til side"})