({"gotoButtonTitle":"Przejdź do konkretnej strony","prevTip":"Poprzednia strona","pageCountIndication":" (liczba stron: ${0})","pageStepLabelTemplate":"Strona ${0}","descTemplate":"Od ${2} do ${3} z ${1} ${0}","lastTip":"Ostatnia strona","nextTip":"Następna strona","pageSizeLabelTemplate":"Liczba elementów na stronę: ${0}","dialogCancel":"Anuluj","itemTitle":"elementy","dialogConfirm":"Przejdź","firstTip":"Pierwsza strona","allItemsLabelTemplate":"Wszystkie elementy","dialogIndication":"Określ numer strony","dialogTitle":"Przechodzenie do strony"})