({"descending":"Laskeva","ascending":"Nouseva","nestedSort":"Sisäkkäinen lajittelu","unsorted":"Älä lajittele tätä saraketta","singleSort":"Yksinkertainen lajittelu","indirectSelectionRadio":"Row ${0}, single selection, radio box","indirectSelectionCheckBox":"Row ${0}, multiple selection, check box","selectAll":"Select all","sortingState":"${0} - ${1}"})