({"replaceDialogText":"Replaced ${0} occurrences.","eofDialogTextFind":"found","eofDialogText":"Last occurrence ${0}","backwards":"Backwards","replaceButton":"Replace","replaceLabel":"Replace with:","matchCase":"Match case","findTooltip":"Enter text to find","replaceTooltip":"Enter text to replace with","replaceAllButton":"Replace All","eofDialogTextReplace":"replaced","findReplace":"Find and Replace","backwardsTooltip":"Search backwards for text","replaceAllButtonTooltip":"Replace all the text","replaceButtonTooltip":"Replace the text","findLabel":"Find:","findButton":"Find","matchCaseTooltip":"Match case","findButtonTooltip":"Find the text"})