({"cancel":"Отменить","toDic":"Добавить в словарь","skip":"Пропустить","iMsg":"Нет вариантов написания","replaceAll":"Заменить все","widgetLabel":"Проверка орфографии","skipAll":"Пропустить все","suggestions":"Варианты","iSkipAll":"Пропусть все схожие","replaceWith":"Заменить на","msg":"Ошибок не найдено","replace":"Заменить","unfound":"Не найдено","iSkip":"Пропустить это"})