({"gotoButtonTitle":"Gå til en bestemt side","prevTip":"Forrige side","pageCountIndication":" (${0} sider)","pageStepLabelTemplate":"Side ${0}","descTemplate":"${2}-${3} af ${1} ${0}","lastTip":"Sidste side","nextTip":"Næste side","pageSizeLabelTemplate":"${0} elementer pr. side","dialogCancel":"Annullér","itemTitle":"elementer","dialogConfirm":"Udfør","firstTip":"Første side","allItemsLabelTemplate":"Alle elementer","dialogIndication":"Angiv sidenummeret","dialogTitle":"Gå til side"})