.tundra .dijitTooltipAbove .dijitTooltipConnector, .dj_ie .tundra .dijitTooltipAbove .dijitTooltipConnector {
	display: none;
}

.dojoxLegendNode {
	border: 1px solid #ccc;
	margin: 5px 10px 5px 10px;
	padding: 3px;
	float:right;
}

.dojoxLegendText {
	vertical-align: text-top;
	padding-right: 10px
}

.mapZoomIn {
	width: 21px;
	height: 21px;
	background: url(img/zoomin.png) no-repeat;
	position: absolute;
}

.mapZoomOut {
	width: 21px;
	height: 21px;
	background: url(img/zoomout.png) no-repeat;
	position: absolute;
}

