({"gotoButtonTitle":"Go to a specific page","prevTip":"Previous Page","pageCountIndication":" (${0} pages)","pageStepLabelTemplate":"Page ${0}","descTemplate":"${2} - ${3} of ${1} ${0}","lastTip":"Last Page","nextTip":"Next Page","pageSizeLabelTemplate":"${0} items per page","dialogCancel":"Cancel","itemTitle":"items","dialogConfirm":"Go","firstTip":"First Page","allItemsLabelTemplate":"All items","dialogIndication":"Specify the page number","dialogTitle":"Go to Page"})