({"buttonSet":"Definér","insertTableTitle":"Indsæt tabel","insertTableRowAfterLabel":"Tilføj række efter","buttonCancel":"Annullér","center":"centreret","deleteTableColumnLabel":"Slet kolonne","right":"højre","insertTableColumnBeforeLabel":"Tilføj kolonne før","tableWidth":"Tabelbredde:","buttonInsert":"Indsæt","default":"standard","align":"Justér:","insertTableRowBeforeLabel":"Tilføj række før","cellSpacing":"Celleafstand:","pixels":"pixel","selectTableLabel":"Markér tabel","rows":"Rækker:","modifyTableTitle":"Revidér tabel","cellPadding":"Cellemargen:","deleteTableRowLabel":"Slet række","backgroundColor":"Baggrundsfarve:","insertTableColumnAfterLabel":"Tilføj kolonne efter","left":"venstre","borderThickness":"Kanttykkelse","columns":"Kolonner:","percent":"procent","borderColor":"Kantfarve:"})