({"replaceDialogText":"Au fost înlocuite ${0} apariţii.","eofDialogTextFind":"găsită","eofDialogText":"Ultima apariţie ${0}","backwards":"Înapoi","replaceButton":"Înlocuire","replaceLabel":"Înlocuire cu:","matchCase":"Potrivire majuscule","findTooltip":"Introduceţi textul de găsit","replaceTooltip":"Introduceţi textul cu care se înlocuieşte","replaceAllButton":"Înlocuire toate","eofDialogTextReplace":"înlocuită","findReplace":"Găsire şi înlocuire","backwardsTooltip":"Căutaţi înapoi pentru text","replaceAllButtonTooltip":"Înlocuiţi tot textul","replaceButtonTooltip":"Înlocuiţi textul","findLabel":"Găsire:","findButton":"Găsire","matchCaseTooltip":"Potrivire majuscule","findButtonTooltip":"Găsiţi textul"})