.dojoxScrollWindow {
	position:relative;
	overflow:hidden;
}

.dojoxScrollHelper .helperInner {
	visibility: hidden;
}

.dojoxScrollHelper {
	border:1px solid #b7b7b7;
	width:4px;
	background:#ededed;
	height:4px;
	position:absolute;
	top:4px;
	left:4px;
	border-radius: 3px;
	-moz-border-radius: 3px;
	-webkit-border-radius: 3px;
}

.dojoxScrollWrapper {
}
