<div class="dojoxGridClearFilterConfirm">
	<div class="dojoxGridClearFilterMsg">
		${_clearFilterMsg}
	</div>
	<div class="dojoxGridClearFilterBtns" dojoAttachPoint="btnsNode">
		<span dojoType="dijit.form.Button" label="${_cancelBtnLabel}" dojoAttachPoint="cancelBtn" dojoAttachEvent="onClick:_onCancel"></span>
		<span dojoType="dijit.form.Button" label="${_clearBtnLabel}" dojoAttachPoint="clearBtn" dojoAttachEvent="onClick:_onClear"></span>
	</div>
</div>
