({"saveLabel":"Shrani","saveSettingdialogTitle":"Samodejno shranjevanje ","saveSettingdialogButtonOk":"Nastavi interval","saveMessageSuccess":"Shranjeno ob ${0}","saveSettingdialogButtonCancel":"Prekliči","saveSettingdialogDescription":"Podaj interval za samodejno shranjevanje ","saveSettingLabelOn":"Nastavi interval za samodejno shranjevanje ... ","saveSettingdialogParamLabel":"min","saveSettingdialogParamName":"Interval za samodejno shranjevanje ","saveMessageFail":"Shranjevanje ob ${0} ni uspelo ","saveSettingLabelOff":"Izključi samodejno shranjevanje "})