({"emoticonLaughing":"大笑","emoticonCool":"酷","emoticonTongue":"吐舌","emoticonCrying":"哭泣","emoticonOops":"oops","emoticonFrown":"皱眉","emoticonAngry":"愤怒","emoticonShy":"害羞","emoticonNo":"摇头","emoticonAngel":"天使","smiley":"插入表情图标","emoticonIdea":"思考","emoticonEyebrow":"挑眉","emoticonSmile":"微笑","emoticonWink":"眨眼","emoticonYes":"点头","emoticonGrin":"咧着嘴笑","emoticonGoofy":"傻了","emoticonHalf":"左右为难"})