<div id="${id}" class="gridContainer" dojoAttachPoint="containerNode" tabIndex="0" dojoAttachEvent="onkeypress:_selectFocus">
	<div dojoAttachPoint="gridContainerDiv">
		<table class="gridContainerTable" dojoAttachPoint="gridContainerTable" cellspacing="0" cellpadding="0">
			<tbody>
				<tr dojoAttachPoint="gridNode" >
					
				</tr>
			</tbody>
		</table>
	</div>
</div>