.dj_ie .dojoxGridRtl .dojoxGridHeader table {
	float:none;
}

.dojoxGridRtl .dojoxGridCell {
	text-align:right;
}

.dj_ie8 .dojoxGridRtl .dojoxGridCell {
	border-left: none;
}

.dojoxGridRtl .dojoxGridArrowButtonNode {
	float:left;
}