.dojoxGridRtl .dojoxGridCellBorderLeftTopDIV{
	left: auto;
	right: 0;
	border-width: 2px 2px 0 0;
	border-color: gray gray transparent transparent;
}
.dojoxGridRtl .dojoxGridCellBorderRightBottomDIV{
	right: auto;
	left: 0;
	border-width: 0 0 2px 2px;
	border-color: transparent transparent gray gray;
}
