({"gotoButtonTitle":"Naar specifieke pagina","prevTip":"Vorige pagina","pageCountIndication":" (${0} pagina's)","pageStepLabelTemplate":"Pagina ${0}","descTemplate":"${2} - ${3} van ${1} ${0}","lastTip":"Laatste pagina","nextTip":"Volgende pagina","pageSizeLabelTemplate":"${0} items per pagina","dialogCancel":"Annuleren","itemTitle":"items","dialogConfirm":"Go","firstTip":"Eerste pagina","allItemsLabelTemplate":"Alle items","dialogIndication":"Geef het paginanummer op","dialogTitle":"Naar pagina"})