({"pasteFromWord":"Επικόλληση από το Word","paste":"Επικόλληση","cancel":"Ακύρωση","instructions":"Επικολλήστε το περιεχόμενο από το Word στο παρακάτω πλαίσιο. Όταν το κείμενο για εισαγωγή είναι σωστό, πατήστε το κουμπί Επικόλληση. Για να ακυρώσετε την εισαγωγή κειμένου, πατήστε το κουμπί Ακύρωση."})