({"saveLabel":"Desa","saveSettingdialogTitle":"Desat automàtic","saveSettingdialogButtonOk":"Estableix l'interval","saveMessageSuccess":"Desat a les ${0}","saveSettingdialogButtonCancel":"Cancel·la","saveSettingdialogDescription":"Especifiqueu l'interval de desat automàtic","saveSettingLabelOn":"Estableix l'interval per desar automàticament...","saveSettingdialogParamLabel":"min","saveSettingdialogParamName":"Interval de desat automàtic","saveMessageFail":"No s'ha pogut desar a les ${0}","saveSettingLabelOff":"Deshabilita el desat automàtic"})