({"gotoButtonTitle":"Zu einer bestimmten Seite wechseln","prevTip":"Vorherige Seite","pageCountIndication":" (${0} Seiten)","pageStepLabelTemplate":"Seite ${0}","descTemplate":"${2} - ${3} von ${1} ${0}","lastTip":"Letzte Seite","nextTip":"Nächste Seite","pageSizeLabelTemplate":"${0} Elemente pro Seite","dialogCancel":"Abbrechen","itemTitle":"Elemente","dialogConfirm":"Start","firstTip":"Erste Seite","allItemsLabelTemplate":"Alle Elemente","dialogIndication":"Seitenzahl angeben","dialogTitle":"Zur Seite wechseln"})