({"replaceDialogText":"取代了 ${0} 項。","eofDialogTextFind":"找到","eofDialogText":"前次出現 ${0}","backwards":"向後","replaceButton":"取代","replaceLabel":"取代為：","matchCase":"大小寫相符","findTooltip":"輸入要尋找的文字","replaceTooltip":"輸入要取代的文字","replaceAllButton":"全部取代","eofDialogTextReplace":"已取代","findReplace":"尋找/取代","backwardsTooltip":"往回搜尋文字","replaceAllButtonTooltip":"取代所有文字","replaceButtonTooltip":"取代文字","findLabel":"尋找：","findButton":"尋找","matchCaseTooltip":"大小寫相符","findButtonTooltip":"尋找文字","replaceAll":"所有出現項目"})