@import url("../../widget/FilePicker/FilePicker.css");

.dojoxFilePickerTextBox{
	width: 30em;
	vertical-align: middle;
}
.dojoxFilePickerTextBox input:focus{
	outline: none;
}
.dojoxFilePickerTextBoxFocused{
	outline: auto 5px -webkit-focus-ring-color;
}
.dojoxFilePickerTextBox INPUT{
	border-left: solid black 1px;
	display:inline;
	position:static !important;
	border:0 !important;	
	margin:0 !important;
	vertical-align:top !important;
	visibility:visible !important;
	background-color:transparent !important;
	background-image:none !important;
	width:100% !important;
}
.dijitRtl .dojoxFilePickerTextBox .dijitInputField {
	border-right-width:1px !important;
	border-left-width:0 !important;
}

/* Tundra stylings */
.tundra .dojoxFilePickerTextBoxDisabled *
{
	cursor: not-allowed !important;
}
.tundra .dojoxFilePickerTextBox {
	font-family: sans-serif;
	font-size: 100%;
}
.tundra .dojoxFilePickerTextBox {
	background:#fff url("../../../dijit/themes/tundra/images/validationInputBg.png") repeat-x top left;
	#background:#fff url('../../../dijit/themes/tundra/images/validationInputBg.gif') repeat-x top left;
	border:1px solid #b3b3b3;
	line-height: normal;
}
.tundra .dojoxFilePickerTextBoxDisabled {
	color: gray;
}
.dj_webkit .tundra .dojoxFilePickerTextBoxDisabled {
	color: #eee;
}
.tundra .dojoxFilePickerTextBox .dijitInputField {
	padding: 1px 0;
}
.tundra .dojoxFilePickerTextBox .dijitButtonNode {
	padding: 1px 0.2em;
}
.tundra .dojoxFilePickerTextBox .dijitButtonNode{
	border-color: #9b9b9b;
	border-width: 0px 0px 0px 1px;
	border-style: solid;
}
.tundra .dojoxFilePickerTextBoxFocused {
	border-color:#406b9b;
}
.tundra .dojoxFilePickerTextBoxFocused .dijitButtonNode {
	border-left-color:#366dba;
}
.dijitRtl .tundra .dojoxFilePickerTextBox .dijitButtonNode{
	border-color: #9b9b9b;
	border-width: 0px 1px 0px 0px;
}
.tundra .dojoxFilePickerTextBoxDisabled {
	border-color: #d5d5d5 #d5d5d5 #bdbdbd #d5d5d5; 
	background:#e4e4e4 url("../../../dijit/themes/tundra/images/buttonDisabled.png") top repeat-x;
}
.tundra .dojoxFilePickerTextBoxHover .dijitDownArrowButton {
	border-color:		#a5beda; 
	border-bottom-color:#5c7590; 
	color:#000;
	background:#fcfdff url("../../../dijit/themes/tundra/images/buttonHover.png") repeat-x bottom;
}
.tundra .dojoxFilePickerTextBoxActive .dijitDownArrowButton {
	border-color:#366dba;
	background: #ededed url("../../../dijit/themes/tundra/images/buttonActive.png") bottom repeat-x;
}
.tundra .dojoxFilePickerTextBox .dijitArrowButton .dijitArrowButtonInner {
	background-image: url("images/tundraFolderSprite.gif");
	background-repeat: no-repeat;
	background-attachment: scroll;
	background-position: left center;
	height: auto;
	width: 16px;
	font-size: 100%;
	font-size: inherit;
}
.tundra .dojoxFilePickerTextBox .dojoxHasDropDownOpen .dijitArrowButtonInner {
	background-position: -16px;
}

.tundra .dojoxFilePickerTextBoxError {
	background-color:#f9f7ba;
	background-image:none;
}
.dj_ie6 .tundra .dojoxFilePickerTextBoxError INPUT {
	background-color:#f9f7ba !important;
}
.tundra .dojoxFilePickerTextBoxErrorFocused {
	background-color:#f9f999;
	background-image:none;
}
.dj_ie6 .tundra .dojoxFilePickerTextBoxErrorFocused INPUT {
	background-color:#f9f999 !important;
}

/* nihilo theme */
.nihilo .dojoxFilePickerTextBoxDisabled *
{
	cursor: not-allowed !important;
}
.nihilo .dojoxFilePickerTextBox{
	margin: 0em 0.1em;
}
.nihilo .dojoxFilePickerTextBox {
	background:#fff url("../../../dijit/themes/nihilo/images/validationInputBg.png") repeat-x top left;
	#background:#fff url('../../../dijit/themes/nihilo/images/validationInputBg.gif') repeat-x top left;
	border:1px solid #d3d3d3;
	line-height: normal;
}
.nihilo .dojoxFilePickerTextBox .dijitButtonNode {
	padding: 0 0.2em;
}
.nihilo .dojoxFilePickerTextBox .dijitButtonNode{
	border-color: #d3d3d3;
	border-left: 1px solid #d3d3d3;
}
.nihilo .dojoxFilePickerTextBoxDisabled {
	color: gray;
}
.dj_safari .nihilo .dojoxFilePickerTextBoxDisabled {
	color: #eee;
}
.nihilo .dojoxFilePickerTextBoxFocused {
	border-color:#b3b3b3;
}
.nihilo .dojoxFilePickerTextBoxFocused .dijitButtonNode, {
	border-left-color:#d3d3d3;
}
.dijitRtl .nihilo .dojoxFilePickerTextBox .dijitButtonNode {
	border-color: #8ba0bd;
	border-left: 0px solid #8ba0bd;
	border-right: 1px solid #8ba0bd;
}
.nihilo .dojoxFilePickerTextBoxDisabled {
	border-color: #dedede;
	background:#fafafa url("../../../dijit/themes/nihilo/images/buttonDisabled.png") top repeat-x;
	opacity: 0.60;
}
.nihilo .dojoxFilePickerTextBoxHover .dijitDownArrowButton {
	color:#000;
	background:#fcfcfc url("../../../dijit/themes/nihilo/images/buttonHover.png") repeat-x top left;
}
.nihilo .dojoxFilePickerTextBoxActive .dijitDownArrowButton {
	border-color:#dedede;
	background: #f5f5f5 url("../../../dijit/themes/nihilo/images/buttonActive.png") top left repeat-x;
}
.dijitRtl .nihilo .dojoxFilePickerTextBox .dijitButtonNode {
	border-width: 0px 0px 0px 1px;
}
.nihilo .dojoxFilePickerTextBox .dijitArrowButton .dijitArrowButtonInner {
	background:url("images/nihiloFolderSprite.gif") no-repeat left center;
	width: 16px;
}
.nihilo .dojoxFilePickerTextBox .dojoxHasDropDownOpen .dijitArrowButtonInner {
	background-position: -16px;
}
.nihilo .dojoxFilePickerTextBoxError {
	border-color:#b3b3b3;
	background-color:#f9f7ba;
	background-image:none;
}
.dj_ie6 .nihilo .dojoxFilePickerTextBoxError INPUT {
	background-color:#f9f7ba !important;
}

.nihilo .dojoxFilePickerTextBoxErrorFocused {
	background-color:#ff6;
	background-image:none;
}
.dj_ie6 .nihilo .dojoxFilePickerTextBoxErrorFocused INPUT {
	background-color:#ff6 !important;
}

/* soria theme */
.soria .dojoxFilePickerTextBoxDisabled *
{
	cursor: not-allowed !important;
}
.soria .dojoxFilePickerTextBox{
	margin: 0em 0.1em;
}
.soria .dojoxFilePickerTextBox {
	background:#fff url("../images/validationInputBg.png") repeat-x top left;
	#background:#fff url('../images/validationInputBg.gif') repeat-x top left;
	border:1px solid #8ba0bd;
	line-height: normal;
}
.soria .dojoxFilePickerTextBoxDisabled{
	color: gray;
}
.dj_safari .soria .dojoxFilePickerTextBoxDisabled{
	color: #eee;
}
.soria .dojoxFilePickerTextBox .dijitButtonNode {
	padding: 0 0.2em;
}
.soria .dojoxFilePickerTextBox .dijitButtonNode{
	border-color: #8ba0bd;
	border-left: 1px solid #8ba0bd;
}
.soria .dojoxFilePickerTextBoxFocused {
	border-color:#406b9b;
}
.soria .dojoxFilePickerTextBoxFocused .dijitButtonNode {
	border-left-color:#8ba0bd;
}
.dijitRtl .soria .dojoxFilePickerTextBox .dijitButtonNode{
	border-color: #8ba0bd;
	border-left: 0px solid #8ba0bd;
	border-right: 1px solid #8ba0bd;
}
.soria .dojoxFilePickerTextBox .dijitButtonNode {
	border-width: 0px 0px 0px 1px;
}
.soria .dojoxFilePickerTextBoxDisabled{
	border-color: #b9bbdd #b9bbdd #b9bbdd #b9bbdd;
	background:#c3d3e5 url("../../../dijit/themes/soria/buttonDisabled.png") top repeat-x;
	opacity: 0.60; /* Safari, Opera and Mozilla */
}
.soria .dojoxFilePickerTextBoxHover .dijitDownArrowButton{
	color:#000;
	background:#acc5e2 url("../../../dijit/themes/soria/buttonHover.png") repeat-x top left;
}
.soria .dojoxFilePickerTextBoxActive .dijitDownArrowButton {
	border-color:#657c9c;
	background: #91b4e5 url("../../../dijit/themes/soria/buttonActive.png") top left repeat-x;
}
.dijitRtl .soria .dojoxFilePickerTextBox .dijitButtonNode {
	border-width: 0px 0px 0px 1px;
}
.soria .dojoxFilePickerTextBox .dijitArrowButton .dijitArrowButtonInner {
	background:url("images/soriaFolderSprite.gif") no-repeat left center;
	width: 16px;
}
.soria .dojoxFilePickerTextBox .dojoxHasDropDownOpen .dijitArrowButtonInner {
	background-position: -16px;
}
.soria .dojoxFilePickerTextBoxError {
	border-color:#f3d118;
	background-color:#f9f7ba;
	background-image:none;
}
.dj_ie6 .soria .dojoxFilePickerTextBoxError INPUT {
	background-color:#f9f7ba !important;
}

.soria .dojoxFilePickerTextBoxErrorFocused {
	background-color:#ff6;
	background-image:none;
}
.dj_ie6 .soria .dojoxFilePickerTextBoxErrorFocused INPUT {
	background-color:#ff6 !important;
}
