({"selectContents":"تحديد المحتويات","deleteElement":"حذف عنصر","moveEnd":"نقل المؤشر للنهاية","deleteContents":"حذف محتويات","nodeActions":"${nodeName} تصرفات","selectElement":"تحديد عنصر","moveStart":"نقل المؤشر للبداية"})