<div dojoAttachPoint="outerNode" class="thumbOuter">
	<div dojoAttachPoint="navPrev" class="thumbNav thumbClickable">
	  <img src="" dojoAttachPoint="navPrevImg"/>    
	</div>
	<div dojoAttachPoint="thumbScroller" class="thumbScroller">
	  <div dojoAttachPoint="thumbsNode" class="thumbWrapper"></div>
	</div>
	<div dojoAttachPoint="navNext" class="thumbNav thumbClickable">
	  <img src="" dojoAttachPoint="navNextImg"/>  
	</div>
</div>