({"replaceDialogText":"${0} tekrar değiştirildi.","eofDialogTextFind":"bulundu","eofDialogText":"Son tekrar ${0}","backwards":"Geri","replaceButton":"Değiştir","replaceLabel":"Değiştir:","matchCase":"Büyük/küçük harf eşleştir","findTooltip":"Bulunacak metni girin","replaceTooltip":"Değiştirilecek metni girin","replaceAllButton":"Tümünü Değiştir","eofDialogTextReplace":"değiştirildi","findReplace":"Bul ve Değiştir","backwardsTooltip":"Metni geriye doğru ara","replaceAllButtonTooltip":"Tüm metni değiştir","replaceButtonTooltip":"Metni değiştir","findLabel":"Bul:","findButton":"Bul","matchCaseTooltip":"Büyük/küçük harf eşleştir","findButtonTooltip":"Metni bul","replaceAll":"Tüm Tekrarlar"})