.dijitEditorEilDialogTitle {
	font-weight: bold;
	margin-bottom: 0.6em;
}

.dijitEditorEilDialogDescription {
	white-space: normal;
	margin: 0em 0em 0.3em 0em;
}

.dijitEditorEilDialogField{
	width: 20em;
}
