({"set":"Insérer","text":"Description","insertImageTitle":"Insérer une image","invalidMessage":"Type de fichier image non valide","prePopuTextBrowse":" ou sélectionnez un fichier local.","browse":"Parcourir...","prePopuTextUrl":"Entrez une URL d'image","url":"Image"})