({"conditionBefore":"이전","filterBarDefButton":"필터 정의","filterBarMsgNoFilterTemplate":"적용된 필터 없음","conditionContains":"포함","clearFilterMsg":"이렇게 하면 필터가 제거되며 사용 가능한 모든 레코드가 표시됩니다.","all":"모두","waiValueBoxTemplate":"규칙 ${0}에 대해 필터링할 값 입력","conditionRange":"범위","waiCancelButton":"이 대화 상자 취소","relationAll":"모든 규칙","conditionLarger":"보다 큼","valueBoxLabel":"값","statusTipMsg":"${0}에서 값에 대해 필터링하려면 여기서 필터 막대를 클릭하십시오.","waiRelAll":"다음 모든 규칙과 일치:","closeFilterBarBtn":"필터 막대 닫기","a11yFilterBarDefButton":"필터링...","trueLabel":"True","conditionIsNot":"동일하지 않음","conditionIs":"동일함","ruleTitleTemplate":"규칙 ${0}","waiRemoveRuleButtonTemplate":"규칙 ${0} 제거","and":"및","conditionLess":"보다 작음","waiColumnSelectTemplate":"규칙 ${0}에 대한 열","anycolumn":"임의의 열","filterDefDialogTitle":"필터","waiConditionSelectTemplate":"규칙 ${0}의 조건","waiFilterButton":"필터 제출","statusTipTitle":"필터 막대","statusTipRelPre":"일치","anyColumnOption":"임의의 열","conditionSelectLabel":"조건","cancelButton":"취소","conditionNotStartWith":"다음으로 시작하지 않음","conditionEndWith":"다음으로 종료","statusTipTitleNoFilter":"필터 막대","columnSelectLabel":"열","conditionNotContain":"포함하지 않음","clearButton":"지우기","conditionLargerEqual":"크거나 같음","any":"임의","relationMsgFront":"일치","statusTipHeaderColumn":"열","defaultItemsName":"항목","conditionEqual":"같음","relationAny":"임의의 규칙","trueLabelEditable":"확인됨","or":"또는","filterBarMsgHasFilterTemplate":"표시된 ${1} ${2} 중 ${0}입니다.","rangeTo":"까지","conditionStartsWith":"다음으로 시작","waiRelAny":"다음 중 임의의 규칙과 일치:","statusTipRelPost":"규칙.","relationMsgTail":"","filterBarClearButton":"필터 지우기","incompleteRuleTip":"이 규칙은 완전하지 않습니다.","conditionLessEqual":"작거나 같음","waiFilterBarClearButton":"필터 지우기","conditionAfter":"이후","statusTipHeaderCondition":"규칙","falseLabel":"False","filterButton":"필터","clearFilterDialogTitle":"필터 지우기","statusTipTitleHasFilter":"필터","addRuleButton":"규칙 추가","waiAddRuleButton":"새 규칙 추가","waiClearButton":"필터 지우기","conditionNotEqual":"같지 않음","conditionNotEndWith":"다음으로 종료되지 않음","rangeTemplate":"${0}부터 ${1}까지","removeRuleButton":"규칙 제거","waiFilterBarDefButton":"테이블 필터링","conditionIsEmpty":"is empty"})