.dojoxGridPaginator {
	border-top:1px #DBDBDB solid;
	text-align:center;
	width: 100%;
	height: 24px;
	table-layout : fixed;
	background-color: #EAEAEA;
}
.dojoxGridDescriptionTd {
	text-align: left;
	width: 35%;
}
.dojoxGridDescription {
	text-align: left;
	margin-left: 9px;
	overflow: hidden;
}
.dojoxGridPaginatorFastStep {
	text-align: right;
	width: 35%;
	overflow: hidden;
}
.dojoxGridPaginatorStep {
	float: right;
}
.dojoxGridPaginatorGotoTd {
	width: 20px!important;
}
.dojoxGridPaginatorGotoDiv {
	cursor: pointer;
	width: 12px!important;
	height: 16px!important;
	margin: 0 6px 0 2px;
	background: url("images/sprite_icons.png") no-repeat -77px 4px;
}
.dojoxGridPaginatorGotoDivDisabled {
	background: url("images/sprite_icons.png") no-repeat -77px -16px;
}
.dojoxGridWardButton {
	margin-top: 2px;
	width: 12px!important;
	height: 12px!important;
	float: left;
	background: url("images/sprite_icons.png") no-repeat;
}
.dojoxGridWardButtonInner {
	visibility: hidden;
}
.dijit_a11y .dojoxGridWardButtonInner {
	visibility: visible;
	margin-bottom: 8px;
}
.dojoxGridfirstPageBtn {
	cursor: pointer;
	margin-left: 1px;
	background-position: -57px 3px;
}
.dojoxGridfirstPageBtnDisable {
	margin-left: 1px;
	background-position: -57px -17px;
}
.dojoxGridprevPageBtn {
	cursor: pointer;
	margin: 2px 2px 0 9px;
	background-position: 3px 3px;
}
.dojoxGridprevPageBtnDisable {
	margin: 2px 2px 0 9px;
	background-position: 3px -17px;
}
.dojoxGridlastPageBtn {
	cursor: pointer;
	margin: 2px 9px 0 9px;
	background-position: -37px 3px;
}
.dojoxGridlastPageBtnDisable {
	margin: 2px 9px 0 9px;
	background-position: -37px -17px;
}

.dojoxGridnextPageBtn {
	cursor: pointer;
	margin-left: 3px;
	background-position: -17px 3px;
}
.dojoxGridnextPageBtnDisable {
	margin-left: 3px;
	background-position: -17px -17px;
}
.dojoxGridInactived {
	font-weight: normal;
	color: #5D88AF;
	cursor: pointer;
	margin: 1px 6px 0 5px;
	float: left;
	zoom: 1; /* for IE */
}
.dojoxGridActived {
	font-weight: bold;
	color: black;
	margin: 1px 6px 0 5px;
	float: left;
	text-decoration: none!important; 
	zoom: 1; /* for IE */
}
.dojoxGridInactiveSwitch {
	font-weight: normal;
	color: #5D88AF;
	float: left;
	cursor: pointer;
	margin: 1px 7px 0 7px;
	zoom: 1; /* for IE */
}
.dojoxGridActivedSwitch {
	font-weight: bold;
	color: black;
	float: left;
	margin: 1px 7px 0 7px;
	text-decoration: none!important; 
	zoom: 1; /* for IE */
}
.dojoxGridSeparator {
	float: left;
}
.dojoxGridPageTextHover {
	text-decoration: underline;
}
.dojoxGridDialogMargin {
	width: 220px!important;
	margin-bottom: 10px;
}
.dj_ie6 .dojoxGridDialogMargin {
	position: relative;
}
.dojoxGridDialogButton {
	width: 220px!important;
	text-align: right;
}
