({"gotoButtonTitle":"Přejít na určitou stránku","prevTip":"Předchozí stránka","pageCountIndication":" (Počet stránek: ${0})","pageStepLabelTemplate":"Stránka ${0}","descTemplate":"${2} - ${3} z ${1} ${0}","lastTip":"Poslední stránka","nextTip":"Další stránka","pageSizeLabelTemplate":"${0} položek na stránce","dialogCancel":"Storno","itemTitle":"položek","dialogConfirm":"Přejít","firstTip":"První stránka","allItemsLabelTemplate":"Všechny položky","dialogIndication":"Zadat číslo stránky","dialogTitle":"Přejít na stránku"})