({"waiRemoveRuleButtonTemplate":"Remove rule ${0}","addRuleButton":"Add Rule","a11yFilterBarDefButton":"Filter...","falseLabel":"False","conditionAfter":"after","relationAny":"any rules","conditionNotStartWith":"does not start with","relationMsgTail":"","defaultItemsName":"items","conditionContains":"contains","waiRelAny":"Match any of the following rules:","and":"and","waiClearButton":"Clear the filter","waiAddRuleButton":"Add a new rule","conditionLargerEqual":"greater than or equal","filterBarClearButton":"Clear filter","statusTipRelPost":"rules.","cancelButton":"Cancel","removeRuleButton":"Remove Rule","relationAll":"all rules","waiValueBoxTemplate":"Enter value to filter for rule ${0}","conditionNotContain":"does not contain","filterButton":"Filter","statusTipMsg":"Click the filter bar here to filter on values in ${0}.","waiRelAll":"Match all of the following rules:","closeFilterBarBtn":"Close filter bar","waiCancelButton":"Cancel this dialog","waiFilterBarDefButton":"Filter the table","statusTipTitleHasFilter":"Filter","any":"any","waiFilterButton":"Submit the filter","conditionIsEmpty":"is empty","conditionEqual":"equal","columnSelectLabel":"Column","conditionLessEqual":"less than or equal","conditionNotEndWith":"does not end with","or":"or","clearFilterDialogTitle":"Clear Filter","conditionLarger":"is greater than","relationMsgFront":"Match","statusTipHeaderColumn":"Column","conditionEndWith":"ends with","clearButton":"Clear","statusTipTitle":"Filter Bar","valueBoxLabel":"Value","conditionRange":"range","waiColumnSelectTemplate":"Column for rule ${0}","all":"all","conditionSelectLabel":"Condition","filterBarMsgHasFilterTemplate":"${0} of ${1} ${2} shown.","conditionStartsWith":"starts with","filterBarMsgNoFilterTemplate":"No filter applied","trueLabel":"True","filterBarDefButton":"Define filter","waiFilterBarClearButton":"Clear the filter","conditionBefore":"before","rangeTo":"to","rangeTemplate":"from ${0} to ${1}","anyColumnOption":"Any Column","filterDefDialogTitle":"Filter","ruleTitleTemplate":"Rule ${0}","conditionNotEqual":"does not equal","conditionIsNot":"is not","statusTipRelPre":"Match","conditionIs":"is","clearFilterMsg":"This will remove the filter and show all available records.","statusTipHeaderCondition":"Rules","waiConditionSelectTemplate":"Condition for rule ${0}","statusTipTitleNoFilter":"Filter Bar","anycolumn":"any column","conditionLess":"is less than"})