<div class="dojoxFileInfoPane">
	<table>
		<tbody>
			<tr>
				<td class="dojoxFileInfoLabel dojoxFileInfoNameLabel">${_messages.name}</td>
				<td class="dojoxFileInfoName" dojoAttachPoint="nameNode"></td>
			</tr>
			<tr>
				<td class="dojoxFileInfoLabel dojoxFileInfoPathLabel">${_messages.path}</td>
				<td class="dojoxFileInfoPath" dojoAttachPoint="pathNode"></td>
			</tr>
			<tr>
				<td class="dojoxFileInfoLabel dojoxFileInfoSizeLabel">${_messages.size}</td>
				<td class="dojoxFileInfoSize" dojoAttachPoint="sizeNode"></td>
			</tr>
		</tbody>
	</table>
	<div dojoAttachPoint="containerNode" style="display:none;"></div>
</div>