({"replaceDialogText":"Korvattu ${0} esiintymää.","eofDialogTextFind":"löytynyt","eofDialogText":"Viimeinen esiintymä ${0}","backwards":"Taaksepäin","replaceButton":"Korvaa","replaceLabel":"Korvaava:","matchCase":"Sama kirjainkoko","findTooltip":"Anna etsittävä teksti","replaceTooltip":"Anna korvaava teksti","replaceAllButton":"Korvaa kaikki","eofDialogTextReplace":"korvattu","findReplace":"Etsi ja korvaa","backwardsTooltip":"Etsi tekstiä taaksepäin","replaceAllButtonTooltip":"Korvaa kaikki teksti","replaceButtonTooltip":"Korvaa teksti","findLabel":"Etsi:","findButton":"Etsi","matchCaseTooltip":"Sama kirjainkoko","findButtonTooltip":"Etsi teksti","replaceAll":"Kaikki esiintymät"})