({"replaceDialogText":"Foram substituídas ${0} ocorrências.","eofDialogTextFind":"localizado","eofDialogText":"Última ocorrência ${0}","backwards":"Para trás","replaceButton":"Substituir","replaceLabel":"Substituir por:","matchCase":"Correspondência de maiúsculas/minúsculas","findTooltip":"Introduzir texto a localizar","replaceTooltip":"Introduzir texto de substituição","replaceAllButton":"Substituir tudo","eofDialogTextReplace":"substituído","findReplace":"Localizar e substituir","backwardsTooltip":"Procura de texto para trás","replaceAllButtonTooltip":"Substituir todo o texto","replaceButtonTooltip":"Substituir o texto","findLabel":"Localizar:","findButton":"Localizar","matchCaseTooltip":"Correspondência de maiúsculas/minúsculas","findButtonTooltip":"Localizar o texto","replaceAll":"Todas as ocorrências"})