({"singleSort":"Single Sort","indirectSelectionRadio":"Row ${0}, single selection, radio box","ascending":"Ascending","indirectSelectionCheckBox":"Row ${0}, multiple selection, check box","selectAll":"Select all","descending":"Descending","nestedSort":"Nested Sort","unsorted":"Do not sort this column","sortingState":"${0} - ${1}"})