<div class="dojoxGridView" role="presentation">
	<div class="dojoxGridHeader" dojoAttachPoint="headerNode" role="presentation">
		<div dojoAttachPoint="headerNodeContainer" style="width:9000em" role="presentation">
			<div dojoAttachPoint="headerContentNode" role="row"></div>
		</div>
	</div>
	<input type="checkbox" class="dojoxGridHiddenFocus" dojoAttachPoint="hiddenFocusNode" role="presentation" />
	<input type="checkbox" class="dojoxGridHiddenFocus" role="presentation" />
	<div class="dojoxGridScrollbox" dojoAttachPoint="scrollboxNode" role="presentation">
		<div class="dojoxGridContent" dojoAttachPoint="contentNode" hidefocus="hidefocus" role="presentation"></div>
	</div>
</div>
