({"conditionBefore":"abans","filterBarDefButton":"Defineix filtre","filterBarMsgNoFilterTemplate":"No s'ha aplicat cap filtre","conditionContains":"conté","clearFilterMsg":"Això eliminarà el filtre i mostrarà tots els registres disponibles.","all":"tot","waiValueBoxTemplate":"Especifiqueu el valor de filtre per a la regla ${0}","conditionRange":"interval","waiCancelButton":"Cancel·la aquest diàleg","relationAll":"totes les regles","conditionLarger":"és més que","valueBoxLabel":"Valor","statusTipMsg":"Feu clic aquí a la barra de filtre per filtrar els valors a ${0}.","waiRelAll":"Fes coincidir totes les regles següents:","closeFilterBarBtn":"Tancar la barra de filtre","a11yFilterBarDefButton":"Filtre...","trueLabel":"Cert","conditionIsNot":"no és","conditionIs":"és","ruleTitleTemplate":"Regla ${0}","waiRemoveRuleButtonTemplate":"Elimina la regla ${0}","and":"i","conditionLess":"és menys que","waiColumnSelectTemplate":"Columna per a la regla ${0}","anycolumn":"qualsevol columna","filterDefDialogTitle":"Filtre","waiConditionSelectTemplate":"Condició per a la regla ${0}","waiFilterButton":"Envia el filtre","statusTipTitle":"Barra de filtre","statusTipRelPre":"Coincidència","anyColumnOption":"Qualsevol columna","conditionSelectLabel":"Condició","cancelButton":"Cancel·la","conditionNotStartWith":"no comença per","conditionEndWith":"acaba per","statusTipTitleNoFilter":"Barra de filtre","columnSelectLabel":"Columna","conditionNotContain":"no conté","clearButton":"Esborra","conditionLargerEqual":"és més o igual que","any":"qualsevol","relationMsgFront":"Coincidència","statusTipHeaderColumn":"Columna","defaultItemsName":"elements","conditionEqual":"igual que","relationAny":"qualsevol regla","trueLabelEditable":"Seleccionat","or":"o","filterBarMsgHasFilterTemplate":"Es mostren ${0} de ${1} ${2}.","rangeTo":"a","conditionStartsWith":"comença per","waiRelAny":"Fes coincidir qualsevol de les regles següents:","statusTipRelPost":"regles.","relationMsgTail":"","filterBarClearButton":"Netejar filtre","incompleteRuleTip":"Aquesta regla no està completa","conditionLessEqual":"és menys o igual que","waiFilterBarClearButton":"Neteja el filtre","conditionAfter":"després","statusTipHeaderCondition":"Regles","falseLabel":"Fals","filterButton":"Filtre","clearFilterDialogTitle":"Netejar el filtre","statusTipTitleHasFilter":"Filtre","addRuleButton":"Afegeix regla","waiAddRuleButton":"Afegeix una regla nova","waiClearButton":"Neteja el filtre","conditionNotEqual":"no és igual que","conditionNotEndWith":"no acaba per","rangeTemplate":"de ${0} a ${1}","removeRuleButton":"Elimina regla","waiFilterBarDefButton":"Filtra la taula","conditionIsEmpty":"is empty"})