({"cancel":"Anuluj","toDic":"Dodaj do słownika","skip":"Pomiń","iMsg":"Brak propozycji pisowni","replaceAll":"Zastąp wszystko","widgetLabel":"Wsadowe sprawdzanie pisowni","skipAll":"Pomiń wszystko","suggestions":"Propozycje","iSkipAll":"Pomiń wszystkie pozycje podobne do tej","replaceWith":"Zastąp przez","msg":"Nie znaleziono błędów pisowni","replace":"Zastąp","unfound":"Nie znaleziono","iSkip":"Pomiń tę pozycję"})