({"replaceDialogText":"已替换 ${0} 个匹配项","eofDialogTextFind":"已找到","eofDialogText":"最后一个匹配项 ${0}","backwards":"向后","replaceButton":"替换","replaceLabel":"替换为：","matchCase":"匹配大小写","findTooltip":"输入要查找的文本","replaceTooltip":"输入要替换成的文本","replaceAllButton":"全部替换","eofDialogTextReplace":"已替换","findReplace":"查找和替换","backwardsTooltip":"向后搜索文本","replaceAllButtonTooltip":"替换所有文本","replaceButtonTooltip":"替换文本","findLabel":"查找：","findButton":"查找","matchCaseTooltip":"匹配大小写","findButtonTooltip":"查找文本","replaceAll":"所有匹配项"})