({"gotoButtonTitle":"Ir a una página específica","prevTip":"Página anterior","pageCountIndication":" (${0} páginas)","pageStepLabelTemplate":"Página ${0}","descTemplate":"${2} - ${3} de ${1} ${0}","lastTip":"Última página","nextTip":"Página siguiente","pageSizeLabelTemplate":"${0} elementos por página","dialogCancel":"Cancelar","itemTitle":"elementos","dialogConfirm":"Ir","firstTip":"Primera página","allItemsLabelTemplate":"Todos los elementos","dialogIndication":"Especifique el número de página","dialogTitle":"Ir a la página"})