<div class="dojoxGridBoolValueBox">
	<div class="dojoxGridTrueBox">
		<input dojoType="dijit.form.RadioButton" type='radio' name='a1' id='${_baseId}_rbTrue' checked="true" 
			dojoAttachPoint="rbTrue" dojoAttachEvent="onChange: onChange"/>
		<div class="dojoxGridTrueLabel" for='${_baseId}_rbTrue'>${_lblTrue}</div>
	</div>
	<div class="dojoxGridFalseBox">
		<input dojoType="dijit.form.RadioButton" dojoAttachPoint="rbFalse" type='radio' name='a1' id='${_baseId}_rbFalse'/>
		<div class="dojoxGridTrueLabel" for='${_baseId}_rbFalse'>${_lblFalse}</div>
	</div>
</div>
