.feedEntryEditor {
        color:#000000;
        background: #FFFFFF;
}

.feedEntryEditorMenuTable {
        background: #C0C0C0;      
        border:2px solid;
        border-top-color: lightgrey;
        border-left-color: lightgrey;
        border-right-color: darkgray;
        border-bottom-color: darkgray;
}

.feedEntryEditorMenu {
        cursor: pointer;
        color:#0000FF;
        text-align: left;
        font-size:  small;
}

.feedEntryEditorDisplayCheckbox {
        font-size:  small;
        text-align: left;
}

.feedEntryEditorMenu:hover {
        text-decoration: underline;
}

.feedEntryEditorTitle {
        color:#000000;
        background: #FFFFFF;
        text-align: left;
        #padding-left: 10px;
        font-size:  125%;
        font-weight: bold; 
}

.feedEntryEditorAuthor {
        color:#000000;
        background: #FFFFFF;
}

.feedEntryEditorAuthorNames {
        color:#000000;
        background: #FFFFFF;
        text-align: left;
        #padding-left: 10px;
        font-size:  110%;
        font-style: italic;
        font-weight: normal;
}

.feedEntryEditorSummary {
        color:#000000;
        background: #FFFFFF;
}

.feedEntryEditorUpdated {
        color:#000000;
        background: #FFFFFF;
}

.feedEntryEditorUpdatedText {
        color:#000000;
        background: #FFFFFF;
        font-size:  110%;
        font-style: italic;
        font-weight: normal;
}

.feedEntryEditorId {
        color:#000000;
        background: #FFFFFF;
}

.feedEntryEditorIdText {
        color:#000000;
        background: #FFFFFF;
        font-style: italic;
        font-size:  110%;
        font-weight: normal;
}

.feedEntryEditorContent {
        color:#000000;
        background: #FFFFFF;
}

.feedEntryEditorContributorNames {
        color:#000000;
        text-align: left;
        #padding-left: 10px;
        font-size:  125%;
        font-style: italic;
        font-weight: normal;
}
