({"pasteFromWord":"Pegar desde Word","paste":"Pegar","cancel":"Cancelar","instructions":"Pegue el contenido de Word en el siguiente recuadro de texto. Cuando esté satisfecho con el contenido que se debe insertar, pulse el botón pegar. Para abortar la inserción de texto, pulse el botón cancelar."})