({"set":"הוספה","text":"תיאור ","insertImageTitle":"הוספת תמונה","invalidMessage":"סוג קובץ תמונה לא חוקי","prePopuTextBrowse":" או נווטו לקובץ מקומי. ","browse":"עיון...‏ ","prePopuTextUrl":"ציינו URL של תמונה","url":"תמונה"})