({"cancel":"Annuleren","toDic":"Toevoegen aan woordenboek","skip":"Overslaan","iMsg":"Geen spellingsuggesties","replaceAll":"Alle vervangen","widgetLabel":"Groepsgewijze spellingcontrole","skipAll":"Alle overslaan","suggestions":"Suggesties","iSkipAll":"Al deze overslaan","replaceWith":"Vervangen door","msg":"Geen spelfouten gevonden","replace":"Vervangen","unfound":"Niet gevonden","iSkip":"Deze overslaan"})