.dijitAdditionalEditorIconSmiley {
	/* Toolbar icon for Smiley plugin */
	background-repeat: no-repeat;
	background-position: center center;
	width: 18px;
	height: 18px;
	background-image: url(../icons/smiley.gif);
}

.dijitDisabled .dijitAdditionalEditorIconSmiley {
	/* Toolbar icon for disabled Smiley plugin */
	background-image: url(../icons/smileyDisabled.gif);
}

.dijitEditorSmileyPalette {
	border: 1px solid black;
	background: white;
	overflow: hidden;
	overflow-x: hidden;
	overflow-y: hidden;
}

.dijitEditorSmileyPalette .dijitPaletteCell {
    height: 23px;
    width: 27px;
}

.dijitEditorSmileyPalette .dijitPaletteImg {
    border: none;
    height: 20px;
    width: 20px;
    padding: 2px 2px;
}

.dijitEditorSmileyPalette .dijitPaletteCellHover .dijitPaletteImg {
	border: 1px solid black;
	padding: 1px 1px;
}

