({"conditionBefore":"алдында","filterBarDefButton":"Сүзгіні анықтау","filterBarMsgNoFilterTemplate":"Сүзгі қолданылмады","conditionContains":"құрамында бар","clearFilterMsg":"Бұл сүзгіні жояды және барлық қол жетімді жазбаларды көрсетеді.","all":"барлығы","waiValueBoxTemplate":"${0} ережесін сүзу үшін мәнді енгізу","conditionRange":"ауқым","waiCancelButton":"Осы тілқатысу терезесін болдырмау","relationAll":"барлық ережелер","conditionLarger":"үлкендеу","valueBoxLabel":"Мән","statusTipMsg":"${0} ішіндегі мәндер бойынша сүзу үшін сүзгі тақтасын нұқыңыз.","waiRelAll":"Барлық мына ережелерге сәйкес:","closeFilterBarBtn":"Сүзгі тақтасын жабу","a11yFilterBarDefButton":"Сүзгі...","trueLabel":"Шын","conditionIsNot":"емес","conditionIs":"–","ruleTitleTemplate":"${0} ережесі","waiRemoveRuleButtonTemplate":"${0} ережесін алып тастау","and":"және","conditionLess":"аздау","waiColumnSelectTemplate":"${0} ережесінің бағаны","anycolumn":"кез келген баған","filterDefDialogTitle":"Сүзгі","waiConditionSelectTemplate":"${0} ережесінің шарты","waiFilterButton":"Сүзгіні жіберу","statusTipTitle":"Сүзгі тақтасы","statusTipRelPre":"Сәйкес келу","anyColumnOption":"Кез келген баған","conditionSelectLabel":"Шарт","cancelButton":"Болдырмау","conditionNotStartWith":"басталмайды","conditionEndWith":"аяқталады","statusTipTitleNoFilter":"Сүзгі тақтасы","columnSelectLabel":"Баған","conditionNotContain":"құрамында жоқ","clearButton":"Тазалау ","conditionLargerEqual":"үлкендеу немесе тең","any":"кез келген","relationMsgFront":"Сәйкес келу","statusTipHeaderColumn":"Баған","defaultItemsName":"элементтер","conditionEqual":"тең","relationAny":"кез келген ереже","trueLabelEditable":"Белгіленген","or":"немесе","filterBarMsgHasFilterTemplate":"${1} ${2} ішінен ${0} көрсетілді.","rangeTo":"неге","conditionStartsWith":"басталады","waiRelAny":"Мына ережелерге сәйкес:","statusTipRelPost":"ережелер.","relationMsgTail":"","filterBarClearButton":"Сүзгіні тазалау","incompleteRuleTip":"Бұл ереже толық емес.","conditionLessEqual":"аздау немесе тең","waiFilterBarClearButton":"Сүзгіні тазалау","conditionAfter":"артында","statusTipHeaderCondition":"Ережелер","falseLabel":"Жалған","filterButton":"Сүзгі","clearFilterDialogTitle":"Сүзгіні тазалау","statusTipTitleHasFilter":"Сүзгі","addRuleButton":"Ереже қосу","waiAddRuleButton":"Жаңа ереже қосу","waiClearButton":"Сүзгіні тазалау","conditionNotEqual":"тең емес","conditionNotEndWith":"аяқталмайды","rangeTemplate":"${0} мәнінен ${1} мәніне","removeRuleButton":"Ережені алып тастау","waiFilterBarDefButton":"Кестені сүзу","conditionIsEmpty":"is empty"})