<div class="dojoxWizard" dojoAttachPoint="wizardNode">
    <div class="dojoxWizardContainer" dojoAttachPoint="containerNode"></div>
    <div class="dojoxWizardButtons" dojoAttachPoint="wizardNav">
        <button dojoType="dijit.form.Button" type="button" dojoAttachPoint="previousButton">${previousButtonLabel}</button>
        <button dojoType="dijit.form.Button" type="button" dojoAttachPoint="nextButton">${nextButtonLabel}</button>
        <button dojoType="dijit.form.Button" type="button" dojoAttachPoint="doneButton" style="display:none">${doneButtonLabel}</button>
        <button dojoType="dijit.form.Button" type="button" dojoAttachPoint="cancelButton">${cancelButtonLabel}</button>
    </div>
</div>
