({"cancel":"Болдырмау","toDic":"Сөздікке қосу","skip":"Өткізіп жіберу","iMsg":"Емле ұсыныстары жоқ","replaceAll":"Барлығын ауыстыру","widgetLabel":"Бума емлесін тексеру","skipAll":"Барлығын өткізіп жіберу","suggestions":"Ұсыныстар","iSkipAll":"Осы сияқты барлығын өткізіп жіберу","replaceWith":"Келесімен ауыстыру","msg":"Қате жазылған сөздер табылмады","replace":"Ауыстыру","unfound":"Табылмады","iSkip":"Бұны өткізіп жіберу"})