({"emoticonLaughing":"rient","emoticonCool":"guai","emoticonTongue":"llengua","emoticonCrying":"plorant","emoticonOops":"ep","emoticonFrown":"espantat","emoticonAngry":"enfadat","emoticonShy":"avergonyit","emoticonNo":"no","emoticonAngel":"àngel","smiley":"Insereix emoticona","emoticonIdea":"idea","emoticonEyebrow":"cella","emoticonSmile":"somriure","emoticonWink":"ullet","emoticonYes":"sí","emoticonGrin":"gran somriure","emoticonGoofy":"babau","emoticonHalf":"meitat"})