({"gotoButtonTitle":"Gå till en viss sida","prevTip":"Föregående sida","pageCountIndication":" (${0} sidor)","pageStepLabelTemplate":"Sidan ${0}","descTemplate":"${2} - ${3} av ${1} ${0}","lastTip":"Sista sidan","nextTip":"Nästa sida","pageSizeLabelTemplate":"${0} objekt per sida","dialogCancel":"Avbryt","itemTitle":"objekt","dialogConfirm":"OK","firstTip":"Första sidan","allItemsLabelTemplate":"Alla objekt","dialogIndication":"Ange sidnumret","dialogTitle":"Gå till sida"})