({"emoticonLaughing":"하하","emoticonCool":"멋진","emoticonTongue":"메롱","emoticonCrying":"울음","emoticonOops":"어머나","emoticonFrown":"불만","emoticonAngry":"화남","emoticonShy":"부끄러움","emoticonNo":"아니오","emoticonAngel":"천사","smiley":"이모티콘 삽입","emoticonIdea":"아이디어","emoticonEyebrow":"글쎄","emoticonSmile":"미소","emoticonWink":"윙크","emoticonYes":"예","emoticonGrin":"씨익","emoticonGoofy":"바보","emoticonHalf":"고민 중"})