/*
	Copyright (c) 2004-2011, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["dojox.charting.themes.Midwest"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.themes.Midwest"] = true;
dojo.provide("dojox.charting.themes.Midwest");
dojo.require("dojox.charting.Theme");

(function(){
	var dxc=dojox.charting;
	dxc.themes.Midwest=new dxc.Theme({
		colors: [
			"#927b51",
			"#a89166",
			"#80c31c",
			"#bcdd5a",
			"#aebc21"
		]
	});
})();

}
