({"buttonSet":"設定","insertTableTitle":"插入表格","insertTableRowAfterLabel":"在後面新增一個列","buttonCancel":"取消","center":"置中","deleteTableColumnLabel":"刪除欄","right":"靠右","insertTableColumnBeforeLabel":"在前面新增一個直欄","tableWidth":"表格寬度：","buttonInsert":"插入","default":"預設值","align":"對齊：","insertTableRowBeforeLabel":"在前面新增一個列","cellSpacing":"資料格間距：","pixels":"像素","selectTableLabel":"選取表格","rows":"列：","modifyTableTitle":"修改表格","cellPadding":"資料格行距：","deleteTableRowLabel":"刪除列","backgroundColor":"背景顏色：","insertTableColumnAfterLabel":"在後面新增一個直欄","left":"靠左","borderThickness":"邊框寬度","columns":"欄：","percent":"百分比","borderColor":"邊框顏色："})