({"gotoButtonTitle":"특정 페이지 찾아가기","prevTip":"이전 페이지","pageCountIndication":"(${0}페이지)","pageStepLabelTemplate":"${0}페이지","descTemplate":"${2} - ${1} ${0} 중 ${3}","lastTip":"마지막 페이지","nextTip":"다음 페이지","pageSizeLabelTemplate":"페이지당 ${0}개 항목","dialogCancel":"취소","itemTitle":"항목","dialogConfirm":"이동","firstTip":"첫 페이지","allItemsLabelTemplate":"모든 항목","dialogIndication":"페이지 번호 지정","dialogTitle":"페이지 찾아가기"})