({"saturationPickerTitle":"Telítettség kiválasztó","huePickerTitle":"Árnyalat kiválasztó","valueLabel":"v","blueLabel":"b","saturationLabel":"s","greenLabel":"g","redLabel":"r","hueLabel":"h","hexLabel":"hex","degLabel":"°"})