({"replaceDialogText":"S'han substituït ${0} aparicions.","eofDialogTextFind":"trobat","eofDialogText":"Darrera aparició ${0}","backwards":"Cap enrere","replaceButton":"Substitueix","replaceLabel":"Substitueix per:","matchCase":"Coincidència de majúscules i minúscules","findTooltip":"Especifiqueu el text que voleu trobar","replaceTooltip":"Especifiqueu el text amb què el voleu substituir","replaceAllButton":"Substitueix tot","eofDialogTextReplace":"substituït","findReplace":"Cerca i substitueix","backwardsTooltip":"Cerca text cap enrere","replaceAllButtonTooltip":"Substitueix tot el text","replaceButtonTooltip":"Substitueix el text","findLabel":"Cerca:","findButton":"Cerca","matchCaseTooltip":"Coincidència de majúscules i minúscules","findButtonTooltip":"Troba el text","replaceAll":"Totes les aparicions"})