/*
You can use this file as is or as a starting point for you own styling
*/

.string,
.function .title,
.class .title, 
.tag .attribute .value,
.css .rules .value,
.preprocessor,
.ruby .symbol,
.ruby .instancevar,
.ruby .class .parent,
.built_in,
.sql .aggregate,
.django .template_tag,
.django .variable,
.smalltalk .class {
	color: #800;
}

.comment,
.java .annotation,
.template_comment {
	color: #888;
}

.number,
.regexp,
.javascript .literal,
.smalltalk .symbol,
.smalltalk .char {
	color: #080;
}

.javadoc,
.ruby .string,
.python .decorator,
.django .filter .argument,
.smalltalk .localvars,
.smalltalk .array,
.css .attr_selector,
.xml .pi {
	color: #88F;
}

.keyword,
.css .id,
.phpdoc,
.function .title,
.class .title,
.vbscript .built_in,
.sql .aggregate,
.rsl .built_in,
.smalltalk .class,
.xml .tag .title {
	font-weight: bold;
}
