<div class="dojoxExpandoPane">
	<div dojoAttachPoint="titleWrapper" class="dojoxExpandoTitle">
		<div class="dojoxExpandoIcon" dojoAttachPoint="iconNode" dojoAttachEvent="onclick:toggle"><span class="a11yNode">X</span></div>			
		<span class="dojoxExpandoTitleNode" dojoAttachPoint="titleNode">${title}</span>
	</div>
	<div class="dojoxExpandoWrapper" dojoAttachPoint="cwrapper" dojoAttachEvent="ondblclick:_trap">
		<div class="dojoxExpandoContent" dojoAttachPoint="containerNode"></div>
	</div>
</div>
