.ganttToolbarZoomIn {
	background-image: url("images/zoomIn.png");
	background-repeat: no-repeat;
	height: 30px;
	width: 30px;
	padding: 2px;
}
.ganttToolbarZoomOut {
	background-image: url("images/zoomOut.png");
	background-repeat: no-repeat;
	height: 30px;
	width: 30px;
	padding: 2px;
}
.ganttToolbarMicro {
	background-image: url("images/zoomInTime.png");
	background-repeat: no-repeat;
	height: 30px;
	width: 30px;
	padding: 2px;
}
.ganttToolbarTele {
	background-image: url("images/zoomOutTime.png");
	background-repeat: no-repeat;
	height: 30px;
	width: 30px;
	padding: 2px;
}
.ganttToolbarSave {
	background-image: url("images/save.png");
	background-repeat: no-repeat;
	height: 30px;
	width: 30px;
	padding: 2px;
}
.ganttToolbarLoad {
	background-image: url("images/load.png");
	background-repeat: no-repeat;
	height: 30px;
	width: 30px;
	padding: 2px;
}
.ganttToolbarActionHover{
	border: 2px solid blue;
	padding: 0px;
}
.ganttToolbar {
	cursor: pointer;
}
.ganttPanelHeader{
	position: relative;
}
.ganttPanelNameHeaders{
	position: relative;
	overflow: hidden;
}
.ganttPanelNamesContainer{
	position: relative;
	overflow: hidden;
	border-left: #f1f3f1 1px solid;
	border-bottom: #f1f3f1 1px solid;
}
.ganttPanelNames{
	position: relative;
	overflow: hidden;
	border-left: #f1f3f1 1px solid;
	border-bottom: #f1f3f1 1px solid;
	background: url("images/bg.png") repeat scroll 0 0 transparent;
}
.ganttPanelTime{
	position: relative;
}
.ganttPanelTimeContainer{
	position: relative;
	overflow: hidden;
	direction: ltr;
}
.ganttContentDataContainer{
	position: relative;
	overflow: scroll;
	border-left: #f1f3f1 1px solid;
	direction: ltr;
}
.ganttDivCell{
	position: relative;
}
.ganttMoveInfo {
	font-family: Tahoma, Arial;
	font-size: 10px;
	color: #006600;
	white-space: nowrap;
}
.ganttDescProject {
	font-family: Tahoma, Arial;
	font-size: 10px;
	color: #3B3B3B;
	cursor: default;
	white-space: nowrap;
	z-index: 6;
	position: absolute;
}
.ganttProjectNameItem{
	cursor:pointer;
	color: #003366;
	font-weight: bold;
	font-size: 12px;
	font-family: Tahoma, Arial;
	white-space:nowrap;
	height:15px;
	z-index:6;
	position: absolute;
}
.ganttProjectNameItemHover{
	border-top: 1px solid black;
	border-bottom: 1px solid black;
}
.ganttProjectItem{
	z-index: 6;
	position: absolute;
}
.ganttTblProjectItem{
	border: solid 1px #555555;
}
.ganttDivTaskInfo{
	text-align: center;
	z-index: 7;
	position: absolute;
	left: 0px;
	top: 0px;
}
.ganttTblTime {
	background-color: transparent;
	cursor: pointer;
	margin-top: 0px;
}
.ganttHourNumber {
	font-size: 7px;
	color: #858585;
	border: 1px solid transparent;
	border-top: 1px solid #DBECFF;
	height: 30px;
}
.ganttHourClass {
	text-align: center;
	padding: 5px 2px;
	height: 20px;
}
.ganttHourNumberAM {
	background: url("images/am.png") repeat scroll 0 0 transparent;
}
.ganttHourNumberPM {
	background: url("images/pm.png") repeat scroll 0 0 transparent;
}
.ganttDayNumber {
	font-family: Tahoma, Arial;
	font-weight: bold;
	font-size: 9px;
	color: #858585;
	border: 1px solid transparent;
	border-top: 1px solid #DBECFF;
	height: 30px;
}
.ganttDayNumberWeekend {
	background: url("images/bg.png") repeat scroll 0 0 transparent;
}
.ganttWeekNumber {
	font-family: Tahoma, Arial;
	font-weight: bold;
	font-size: 9px;
	color: #858585;
	border: 1px solid transparent;
	border-top: 1px solid #DBECFF;
	border-right: 1px solid #DBECFF;
	height:30px;
}
.ganttMonthNumber {
	font-family: Tahoma, Arial;
	font-weight: bold;
	font-size: 9px;
	color: #858585;
	border: 1px solid transparent;
	border-top: 1px solid #DBECFF;
	border-right: 1px solid #DBECFF;
	height: 30px;
}
.ganttYearNumber {
	font-family: Tahoma, Arial;
	font-weight: bold;
	font-size: 9px;
	color: #858585;
	border: 1px solid transparent;
	border-top: 1px solid #DBECFF;
	border-right: 1px solid #DBECFF;
	height: 30px;
}
.ganttTabelControl{
	width: 100%; 
	position: relative;
}
.ganttContextMenu {
	z-index: 10;
	width: 200px;
	cursor: pointer;
	font-family: Tahoma, Arial;
	font-size: 12px;
	border: 1px solid #b5bcc7;
	margin: 0px;
	padding: 0px;
	background-color: #fff;
	background-repeat: repeat-x;
	border-collapse: separate;
	border-spacing: 0 0;
}
.ganttContextMenuItem {
	background-image: url("images/menuHighlight.png");
	background-position:0px -40px;
	background-repeat:repeat-x;
	padding: 3px 10px 4px;
	height:18px;
}
.ganttContextMenuItemHover {
	border:solid 1px #769dc0;
	padding: 2px 9px 3px;
	background-color: #9dcfff;
	background-position:0px 0px;
	color:#000;
}
.ganttMenuDialogDescCell{
	width:150px;
	padding: 3px;
	border-bottom: 1px solid gray;
	text-align: center;
}
.ganttMenuDialogDescCellValue{
	width:150px;
	padding: 3px;
	border-bottom: 1px solid gray;
	text-align: center;
}
.ganttMenuDialogInputCellHeader{
	font-size:12px;
	font-family:Tahoma,Arial;
	font-weight: bold;
	padding: 10px 3px 5px;
}
.ganttMenuDialogInputCell{
	padding: 3px;
}
.ganttMenuDialogInputCellValue{
	padding: 3px;
}
.ganttHeaderCover {
	z-index: 999;
	position: absolute;
}
.ganttResourceHeader {
	font-family: Tahoma, Arial;
	background: url("images/resourceHeader.png") repeat scroll 0 0 transparent;
	color: black;
	padding: 3px 0px 0px 6px;
	height: 25px;
	font-weight: bold;
	position:relative;
}
.ganttResourceContent{
	position:relative;
}
.ganttTaskNameItem {
	font-family: Tahoma, Arial;
	font-size: 11px;
	font-weight: bold;
	color: #7D7D7D;
	white-space:nowrap;
	height:15px;
	z-index:1;
	position: absolute;
	left:20px;
}
.ganttMenuPanel{
	visibility:hidden;
	z-index:10;
	position:absolute;
}
.ganttDialogContentCell{
	width:300px;
	font-size:11px;
	font-family:Tahoma,Arial;
}
.ganttDialogContentHeader{
	font-size:12px;
	font-family:Tahoma,Arial;
	font-weight: bold;
}
.ganttOwnerNameItem {
	font-family: Tahoma, Arial;
	font-size: 11px;
	font-weight: bold;
	color: black;
	cursor:pointer;
	white-space:nowrap;
	height:15px;
	z-index:1;
	position: absolute;
	left:20px;
}
.ganttTaskBar {
	background: url("images/taskBar.png") repeat scroll 0 0 transparent;
	-moz-border-radius: 10px;
	z-index:1;
	position:absolute;
}
.ganttOwnerBar {
	z-index:1;
	position:absolute;
}
.ganttOwnerTaskBar {
	background: url("images/ownerBar.png") repeat scroll 0 0 transparent;
	border-top: solid 1px black;
	border-bottom: solid 1px black;
	position:absolute;
	top:0px;
}
.ganttRowHighlight {
	background: url("images/rowHighlight.png") repeat scroll 0 0 transparent;
	position: absolute;
	z-index: 1;
}
.ganttTaskPanel {
	background: url("images/bg.png") repeat scroll 0 0 transparent;
	overflow: hidden;
	position:relative;
}
.ganttOwnerPanel {
	overflow: hidden;
	position:relative;
	background: url("images/resourceBg.png") repeat scroll 0 0 transparent;
}
.ganttContent {
	margin: 15px 63px 0;
}
.ganttTaskLineVerticalLeft{
	border-width: 0px 0px 0px 1px; 
	border-style: dotted; 
	border-color: #86A3BE; 
	margin: 0px; 
	padding: 0px;
	z-index:10;
	position: absolute;
}
.ganttTaskLineHorizontalLeft{
	position: absolute;
	border-width: 1px 0px 0px 0px;
	font-size: 1px;
	border-style: dotted; 
	border-color: #86A3BE;
	margin: 0px; 
	padding: 0px;
	z-index:10;
}
.ganttTaskLineVerticalRight{
	border-width: 0px 0px 0px 1px; 
	border-style: solid; 
	border-color: #4A8F43;
	margin: 0px; 
	padding: 0px;
	z-index:6;
	font-size: 1px;
	position: absolute;
}
.ganttTaskLineHorizontal{
	height:1px;
	border-color: #4A8F43;
	border-style: solid;
	border-width: 1px 0px 0px 0px;
	margin: 0px; 
	padding: 0px;
	z-index:6;
	position: absolute;
}
.ganttTaskArrowImg{
	z-index:6;
	margin: 0px; 
	padding: 0px;
	width:7px;
	height:14px;
	position: absolute;
}
.ganttTaskLineVerticalRightPlus{
	border-width: 0px 0px 0px 1px; 
	border-style: solid; 
	border-color: #519145;
	margin: 0px; 
	padding: 0px;
	z-index:6;
	font-size: 1px;
	position: absolute;
}
.ganttTaskLineHorizontalPlus{
	height:1px;
	border-color: #519145;
	border-style: solid;
	border-width: 1px 0px 0px 0px;
	margin: 0px; 
	padding: 0px;
	z-index:6;
	position: absolute;
}
.ganttTaskArrowImgPlus{
	z-index:6;
	margin: 0px; 
	padding: 0px;
	width:7px;
	height:14px;
	position: absolute;
}
.ganttTaskItemControl{
	z-index:6;
	position:absolute;
}
.ganttTaskDivTaskItem{
	z-index:6;
	position: absolute;
	left:0px;
	top:0px;
}
.ganttTaskTblTaskItem{
	border: solid 1px #5FF55F;
}
.ganttTaskDivTaskInfo{
	text-align:center;
	font-size:9px;
	z-index:7;
	position: absolute;
	left:0px;
	top:0px;
}
.ganttTaskDivTaskName{
	z-index:7;
	position: absolute;
	left:0px;
	top:0px;
}
.ganttTaskDivMoveInput{
	visibility:hidden;
	width:1px;
	height:1px;
}
.ganttTaskDivResizeInput{
	visibility:hidden;
	width:1px;
	height:1px;
}
.ganttTaskDivResize{
	z-index:10;
	position: absolute;
	top:0px;
}
.ganttTaskTaskNameItem{
	font-family: Tahoma, Arial;
	cursor:pointer;
	white-space:nowrap;
	height:15px;
	z-index:6;
	position: absolute;
	left:20px;
}
.ganttTaskTaskNameItemHover{
	border-top: 1px solid black;
	border-bottom: 1px solid black;
}
.ganttTaskDescTask {
	font-family: Tahoma, Arial;
	font-size: 10px;
	color: #008000;
	cursor: default;
	white-space: nowrap;
	z-index:6;
	position:absolute;
}
.ganttTaskIdentifier{
	color:#7D7D7D;
	font-family:Tahoma,Arial;
	font-size:11px;
	font-weight:bold;
	border: solid 1px #7D7D7D;
	-moz-border-radius: 20px;
	-khtml-border-radius: 20px;
	-webkit-border-radius: 20px;
	border-radius: 20px;
	padding: 0px 3px;
	cursor:pointer;
	white-space:nowrap;
	height:14px;
	z-index:6;
	position:absolute;
}
.ganttTaskTreeImage{
	cursor: pointer;
	z-index:12;
	position: absolute;
}

.ganttResourceTableControl{
	width: 100%; 
	position: relative;
}
.ganttResourceContentDataContainer{
	position:relative;
	overflow:scroll;
	border-left:#f1f3f1 1px solid;
	direction: ltr;
}
.ganttResourcePanelNames{
	position:relative;
	overflow:hidden;
	border-left:#f1f3f1 1px solid;
	border-bottom:#f1f3f1 1px solid;
}
.ganttResourceDivCell{
	position: relative;
}
.ganttResourceLineVerticalLeft{
	border-width: 0px 0px 0px 1px; 
	border-style: dotted; 
	border-color: #86A3BE; 
	margin: 0px; 
	padding: 0px;
	z-index:10;
	position: absolute;
}
.ganttResourceLineHorizontalLeft{
	z-index:10;
	height:1px;
	position: absolute;
	border-width: 1px 0px 0px 0px;
	font-size: 1px;
	border-style: dotted; 
	border-color: #86A3BE;
	margin: 0px; 
	padding: 0px;
}
.ganttResourceTreeImage{
	cursor: pointer;
	z-index:12;
	position: absolute;
}
.ganttResourcePanelNamesOwners{
	position:relative;
	background: url("images/resourceBg.png") repeat scroll 0 0 transparent;
}
.ganttImageProgressFilled{
	background: url("images/projProgressFilled.png") repeat scroll 0 0 transparent;
}
.ganttImageProgressBg{
	background: url("images/projProgressBg.png") repeat scroll 0 0 transparent;
}
.ganttImageTaskProgressFilled{
	background: url("images/taskProgressFilled.png") repeat scroll 0 0 transparent;
}
.ganttImageTaskProgressBg{
	background: url("images/taskProgressBg.png") repeat scroll 0 0 transparent;
}
.ganttImageTreeCollapse{
	background: url("images/collapse.png") no-repeat scroll 0 0 transparent;
	height: 9px;
	width: 9px;
}
.ganttImageTreeExpand{
	background: url("images/expand.png") no-repeat scroll 0 0 transparent;
	height: 9px;
	width: 9px;
}
.ganttImageArrow{
	background: url("images/taskArrow.gif") no-repeat scroll 0 0 transparent;
}
