({"emoticonLaughing":"nauraa","emoticonCool":"viileä hymy","emoticonTongue":"näyttää kieltä","emoticonCrying":"itkeä","emoticonOops":"hups","emoticonFrown":"irvistää","emoticonAngry":"vihainen","emoticonShy":"ujo","emoticonNo":"ei","emoticonAngel":"enkeli","smiley":"Lisää hymiö","emoticonIdea":"idea","emoticonEyebrow":"ihmetellä","emoticonSmile":"hymyillä","emoticonWink":"iskeä silmää","emoticonYes":"kyllä","emoticonGrin":"virnistää","emoticonGoofy":"hölmö","emoticonHalf":"puolikas"})