.mblFixedSpliter {
	width: 100%;
	height: 100%;
}

.mblFixedSplitterPane {
	position: absolute;
	overflow-x: hidden;
	overflow-y: auto;
}

.mblFixedSplitterPaneH {
	position: absolute;
	height: 100%;
	top: 0px;
}

.mblFixedSplitterPaneV {
	position: absolute;
	width: 100%;
	left: 0px;
}
