({"emoticonLaughing":"râs","emoticonCool":"calm","emoticonTongue":"limbă","emoticonCrying":"plâns","emoticonOops":"vai","emoticonFrown":"posac","emoticonAngry":"furios","emoticonShy":"timid","emoticonNo":"nu","emoticonAngel":"înger","smiley":"Inserare emoticoane","emoticonIdea":"idee","emoticonEyebrow":"sprânceană","emoticonSmile":"zâmbet","emoticonWink":"face cu ochiul","emoticonYes":"da","emoticonGrin":"rânjet","emoticonGoofy":"tont","emoticonHalf":"jumătate"})