({"conditionBefore":"előtte","filterBarDefButton":"Szűrő megadása","filterBarMsgNoFilterTemplate":"Nincs érvényben szűrő","conditionContains":"tartalmaz","clearFilterMsg":"Ez eltávolítja a szűrőt és megjelenít minden elérhető rekordot.","all":"mind","waiValueBoxTemplate":"Adja meg a szűrési értéket ${0} szabályhoz","conditionRange":"tartomány","waiCancelButton":"Párbeszédpanel visszavonása","relationAll":"minden szabály","conditionLarger":"nagyobb, mint","valueBoxLabel":"Érték","statusTipMsg":"Kattintson erre a szűrősávra ${0} értékeinek szűréséhez.","waiRelAll":"Az összes következő szabály egyeztetése:","closeFilterBarBtn":"Szűrősáv bezárása","a11yFilterBarDefButton":"Szűrő...","trueLabel":"Igaz","conditionIsNot":"nem","conditionIs":" ","ruleTitleTemplate":"Szabály: ${0}","waiRemoveRuleButtonTemplate":"Szabály eltávolítása: ${0}","and":"és","conditionLess":"kisebb, mint","waiColumnSelectTemplate":"${0} szabály oszlopa","anycolumn":"bármely oszlop","filterDefDialogTitle":"Szűrő","waiConditionSelectTemplate":"${0} szabály feltétele","waiFilterButton":"Szűrő elküldése","statusTipTitle":"Szűrősáv","statusTipRelPre":"Egyeztetés","anyColumnOption":"Bármely oszlop","conditionSelectLabel":"Feltétel","cancelButton":"Mégse","conditionNotStartWith":"kezdete nem","conditionEndWith":"vége","statusTipTitleNoFilter":"Szűrősáv","columnSelectLabel":"Oszlop","conditionNotContain":"nem tartalmaz","clearButton":"Törlés","conditionLargerEqual":"nem kisebb, mint","any":"bármely","relationMsgFront":"Egyeztetés","statusTipHeaderColumn":"Oszlop","defaultItemsName":"elem","conditionEqual":"egyenlő","relationAny":"bármely szabály","trueLabelEditable":"Bejelölve","or":"vagy","filterBarMsgHasFilterTemplate":"${0} elem megjelenítve, összesen ${1} ${2} ","rangeTo":" - ","conditionStartsWith":"kezdete","waiRelAny":"A következő szabályok bármelyikének egyeztetése:","statusTipRelPost":"szabály.","relationMsgTail":"","filterBarClearButton":"Szűrő törlése","incompleteRuleTip":"Ez a szabály nem teljes.","conditionLessEqual":"nem nagyobb, mint","waiFilterBarClearButton":"Szűrő törlése","conditionAfter":"utána","statusTipHeaderCondition":"Szabályok","falseLabel":"Hamis","filterButton":"Szűrő","clearFilterDialogTitle":"Szűrő törlése","statusTipTitleHasFilter":"Szűrő","addRuleButton":"Szabály hozzáadása","waiAddRuleButton":"Új szabály hozzáadása","waiClearButton":"Szűrő törlése","conditionNotEqual":"nem egyenlő","conditionNotEndWith":"vége nem","rangeTemplate":"${0} - ${1}","removeRuleButton":"Szabály eltávolítása","waiFilterBarDefButton":"Táblázat szűrése","conditionIsEmpty":"is empty"})