<div class="dojoxScrollWindow" dojoAttachEvent="onmouseenter: _enter, onmouseleave: _leave">
    <div class="dojoxScrollWrapper" style="${style}" dojoAttachPoint="wrapper" dojoAttachEvent="onmousemove: _calc">
	<div class="dojoxScrollPane" dojoAttachPoint="containerNode"></div>
    </div>
    <div dojoAttachPoint="helper" class="dojoxScrollHelper"><span class="helperInner">|</span></div>
</div>