({"gotoButtonTitle":"Aller à une page spécifique","prevTip":"Page précédente","pageCountIndication":" (${0} pages)","pageStepLabelTemplate":"Page ${0}","descTemplate":"${2} - ${3} sur ${1} ${0}","lastTip":"Dernière page","nextTip":"Page suivante","pageSizeLabelTemplate":"${0} éléments par page","dialogCancel":"Annuler","itemTitle":"éléments","dialogConfirm":"Aller","firstTip":"Première page","allItemsLabelTemplate":"Tous les éléments","dialogIndication":"Spécifiez le numéro de page","dialogTitle":"Aller à la page"})