.dijitAdditionalEditorIconInsertAnchor {
	background-image: url(../icons/anchor.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 18px;
	height: 18px;
}

.dijitDisabled .dijitAdditionalEditorIconInsertAnchor {
	background-image: url(../icons/anchorDisabled.gif);
	background-repeat: no-repeat;
	background-position: center center;
	width: 18px;
	height: 18px;
}


