({"emoticonLaughing":"lach","emoticonCool":"cool","emoticonTongue":"tong","emoticonCrying":"bedroefd","emoticonOops":"oeps","emoticonFrown":"frons","emoticonAngry":"kwaad","emoticonShy":"verlegen","emoticonNo":"nee","emoticonAngel":"engel","smiley":"Emoticon invoegen","emoticonIdea":"idee","emoticonEyebrow":"verbaasd","emoticonSmile":"glimlach","emoticonWink":"knipoog","emoticonYes":"ja","emoticonGrin":"grijns","emoticonGoofy":"goofy","emoticonHalf":"half"})