<div class="Progress" dojoAttachEvent="mousedown:startDrag">
    
    <div class="ProgressLoaded" dojoAttachPoint="progressLoaded"></div>
    <div class="ProgressPosition" dojoAttachPoint="progressPosition"></div>
	<div class="ProgressHandle" dojoAttachPoint="handle" dojoAttachEvent="mouseover:handleOver, mouseout:handleOut"></div>
</div>