({"gotoButtonTitle":"מעבר לעמוד מסוים ","prevTip":"העמוד הקודם  ","pageCountIndication":" (${0} עמודים)","pageStepLabelTemplate":"עמוד ${0}","descTemplate":"${2} - ${3} מתוך ${1} ${0}","lastTip":"עמוד אחרון ","nextTip":"העמוד הבא ","pageSizeLabelTemplate":"${0} פריטים בעמוד  ","dialogCancel":"ביטול","itemTitle":"פריטים","dialogConfirm":"ביצוע","firstTip":"עמוד ראשון ","allItemsLabelTemplate":"כל הפריטים ","dialogIndication":"ציינו מספר עמוד ","dialogTitle":"מעבר לעמוד "})