({"conditionBefore":"voor","filterBarDefButton":"Filter definiëren","filterBarMsgNoFilterTemplate":"Geen filter gebruikt","conditionContains":"bevat","clearFilterMsg":"Hiermee verwijdert u het filter en worden alle beschikbare records afgebeeld.","all":"alle","waiValueBoxTemplate":"Geef een filterwaarde op voor regel ${0}","conditionRange":"bereik","waiCancelButton":"Dit venster sluiten","relationAll":"alle regels","conditionLarger":"is groter dan","valueBoxLabel":"Waarde","statusTipMsg":"Klik op de filterbalk om te filterne op waarden in ${0}.","waiRelAll":"Voldoen aan alle volgende regels:","closeFilterBarBtn":"Filterbalk sluiten","a11yFilterBarDefButton":"Filteren...","trueLabel":"Waar","conditionIsNot":"is niet","conditionIs":"is","ruleTitleTemplate":"Regel ${0}","waiRemoveRuleButtonTemplate":"Regel ${0} verwijderen","and":"en","conditionLess":"is kleiner dan","waiColumnSelectTemplate":"Kolom voor regel ${0}","anycolumn":"een kolom","filterDefDialogTitle":"Filteren","waiConditionSelectTemplate":"Voorwaarde voor regel ${0}","waiFilterButton":"Filter activeren","statusTipTitle":"Filterbalk","statusTipRelPre":"Voldoen","anyColumnOption":"Een kolom","conditionSelectLabel":"Voorwaarde","cancelButton":"Annuleren","conditionNotStartWith":"begint niet met","conditionEndWith":"eindigt met","statusTipTitleNoFilter":"Filterbalk","columnSelectLabel":"Kolom","conditionNotContain":"bevat niet","clearButton":"Wissen","conditionLargerEqual":"ggroter dan of gelijk aan","any":"elke","relationMsgFront":"Voldoen","statusTipHeaderColumn":"Kolom","defaultItemsName":"items","conditionEqual":"gelijk aan","relationAny":"een regel","trueLabelEditable":"Geselecteerd","or":"of","filterBarMsgHasFilterTemplate":"${0} van ${1} ${2} wordt afgebeeld.","rangeTo":"tot","conditionStartsWith":"begint met","waiRelAny":"Voldoen aan een van de volgende regels:","statusTipRelPost":"regels","relationMsgTail":"","filterBarClearButton":"Filter wissen","incompleteRuleTip":"Deze regel is onvolledig.","conditionLessEqual":"kleiner dan of gelijk aan","waiFilterBarClearButton":"Filter wissen","conditionAfter":"na","statusTipHeaderCondition":"Regels","falseLabel":"Onwaar","filterButton":"Filteren","clearFilterDialogTitle":"Filter wissen","statusTipTitleHasFilter":"Filteren","addRuleButton":"Regel toevoegen","waiAddRuleButton":"Nieuwe regel toevoegen","waiClearButton":"Filter wissen","conditionNotEqual":"niet gelijk aan","conditionNotEndWith":"eindigt niet met","rangeTemplate":"van ${0} tot ${1}","removeRuleButton":"Regel verwijderen","waiFilterBarDefButton":"Tabel filteren","conditionIsEmpty":"is empty"})