<div class="dojoxGridFCBox">
	<div class="dojoxGridFCBoxSelCol" dojoAttachPoint="selColNode">
		<span class="dojoxGridFCBoxField">${_colSelectLabel}</span>
		<select dojoAttachPoint="_colSelect" dojoType="dijit.form.Select" 
			class="dojoxGridFCBoxColSelect"
			dojoAttachEvent="onChange:_onChangeColumn">
		</select>
	</div>
	<div class="dojoxGridFCBoxCondition" dojoAttachPoint="condNode">
		<span class="dojoxGridFCBoxField">${_condSelectLabel}</span>
		<select dojoAttachPoint="_condSelect" dojoType="dijit.form.Select" 
			class="dojoxGridFCBoxCondSelect"
			dojoAttachEvent="onChange:_onChangeCondition">
		</select>
		<div class="dojoxGridFCBoxCondSelectAlt" dojoAttachPoint="_condSelectAlt" style="display:none;"></div>
	</div>
	<div class="dojoxGridFCBoxValue" dojoAttachPoint="valueNode">
		<span class="dojoxGridFCBoxField">${_valueBoxLabel}</span>
	</div>
</div>
