/* Minus Button */
.mblBlueMinusButton {
	position: relative;
	width: 22px;
	height: 18px;
	border-width: 1px 1px 1px 0px;
	border-style: outset;
	border-color: #6D89C7;
	color: white;
	-webkit-border-radius: 3px;
	-moz-border-radius: 3px;
	background-color: #366EDF;
	background: -webkit-gradient(linear, left top, left bottom, from(#7A9DE9), to(#2362DD), color-stop(0.5, #366EDF), color-stop(0.5, #215FDC));
}
.mblDarkBlueMinusButton {
	position: relative;
	width: 22px;
	height: 18px;
	border-width: 1px 1px 1px 0px;
	border-style: outset;
	border-color: #6D89C7;
	color: white;
	-webkit-border-radius: 3px;
	-moz-border-radius: 3px;
	background-color: #5877A2;
	background: -webkit-gradient(linear, left top, left bottom, from(#8EA4C1), to(#4A6C9B), color-stop(0.5, #5877A2), color-stop(0.5, #476999));
}
.mblRedMinusButton {
	position: relative;
	width: 22px;
	height: 18px;
	border-width: 1px 1px 1px 0px;
	border-style: outset;
	border-color: #cc1122;
	color: white;
	-webkit-border-radius: 3px;
	-moz-border-radius: 3px;
	background-color: #C9404B;
	background: -webkit-gradient(linear, left top, left bottom, from(#D3656D), to(#BC1320), color-stop(0.5, #C9404B), color-stop(0.5, #BC1421));
}
.mblBlueMinusButton DIV, .mblDarkBlueMinusButton DIV, .mblRedMinusButton DIV {
	position: absolute;
	top: 7px;
	left: 7px;
	width: 8px;
	height: 2px;
	margin: 0px;
	font-size: 1px;
	background-color: white;
	border-top: 1px solid #4A5A71;
}

/* Plus Button */
.mblBluePlusButton {
	position: relative;
	width: 22px;
	height: 18px;
	border-width: 1px 1px 1px 0px;
	border-style: outset;
	border-color: #6D89C7;
	color: white;
	-webkit-border-radius: 3px;
	-moz-border-radius: 3px;
	background-color: #366EDF;
	background: -webkit-gradient(linear, left top, left bottom, from(#7A9DE9), to(#2362DD), color-stop(0.5, #366EDF), color-stop(0.5, #215FDC));
}
.mblDarkBluePlusButton {
	position: relative;
	width: 22px;
	height: 18px;
	border-width: 1px 1px 1px 0px;
	border-style: outset;
	border-color: #6D89C7;
	color: white;
	-webkit-border-radius: 3px;
	-moz-border-radius: 3px;
	background-color: #5877A2;
	background: -webkit-gradient(linear, left top, left bottom, from(#8EA4C1), to(#4A6C9B), color-stop(0.5, #5877A2), color-stop(0.5, #476999));
}
.mblRedPlusButton {
	position: relative;
	width: 22px;
	height: 18px;
	border-width: 1px 1px 1px 0px;
	border-style: outset;
	border-color: #6D89C7;
	color: white;
	-webkit-border-radius: 3px;
	-moz-border-radius: 3px;
	background-color: #C9404B;
	background: -webkit-gradient(linear, left top, left bottom, from(#D3656D), to(#BC1320), color-stop(0.5, #C9404B), color-stop(0.5, #BC1421));
}
.mblBluePlusButton DIV, .mblDarkBluePlusButton DIV, .mblRedPlusButton DIV {
	position: absolute;
	top: 7px;
	left: 7px;
	width: 8px;
	height: 2px;
	margin: 0px;
	font-size: 1px;
	background-color: white;
	border-top: 1px solid #4A5A71;
}
.mblBluePlusButton P, .mblDarkBluePlusButton P, .mblRedPlusButton P {
	position: absolute;
	top: 4px;
	left: 10px;
	width: 2px;
	height: 8px;
	margin: 0px;
	font-size: 1px;
	background-color: white;
}

.mblCheckOnButton {
	position: relative;
	width: 30px;
	height: 30px;
	border-width: 1px;
	border-style: outset;
	border-color: #A5A2A5;
	color: white;
	-webkit-border-radius: 3px;
	-moz-border-radius: 3px;
	background-color: #D6D3D6;
	background: -webkit-gradient(linear, left top, left bottom, from(#EFF3EF), to(#BDBEBD));
}
.mblCheckOnButton DIV {
	position: absolute;
	top: 15px;
	left: 3px;
	width: 14px;
	height: 4px;
	margin: 0px;
	font-size: 1px;
	background-color: #00CF00;
	border-top: 1px solid #4A5A71;
	-webkit-border-radius: 2px;
	-webkit-transform: rotate(50deg);
}
.mblCheckOnButton P {
	position: absolute;
	top: 11px;
	left: 9px;
	width: 20px;
	height: 4px;
	margin: 0px;
	font-size: 1px;
	background-color: #00CF00;
	border-bottom: 1px solid #4A5A71;
	-webkit-border-radius: 2px;
	-webkit-transform: rotate(-50deg);
}

.mblCheckOffButton {
	position: relative;
	width: 30px;
	height: 30px;
	border-width: 1px;
	border-style: outset;
	border-color: #A5A2A5;
	color: white;
	-webkit-border-radius: 3px;
	-moz-border-radius: 3px;
	background-color: #D6D3D6;
	background: -webkit-gradient(linear, left top, left bottom, from(#EFF3EF), to(#BDBEBD));
}
.mblCheckOffButton DIV {
	position: absolute;
	top: 11px;
	left: 9px;
	width: 20px;
	height: 4px;
	font-size: 1px;
	background-color: #BDBABD;
	border-top: 1px solid #8C8E8C;
	-webkit-border-radius: 2px;
	-webkit-transform: rotate(-50deg);
}
.mblCheckOffButton P {
	position: absolute;
	top: 15px;
	left: 3px;
	width: 14px;
	height: 4px;
	font-size: 1px;
	background-color: #BDBABD;
	border-bottom: 1px solid #8C8E8C;
	-webkit-border-radius: 2px;
	-webkit-transform: rotate(50deg);
}
